@@ -1,12 +1,12 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
 #   You may obtain a copy of the License at
-# 
+#
 #        http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 #   Unless required by applicable law or agreed to in writing, software
 #   distributed under the License is distributed on an "AS IS" BASIS,
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -17,25 +17,46 @@ setClass(Class = "BaseExpectationNormal",
          contains = "MxBaseExpectation",
          representation = representation(
            expectedCovariance = "MxOptionalCharOrNumber",
-           expectedMean = "MxOptionalCharOrNumber"))
+           expectedMean = "MxOptionalCharOrNumber",
+           expectedFullCovariance = "MxOptionalCharOrNumber",
+           expectedFullMean = "MxOptionalCharOrNumber",
+           thresholds = "MxCharOrNumber",
+           threshnames = "character",
+           discrete = "MxCharOrNumber",
+           discreteSpec = "MxOptionalMatrix",
+           .discreteCheckCount = "logical",
+           selectionPlan = "MxOptionalDataFrame",
+           selectionVector = "MxCharOrNumber"))
 
-setMethod("qualifyNames", signature("BaseExpectationNormal"), 
+setMethod("qualifyNames", signature("BaseExpectationNormal"),
           function(.Object, modelname, namespace) {
-            for (sl in c(paste0('expected', c('Covariance','Mean')))) {
+            for (sl in c(paste0('expected', c('Covariance','Mean')),
+                       'thresholds', 'discrete', 'selectionVector')) {
+              if (!.hasSlot(.Object, sl) || is.null(slot(.Object, sl))) next
               slot(.Object, sl) <- imxConvertIdentifier(slot(.Object, sl), modelname, namespace)
             }
             .Object
           })
 
+setMethod("genericExpDependencies", signature("BaseExpectationNormal"),
+          function(.Object, dependencies) {
+            sources <- c(.Object$discrete, .Object@thresholds, .Object$selectionVector)
+            sources <- sources[!is.na(sources)]
+            dependencies <- imxAddDependency(sources, .Object@name, dependencies)
+            return(dependencies)
+          })
+
 setMethod("genericExpRename", signature("BaseExpectationNormal"),
           function(.Object, oldname, newname) {
-            for (sl in c(paste0('expected', c('Covariance','Mean')))) {
+            for (sl in c(paste0('expected', c('Covariance','Mean')),
+                         'thresholds', 'discrete', 'selectionVector')) {
+              if (!.hasSlot(.Object, sl) || is.null(slot(.Object, sl))) next
               slot(.Object,sl) <- renameReference(slot(.Object,sl), oldname, newname)
             }
             .Object
           })
 
-setMethod("genericExpFunConvert", signature("BaseExpectationNormal"), 
+setMethod("genericExpFunConvert", signature("BaseExpectationNormal"),
           function(.Object, flatModel, model, labelsData, dependencies) {
             for (sl in c(paste0('expected', c('Covariance','Mean')))) {
               matName <- slot(.Object,sl)
@@ -51,18 +72,140 @@ setMethod("genericExpFunConvert", signature("BaseExpectationNormal"),
                 stop(msg, call.=FALSE)
               }
             }
+
+            tname <- .Object$thresholds
+            dname <- .Object$discrete
+            if (!is.na(dname)) {
+              dmat <- flatModel[[dname]]
+              if (length(colnames(dmat)) == 0) {
+                stop(paste(dname, "must have column names"))
+              }
+              ds <- .Object$discreteSpec
+              if (is.null(ds)) stop("discrete parameters but no discreteSpec")
+              if (length(colnames(ds)) != length(colnames(dmat))) {
+                stop(paste("discreteSpec must have the same number of columns",
+                           "as", dname))
+              }
+              if (any(colnames(ds) != colnames(dmat))) {
+                stop(paste("discreteSpec and", dname, "must have the same column names"))
+              }
+            }
+            if (!is.na(tname) && !is.na(dname)) {
+              dup <- intersect(colnames(flatModel[[tname]]), colnames(flatModel[[dname]]))
+              if (length(dup)) {
+                msg <- paste('Manifest', omxQuotes(dup), 'cannot be modelled as',
+                             'a threshold and as a discrete variable at the',
+                             'same time')
+                stop(msg, call.=FALSE)
+              }
+            }
+
+            if (.hasSlot(.Object, 'selectionVector')) {
+              svname <- .Object$selectionVector
+              if (!is.na(svname)) {
+                sv <- flatModel[[svname]]
+                if (ncol(sv) != 1) stop("selectionVector must be a column vector")
+                svSpec <- .Object@selectionPlan
+                if (nrow(sv) != nrow(svSpec)) {
+                  msg <- paste('selectionPlan must have the same number of rows',
+                               'as the selectionVector')
+                  stop(msg, call.=FALSE)
+                }
+              }
+            }
+
             .Object
           })
 
 setMethod("genericNameToNumber", signature("BaseExpectationNormal"),
 	  function(.Object, flatModel, model) {
 		  name <- .Object@name
-      for (sl in c(paste0('expected', c('Covariance','Mean')))) {
+      for (sl in c(paste0('expected', c('Covariance','Mean')),
+                   'data', 'thresholds', 'discrete', 'selectionVector')) {
+        if (!.hasSlot(.Object, sl)) next
         slot(.Object,sl) <- imxLocateIndex(flatModel, slot(.Object,sl), name)
       }
 		  .Object
 	  })
 
+setMethod("genericGetExpected", signature("BaseExpectationNormal"),
+	function(.Object, model, what, defvar.row=1, subname=model@name) {
+		ret <- list()
+		if ('thresholds' %in% what) {
+			thrname <- .Object@thresholds
+      thr <- matrix( , 0 , 0)
+			if(!single.na(thrname)){
+				thrname <- .modifyDottedName(subname, thrname, sep=".")
+				thr <- mxEvalByName(thrname, model, compute=TRUE, defvar.row=defvar.row)
+        tnames <- .Object@threshnames
+        if(!single.na(tnames)){
+          colnames(thr) <- tnames
+        }
+			}
+      disname <- .Object@discrete
+      if (!single.na(disname)) {
+        npart <- mxGetExpected(model, c('means','covariance'),
+                               defvar.row=defvar.row, subname=subname)
+        means <- npart[['means']]
+        if (!is.null(means) && ncol(means)==1 && colnames(means)=='one')
+          means <- t(means)
+        cov <- npart[['covariance']]
+        ds <- .Object@discreteSpec
+				disname <- .modifyDottedName(subname, disname, sep=".")
+				dis <- mxEvalByName(disname, model, compute=TRUE, defvar.row=defvar.row)
+        if (length(colnames(ds)) != length(colnames(dis))) {
+          stop(paste("discreteSpec must have the same number of columns",
+                     "as", disname))
+        }
+        if (any(colnames(ds) != colnames(dis))) {
+          stop(paste("discreteSpec and", disname, "must have the same column names"))
+        }
+
+        # discreteSpec:
+        #   number of outcomes
+        #   1 poisson ; 2 negative binomial prob ; 3 neg binomial mu
+        # discrete 3 rows:
+        #   zero inflation
+        #   poisson: lambda; nbinom: size, prob/mu
+        thList <- vector("list", ncol(ds))
+        names(thList) <- colnames(ds)
+        for (cx in colnames(ds)) {
+          if (is.na(ds[1,cx])) {
+            stop(paste("discreteSpec[1,",cx,"], the maximum observed count,",
+                       "cannot be NA when calcuating the expected distribution"))
+          }
+          outcome <- 1L:as.integer(ds[1,cx]) - 1L
+          zif <- min(max(dis[1,cx],0),1)
+          if (ds[2,cx] == 1) {
+            pr <- ppois(outcome, dis[2,cx])
+          } else if (ds[2,cx] == 2) {
+            pr <- pnbinom(outcome, dis[2,cx], prob=dis[3,cx])
+          } else if (ds[2,cx] == 3) {
+            pr <- pnbinom(outcome, dis[2,cx], mu=dis[3,cx])
+          } else { stop(paste("Unknown discrete distribution code", ds[2,cx],
+                              "in column", cx)) }
+          m1 <- 0
+          if (!is.null(means)) m1 <- means[1,cx]
+          thList[[cx]] <- p2z(zif + (1-zif) * pr) * sqrt(cov[cx,cx]) + m1
+        }
+        thBlock <- mxSimplify2Array(thList)
+        comb <- matrix(NA,
+                       nrow=max(nrow(thr), nrow(thBlock)),
+                       ncol=(ncol(thr) + ncol(thBlock)))
+        if (length(thr)) {
+          comb[1:nrow(thr),1:ncol(thr)] <- thr
+          comb[1:nrow(thBlock),ncol(thr) + 1:ncol(thBlock)] <- thBlock
+          colnames(comb) <- c(colnames(thr), colnames(thBlock))
+          thr <- comb
+        } else {
+          thr <- thBlock
+        }
+      }
+			ret[['thresholds']] <- thr
+      }
+    ret
+    })
+
 constrainCorData <- function(ex, size, model, flatModel) {
   path <- unlist(strsplit(ex@name, imxSeparatorChar, fixed = TRUE))
   if (path[1] == model@name) {
@@ -106,17 +249,16 @@ setClass(Class = "MxExpectationNormal",
 	representation = representation(
 		covariance = "MxCharOrNumber",
 		means = "MxCharOrNumber",
-		thresholds = "MxCharOrNumber",
 		dims = "character",
-		threshnames = "character",
 		ExpCov = "matrix",
 		ExpMean = "matrix",
-	        numStats = "numeric"),
+    numStats = "numeric"),
 	contains = "BaseExpectationNormal")
 
 setMethod("initialize", "MxExpectationNormal",
-	function(.Object, covariance, means, dims, thresholds, threshnames,
-		data = as.integer(NA), name = 'expectation') {
+	function(.Object, covariance, means, dims, thresholds, threshnames, discrete,
+           discreteSpec, selectionVector, selectionPlan,
+           data = as.integer(NA), name = 'expectation') {
 		.Object@name <- name
 		.Object@covariance <- covariance
 		.Object@means <- means
@@ -124,29 +266,33 @@ setMethod("initialize", "MxExpectationNormal",
 		.Object@thresholds <- thresholds
 		.Object@dims <- dims
 		.Object@threshnames <- threshnames
+		.Object@discrete <- discrete
+		.Object@discreteSpec <- discreteSpec
+    .Object@.discreteCheckCount <- TRUE
+    .Object@selectionVector <- selectionVector
+    .Object@selectionPlan <- selectionPlan
 		.Object@ExpCov <- matrix()
 		.Object@ExpMean <- matrix()
 		return(.Object)
 	}
 )
 
-setMethod("qualifyNames", signature("MxExpectationNormal"), 
+setMethod("qualifyNames", signature("MxExpectationNormal"),
 	  function(.Object, modelname, namespace) {
+      .Object <- callNextMethod()
 		.Object@name <- imxIdentifier(modelname, .Object@name)
-		.Object@covariance <- imxConvertIdentifier(.Object@covariance, 
-			modelname, namespace)
-		.Object@means <- imxConvertIdentifier(.Object@means, 
-			modelname, namespace)
-		.Object@data <- imxConvertIdentifier(.Object@data, 
-			modelname, namespace)
-		.Object@thresholds <- sapply(.Object@thresholds,
-			imxConvertIdentifier, modelname, namespace)
+		for (s in c("covariance", "means", "data")) {
+			if (is.null(slot(.Object, s))) next
+			slot(.Object, s) <-
+			  imxConvertIdentifier(slot(.Object, s), modelname, namespace)
+		}
 		return(.Object)
 })
 
 setMethod("genericExpDependencies", signature("MxExpectationNormal"),
 	function(.Object, dependencies) {
-	sources <- c(.Object@covariance, .Object@means, .Object@thresholds)
+    dependencies <- callNextMethod()
+    sources <- c(.Object@covariance, .Object@means)
 	sources <- sources[!is.na(sources)]
 	dependencies <- imxAddDependency(sources, .Object@name, dependencies)
 	return(dependencies)
@@ -162,10 +308,11 @@ setMethod("genericExpConvertEntities", "MxExpectationNormal",
 
 setMethod("genericExpRename", signature("MxExpectationNormal"),
 	function(.Object, oldname, newname) {
-		.Object@means <- renameReference(.Object@means, oldname, newname)
-		.Object@covariance <- renameReference(.Object@covariance, oldname, newname)
-		.Object@data <- renameReference(.Object@data, oldname, newname)
-		.Object@thresholds <- sapply(.Object@thresholds, renameReference, oldname, newname)		
+    .Object <- callNextMethod()
+		for (s in c("covariance", "means", "data")) {
+			if (is.null(slot(.Object, s))) next
+			slot(.Object, s) <- renameReference(slot(.Object, s), oldname, newname)
+    }
 		return(.Object)
 })
 
@@ -181,7 +328,7 @@ setMethod("genericExpRename", signature("MxExpectationNormal"),
 
 setMethod("genericGetExpected", signature("MxExpectationNormal"),
 	function(.Object, model, what, defvar.row=1, subname=model@name) {
-		ret <- list()
+		ret <- callNextMethod()
 		if (any(c('covariance','covariances') %in% what)) {
 			covname <- .modifyDottedName(subname, .Object@covariance)
 			cov <- mxEvalByName(covname, model, compute=TRUE, defvar.row=defvar.row, .extraBack=7L)
@@ -204,18 +351,6 @@ setMethod("genericGetExpected", signature("MxExpectationNormal"),
 			} else {mean <- matrix( , 0, 0)}
 			ret[['means']] <- mean
 		}
-		if ('thresholds' %in% what) {
-			thrname <- .Object@thresholds
-			if(!single.na(thrname)){
-				thrname <- .modifyDottedName(subname, thrname, sep=".")
-				thr <- mxEvalByName(thrname, model, compute=TRUE, defvar.row=defvar.row)
-				tnames <- .Object$threshnames
-				if(!single.na(tnames)){
-					colnames(thr) <- tnames
-				}
-			} else {thr <- matrix( , 0 , 0)}
-			ret[['thresholds']] <- thr
-		}
 		ret
 	})
 
@@ -340,6 +475,7 @@ setMethod("genericGetExpectedStandVector", signature("BaseExpectationNormal"),
 
 imxGetExpectationComponent <- function(model, component, defvar.row=1, subname=model$name)
 {
+  warnModelCreatedByOldVersion(model)
   if (!is(model[[subname]], "MxModel")) {
     stop(paste("Submodel", subname, "in model", model$name, "is not found"))
   }
@@ -377,7 +513,7 @@ sse <- function(x){sum(x^2)}
 #'
 #' @details
 #' The Jacobian is estimated by the central finite difference.
-#' 
+#'
 #' If the \code{standardize} argument is TRUE, then the Jacobian is for the standardized model.
 #' For Normal expectations the standardized manifest model has the covariances returned as correlations, the variances returned as ones, the means returned as zeros, and the thresholds are returned as z-scores.
 #' For the thresholds the z-scores are computed by using the model-implied means and variances.
@@ -403,11 +539,12 @@ omxManifestModelByParameterJacobian <- function(model, defvar.row=1, standardize
 		jac <- numDeriv::jacobian(func=.mat2param, x=theParams, method.args=list(r=2), model=model, defvar.row=defvar.row, standardize=standardize)
 		dimnames(jac) <- list(names(mxGetExpected(model, 'vector')), names(theParams))
 	}
-	
+
 	return(jac)
 }
 
 mxCheckIdentification <- function(model, details=TRUE){
+  warnModelCreatedByOldVersion(model)
 	notAllowedFits <- c("MxFitFunctionAlgebra", "MxFitFunctionRow", "MxFitFunctionR")
 	if( class(model$fitfunction) %in% notAllowedFits ){
 		msg <- paste("Identification check is not possible for models with", omxQuotes(notAllowedFits), 'fit functions.\n', "If you have a multigroup model, use mxFitFunctionMultigroup.")
@@ -465,15 +602,15 @@ mxCheckIdentification <- function(model, details=TRUE){
 
 setGeneric("genericGenerateData",
 	function(.Object, model, nrows, subname, empirical, returnModel, use.miss,
-		   .backend, nrowsProportion) {
+		   .backend, nrowsProportion, silent) {
 	return(standardGeneric("genericGenerateData"))
 })
 
 setMethod("genericGenerateData", signature("MxExpectationNormal"),
 	  function(.Object, model, nrows, subname, empirical, returnModel, use.miss,
-		   .backend, nrowsProportion) {
+		   .backend, nrowsProportion, silent) {
 	    return(generateNormalData(model, nrows, subname, empirical, returnModel,
-				      use.miss, .backend, nrowsProportion))
+				      use.miss, .backend, nrowsProportion, silent))
 	  })
 
 .rmvnorm <- function(nrow, mean, sigma, empirical) {
@@ -502,13 +639,16 @@ calcNumRows <- function(nrows, nrowsProportion, origRows, subname) {
 }
 
 generateNormalData <- function(model, nrows, subname, empirical, returnModel, use.miss,
-			       .backend, nrowsProportion) {
+			       .backend, nrowsProportion, silent) {
   origData <- findDataForSubmodel(model, subname)
   origRows <- if (!is.null(origData)) { nrowMxData(origData) } else { NULL }
   nrows <- calcNumRows(nrows, nrowsProportion, origRows, subname)
 
 	# Check for definition variables
 	if(imxHasDefinitionVariable(model[[subname]])){
+    if (is.null(origData)) {
+      stop("Definition variable(s) found, but no data is available")
+    }
 		if (origData$type != 'raw') {
 			stop(paste("Definition variable(s) found, but original data is type",
 				omxQuotes(origData$type)))
@@ -522,12 +662,17 @@ generateNormalData <- function(model, nrows, subname, empirical, returnModel, us
 		data <- matrix(NA, nrow=nrows, ncol=ncol(theCov))
 		colnames(data) <- colnames(theCov)
 		data <- as.data.frame(data)
+    prevProgressLen <- 0L
 		for(i in 1:nrows){
 			theMeans <- imxGetExpectationComponent(model, "means", defvar.row=i, subname=subname)
 			theCov <- imxGetExpectationComponent(model, "covariance", defvar.row=i, subname=subname)
 			theThresh <- imxGetExpectationComponent(model, "thresholds", defvar.row=i, subname=subname)
 			data[i,] <- .rmvnorm(1, theMeans, theCov, empirical)
+      info <- paste0(subname,' ', i, '/', nrows)
+      if (!silent) imxReportProgress(info, prevProgressLen)
+      prevProgressLen <- nchar(info)
 		}
+    if (!silent) imxReportProgress('', prevProgressLen)
 		data <- ordinalizeDataHelper(data, theThresh, origData)
 		if (!is.null(origData)) {
 			for (dcol in setdiff(colnames(origData), colnames(data))) {
@@ -559,10 +704,11 @@ generateNormalData <- function(model, nrows, subname, empirical, returnModel, us
 		data <- as.data.frame(data)
 		data <- ordinalizeDataHelper(data, theThresh, origData)
 	}
-        if (use.miss && !is.null(origData) && all(colnames(data) %in% colnames(origData))) {
-	  del <- is.na(origData[,colnames(data),drop=FALSE])
+  if (use.miss && !is.null(origData) && origData$type == 'raw' &&
+        all(colnames(data) %in% colnames(origData$observed))) {
+	  del <- is.na(origData$observed[,colnames(data),drop=FALSE])
 	  if (nrows != origRows) {
-	    del    <- del[sample.int(nrow(origData), nrows, replace=TRUE),,drop=FALSE]
+	    del    <- del[sample.int(origRows, nrows, replace=TRUE),,drop=FALSE]
 	  }
 	  data[del] <- NA
 	}
@@ -577,6 +723,7 @@ ordinalizeDataHelper <- function(data, thresh, origData=NULL) {
 	if (!is.null(origData) && is(origData, "MxData")) origData <- origData$observed
 	if( prod(dim(thresh)) != 0){
 		ordvars <- colnames(thresh)
+    if (length(ordvars) != ncol(thresh)) stop("thresholds missing column names")
 		for(avar in ordvars){
 			delthr <- thresh[,avar]
 			usethr <- 1:sum(!is.na(delthr))  # assumes NA indicates unused threshold
@@ -727,7 +874,7 @@ extractObservedData <- function(model) {
 }
 
 mxGenerateData <- function(model, nrows=NULL, returnModel=FALSE, use.miss = TRUE,
-			   ..., .backend=TRUE, subname=NULL, empirical=FALSE, nrowsProportion=NULL) {
+			   ..., .backend=TRUE, subname=NULL, empirical=FALSE, nrowsProportion=NULL, silent=FALSE) {
 	prohibitDotdotdot(list(...))
 	if (!is.null(nrows) && !is.null(nrowsProportion)) {
 	  stop("You cannot specify both nrows and nrowsProportion")
@@ -754,6 +901,7 @@ mxGenerateData <- function(model, nrows=NULL, returnModel=FALSE, use.miss = TRUE
 		if(is.null(nrows)) nrows <- nrow(model)
 		return(mxGenerateData(fake, nrows, returnModel, empirical=empirical))
 	}
+  warnModelCreatedByOldVersion(model)
 	if(is.null(subname)) subname <- model$name
 	if (is.null(model[[subname]]$expectation) && is(model[[subname]]$fitfunction, 'MxFitFunctionMultigroup')) {
 		todo <- sub(".fitfunction", "", model[[subname]]$fitfunction$groups, fixed=TRUE)
@@ -769,7 +917,7 @@ mxGenerateData <- function(model, nrows=NULL, returnModel=FALSE, use.miss = TRUE
 		}
 	}
 	genericGenerateData(model[[subname]]$expectation, model, nrows, subname, empirical,
-			    returnModel, use.miss, .backend, nrowsProportion)
+			    returnModel, use.miss, .backend, nrowsProportion, silent)
 }
 
 verifyExpectedObservedNames <- function(data, covName, flatModel, modelname, objectiveName) {
@@ -786,7 +934,7 @@ verifyExpectedObservedNames <- function(data, covName, flatModel, modelname, obj
 			") and the observed covariance matrix (", omxQuotes(rownames(data)),
 			") in the ", objectiveName, " expectation function in model ",
 			omxQuotes(modelname), " are not identical.")
-		stop(msg, call. = FALSE)		
+		stop(msg, call. = FALSE)
 	}
 }
 
@@ -800,12 +948,13 @@ verifyMeans <- function(meansName, mxDataObject, flatModel, modelname) {
 				"for the observed means.")
 			stop(msg, call. = FALSE)
 		}
-		meanDimnames <- dimnames(means)		
+		meanDimnames <- dimnames(means)
 	}
 }
 
-setMethod("genericExpFunConvert", "MxExpectationNormal", 
+setMethod("genericExpFunConvert", "MxExpectationNormal",
 	function(.Object, flatModel, model, labelsData, dependencies) {
+    .Object <- callNextMethod()
 		modelname <- imxReverseIdentifier(model, .Object@name)[[1]]
 		name <- .Object@name
 		if(is.na(.Object@data)) {
@@ -817,9 +966,7 @@ setMethod("genericExpFunConvert", "MxExpectationNormal",
 
 		mxDataObject <- flatModel@datasets[[.Object@data]]
 		dataName <- .Object@data
-		.Object@data <- imxLocateIndex(flatModel, .Object@data, name)
 		threshName <- .Object@thresholds
-		.Object@thresholds <- imxLocateIndex(flatModel, threshName, name)
 		covName <- .Object@covariance
 		covariance <- flatModel[[covName]]
 		.Object@covariance <- imxLocateIndex(flatModel, .Object@covariance, name)
@@ -842,9 +989,32 @@ setMethod("genericExpFunConvert", "MxExpectationNormal",
 		if (single.na(.Object@dims)) {
 			.Object@dims <- covNames
 		}
+
+    if (.hasSlot(.Object, 'selectionVector')) {
+      .Object@selectionPlan <- prepSelectionPlan(.Object@selectionPlan, covNames)
+    }
+
 		return(.Object)
 })
 
+prepSelectionPlan <- function(selPlan, covNames) {
+  if (is.null(selPlan)) return(NULL)
+
+  ind <- match(selPlan$from, covNames)
+  if (any(is.na(ind))) {
+    stop(paste("Can't find selectionPlan from variables",
+               omxQuotes(selPlan$from[is.na(ind)])))
+  }
+  selPlan$from <- ind - 1L
+  ind <- match(selPlan$to, covNames)
+  if (any(is.na(ind))) {
+    stop(paste("Can't find selectionPlan to variables",
+               omxQuotes(selPlan$to[is.na(ind)])))
+  }
+  selPlan$to <- ind - 1L
+  selPlan
+}
+
 verifyMvnNames <- function(covName, meansName, type, flatModel, modelname, expectationName) {
 	if (is.na(meansName)) {
 		means <- NA
@@ -858,7 +1028,7 @@ verifyMvnNames <- function(covName, meansName, type, flatModel, modelname, expec
 			msg <- paste("The",type,"covariance matrix associated",
 				     "with", expectationName, "in model",
 				     omxQuotes(modelname), "does not contain dimnames.")
-			stop(msg, call. = FALSE)	
+			stop(msg, call. = FALSE)
 		}
 		covRows <- covDimnames[[1]]
 		covCols <- covDimnames[[2]]
@@ -877,7 +1047,7 @@ verifyMvnNames <- function(covName, meansName, type, flatModel, modelname, expec
 			msg <- paste("The",type,"means matrix associated",
 				"with", expectationName, "in model",
 				omxQuotes(modelname), "does not contain dimnames.")
-			stop(msg, call.=FALSE)	
+			stop(msg, call.=FALSE)
 	}
 	meanRows <- meanDimnames[[1]]
 	meanCols <- meanDimnames[[2]]
@@ -894,6 +1064,7 @@ verifyMvnNames <- function(covName, meansName, type, flatModel, modelname, expec
 }
 
 verifyObservedNames <- function(data, means, type, flatModel, modelname, expectationName) {
+  if (is.null(data)) return()
 	dataNames <- dimnames(data)
 	if(is.null(dataNames)) {
 		msg <- paste("The observed data associated with the",
@@ -903,7 +1074,7 @@ verifyObservedNames <- function(data, means, type, flatModel, modelname, expecta
 	}
 	if (type == "cov" || type == "cor") {
 		if (length(dataNames) < 2 ||
-			is.null(dataNames[[1]]) || is.null(dataNames[[2]]) || 
+			is.null(dataNames[[1]]) || is.null(dataNames[[2]]) ||
 			!identical(dataNames[[1]], dataNames[[2]])) {
 				msg <- paste("The dataset associated with the", expectationName,
 					"expectation function in model", omxQuotes(modelname),
@@ -911,9 +1082,9 @@ verifyObservedNames <- function(data, means, type, flatModel, modelname, expecta
 			stop(msg, call. = FALSE)
 		}
 		if (!single.na(means) && is.null(dimnames(means))) {
-			msg <- paste("In model", omxQuotes(modelname), 
+			msg <- paste("In model", omxQuotes(modelname),
 				", the observed means vector does not contain column names.",
-				"Use the name() function to assign names to the means vector.")
+				"Use the names() function to assign names to the means vector.")
 			stop(msg, call. = FALSE)
 		}
 		if (!single.na(means) && !identical(dataNames[[1]], dimnames(means)[[2]])) {
@@ -933,7 +1104,16 @@ verifyObservedNames <- function(data, means, type, flatModel, modelname, expecta
 generateDataColumns <- function(flatModel, covNames, dataName) {
 	retval <- c()
 	if (length(covNames) == 0) return(retval)
-	dataColumnNames <- colnames(flatModel@datasets[[dataName]]@observed)
+  mxd <- flatModel@datasets[[dataName]]
+  if (!is.null(mxd@observed)) {
+    dataColumnNames <- colnames(mxd@observed)
+  } else if (!is.null(mxd$observedStats[['cov']])) {
+    dataColumnNames <- colnames(mxd$observedStats[['cov']])
+  } else {
+    msg <- paste(omxQuotes(dataName), "is type='raw' but",
+                 "raw observed data seems to be unavailable")
+    stop(msg, call.=FALSE)
+  }
 	retval <- match(covNames, dataColumnNames)
 	if (any(is.na(retval))) {
 		msg <- paste("The column name(s)", omxQuotes(covNames[is.na(retval)]),
@@ -955,20 +1135,20 @@ updateThresholdDimnames <- function(flatExpectation, flatModel, labelsData) {
 	thresholds <- flatModel[[threshName]]
 	if (is.null(thresholds)) {
 		modelname <- getModelName(flatExpectation)
-		stop(paste("Unknown thresholds name", 
+		stop(paste("Unknown thresholds name",
 			omxQuotes(simplifyName(threshName, modelname)),
 			"detected in the expectation function",
 			"of model", omxQuotes(modelname)), call. = FALSE)
 	}
 	dims <- flatExpectation@threshnames
-	if (!is.null(colnames(thresholds)) && !single.na(dims) && 
+	if (!is.null(colnames(thresholds)) && !single.na(dims) &&
 		!identical(colnames(thresholds), dims)) {
 		modelname <- getModelName(flatExpectation)
 		msg <- paste("The thresholds matrix associated",
-		"with the expectation function in model", 
+		"with the expectation function in model",
 		omxQuotes(modelname), "contains column names and",
 		"the expectation function has specified non-identical threshnames.")
-		stop(msg, call.=FALSE)      
+		stop(msg, call.=FALSE)
 	}
 	if (is.null(colnames(thresholds)) && !single.na(dims)) {
 		if (!flatModel@unsafe) {
@@ -977,12 +1157,12 @@ updateThresholdDimnames <- function(flatExpectation, flatModel, labelsData) {
 			if (ncol(threshMatrix) != length(dims)) {
 				modelname <- getModelName(flatExpectation)
 				msg <- paste("The thresholds matrix associated",
-					"with the expectation function in model", 
+					"with the expectation function in model",
 					omxQuotes(modelname), "is not of the same length as the 'threshnames'",
 					"argument provided by the expectation function. The 'threshnames' argument is",
 					"of length", length(dims), "and the expected covariance matrix",
 					"has", ncol(threshMatrix), "columns.")
-				stop(msg, call.=FALSE)      
+				stop(msg, call.=FALSE)
 			}
 		}
 		dimnames(flatModel[[threshName]]) <- list(NULL, dims)
@@ -1003,29 +1183,29 @@ updateExpectationDimnames <- function(flatExpectation, flatModel,
 	covariance <- flatModel[[covName]]
 	if (is.null(covariance)) {
 		modelname <- getModelName(flatExpectation)
-		stop(paste("Unknown expected covariance name", 
+		stop(paste("Unknown expected covariance name",
 			omxQuotes(simplifyName(covName, modelname)),
 			"detected in the expectation function",
 			"of model", omxQuotes(modelname)), call. = FALSE)
 	}
 	if (is.null(means)) {
 		modelname <- getModelName(flatExpectation)
-		stop(paste("Unknown expected means name", 
+		stop(paste("Unknown expected means name",
 			omxQuotes(simplifyName(meansName, modelname)),
 			"detected in the expectation function",
 			"of model", omxQuotes(modelname)), call. = FALSE)
 	}
 	dims <- flatExpectation@dims
-	if (!is.null(dimnames(covariance)) && !single.na(dims) && 
+	if (!is.null(dimnames(covariance)) && !single.na(dims) &&
 		!identical(dimnames(covariance), list(dims, dims))) {
 		modelname <- getModelName(flatExpectation)
 		msg <- paste("The expected covariance matrix associated",
-			"with the expectation function in model", 
-			omxQuotes(modelname), "contains dimnames: ", 
+			"with the expectation function in model",
+			omxQuotes(modelname), "contains dimnames: ",
             paste(toString(dimnames(covariance)), ".", sep = ""),
-			"The expectation function has specified dimnames:", 
+			"The expectation function has specified dimnames:",
 			paste(toString(dims), ".", sep =""))
-		stop(msg, call.=FALSE)		
+		stop(msg, call.=FALSE)
 	}
 	if (is.null(dimnames(covariance)) && !single.na(dims)) {
 		if (!unsafe) {
@@ -1034,19 +1214,19 @@ updateExpectationDimnames <- function(flatExpectation, flatModel,
 			if (nrow(covMatrix) != ncol(covMatrix)) {
 				modelname <- getModelName(flatExpectation)
 				msg <- paste("The expected covariance matrix associated",
-					"with the expectation function in model", 
+					"with the expectation function in model",
 					omxQuotes(modelname), "is not a square matrix.")
-				stop(msg, call.=FALSE)		
+				stop(msg, call.=FALSE)
 			}
 			if (nrow(covMatrix) != length(dims)) {
 				modelname <- getModelName(flatExpectation)
 				msg <- paste("The expected covariance matrix associated",
-					"with the expectation function in model", 
+					"with the expectation function in model",
 					omxQuotes(modelname), "is not of the same length as the 'dimnames'",
 					"argument provided by the expectation function. The 'dimnames' argument is",
 					"of length", length(dims), "and the expected covariance matrix",
 					"has", nrow(covMatrix), "rows and columns.")
-				stop(msg, call.=FALSE)		
+				stop(msg, call.=FALSE)
 			}
 		}
 		dimnames(flatModel[[covName]]) <- list(dims, dims)
@@ -1060,12 +1240,12 @@ updateExpectationDimnames <- function(flatExpectation, flatModel,
 		!identical(dimnames(means), list(NULL, dims))) {
 		modelname <- getModelName(flatExpectation)
 		msg <- paste("The expected means matrix associated",
-			"with the expectation function in model", 
-			omxQuotes(modelname), "contains dimnames: ", 
+			"with the expectation function in model",
+			omxQuotes(modelname), "contains dimnames: ",
             paste(toString(dimnames(means)), ".", sep = ""),
-			"The expectation function has specified dimnames:", 
+			"The expectation function has specified dimnames:",
 			paste(toString(dims), ".", sep =""))
-		stop(msg, call.=FALSE)	
+		stop(msg, call.=FALSE)
 	}
 	if (is.null(dimnames(means)) && !single.na(dims)) {
 		if (!unsafe) {
@@ -1074,16 +1254,16 @@ updateExpectationDimnames <- function(flatExpectation, flatModel,
 			if (nrow(meansMatrix) != 1) {
 				modelname <- getModelName(flatExpectation)
 				msg <- paste("The expected means vector associated",
-					"with the expectation function in model", 
+					"with the expectation function in model",
 					omxQuotes(modelname), "is not a 1 x n matrix.",
-					"It has dimensions", nrow(meansMatrix), "x", 
+					"It has dimensions", nrow(meansMatrix), "x",
 					paste(ncol(meansMatrix), '.', sep=''))
-				stop(msg, call.=FALSE)		
+				stop(msg, call.=FALSE)
 			}
 			if (ncol(meansMatrix) != length(dims)) {
 				modelname <- getModelName(flatExpectation)
 				msg <- paste("The expected means vector associated",
-					"with the expectation function in model", 
+					"with the expectation function in model",
 					omxQuotes(modelname), "is not of the same length as the 'dimnames'",
 					"argument provided by the expectation function. The 'dimnames' argument is",
 					"of length", length(dims), "and the expected means vector",
@@ -1116,8 +1296,11 @@ checkThreshnames <- function(threshnames) {
 	return(threshnames)
 }
 
-mxExpectationNormal <- function(covariance, means = NA, 
-	dimnames = NA, thresholds = NA, threshnames = dimnames) {
+mxExpectationNormal <-
+  function(covariance, means = NA, dimnames = NA, thresholds = NA,
+           threshnames = dimnames, ..., discrete = as.character(NA),
+           discreteSpec = NULL) {
+	prohibitDotdotdot(list(...))
 	if (missing(covariance) || typeof(covariance) != "character") {
 		stop("'covariance' argument is not a string (the name of the expected covariance matrix)")
 	}
@@ -1140,7 +1323,10 @@ mxExpectationNormal <- function(covariance, means = NA,
 		stop("NA values are not allowed for 'dimnames' vector")
 	}
 	threshnames <- checkThreshnames(threshnames)
-	return(new("MxExpectationNormal", covariance, means, dimnames, thresholds, threshnames))
+  selectionVector <- NA_character_
+  selectionPlan <- NULL
+	return(new("MxExpectationNormal", covariance, means, dimnames, thresholds, threshnames,
+             discrete, discreteSpec, selectionVector, selectionPlan))
 }
 
 displayMxExpectationNormal <- function(expectation) {
@@ -1157,6 +1343,16 @@ displayMxExpectationNormal <- function(expectation) {
 	} else {
 		cat("$thresholds :", omxQuotes(expectation@thresholds), '\n')
 	}
+	if (is.null(expectation@discreteSpec)) {
+		cat("$discreteSpec : NULL \n")
+	} else {
+		cat("$discreteSpec :", expectation@discreteSpec, '\n')
+	}
+	if (single.na(expectation@discrete)) {
+		cat("$discrete : NA \n")
+	} else {
+		cat("$discrete :", omxQuotes(expectation@discrete), '\n')
+	}
 	if (single.na(expectation@threshnames)) {
 		cat("$threshnames : NA \n")
 	} else {
@@ -1166,10 +1362,10 @@ displayMxExpectationNormal <- function(expectation) {
 }
 
 
-setMethod("print", "MxExpectationNormal", function(x,...) { 
-	displayMxExpectationNormal(x) 
+setMethod("print", "MxExpectationNormal", function(x,...) {
+	displayMxExpectationNormal(x)
 })
 
-setMethod("show", "MxExpectationNormal", function(object) { 
-	displayMxExpectationNormal(object) 
+setMethod("show", "MxExpectationNormal", function(object) {
+	displayMxExpectationNormal(object)
 })
