@@ -1,12 +1,12 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
 #   You may obtain a copy of the License at
-# 
+#
 #        http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 #   Unless required by applicable law or agreed to in writing, software
 #   distributed under the License is distributed on an "AS IS" BASIS,
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -21,7 +21,6 @@ setClass(Class = "MxExpectationRAM",
 		M = "MxCharOrNumber",
 		thresholds = "MxCharOrNumber",
 		dims = "character",
-		threshnames = "character",
 		usePPML = "logical",
 		ppmlData = "MxData",
 		UnfilteredExpCov = "matrix",
@@ -42,7 +41,8 @@ setClass(Class = "MxExpectationRAM",
 
 setMethod("initialize", "MxExpectationRAM",
 	function(.Object, A, S, F, M, dims, thresholds, threshnames,
-           between, verbose, useSparse, expectedCovariance, expectedMean,
+           between, verbose, useSparse, expectedCovariance, expectedMean, discrete,
+           selectionVector, expectedFullCovariance, expectedFullMean,
            data = as.integer(NA), name = 'expectation') {
 		.Object@name <- name
 		.Object@A <- A
@@ -52,6 +52,9 @@ setMethod("initialize", "MxExpectationRAM",
 		.Object@data <- data
 		.Object@dims <- dims
 		.Object@thresholds <- thresholds
+    .Object@discrete <- discrete
+    .Object@.discreteCheckCount <- TRUE
+    .Object@selectionVector <- selectionVector
 		.Object@threshnames <- threshnames
 		.Object@usePPML <- FALSE
 		.Object@UnfilteredExpCov <- matrix()
@@ -67,48 +70,50 @@ setMethod("initialize", "MxExpectationRAM",
     .Object@.useSparse <- useSparse
     .Object@expectedCovariance <- expectedCovariance
     .Object@expectedMean <- expectedMean
+    .Object@expectedFullCovariance <- expectedFullCovariance
+    .Object@expectedFullMean <- expectedFullMean
 		return(.Object)
 	}
 )
 
 setMethod("genericExpDependencies", signature("MxExpectationRAM"),
 	function(.Object, dependencies) {
+    dependencies <- callNextMethod()
 	sources <- c(.Object@A, .Object@S, .Object@F, .Object@M, .Object@thresholds, .Object@between)
 	sources <- sources[!is.na(sources)]
   sink <- .Object@name
-  sink <- c(sink, .Object@expectedCovariance, .Object@expectedMean)
+    sink <- c(sink, .Object@expectedCovariance, .Object@expectedMean,
+              .Object@expectedFullCovariance, .Object@expectedFullMean)
 	dependencies <- imxAddDependency(sources, sink, dependencies)
 	return(dependencies)
 })
 
-setMethod("qualifyNames", signature("MxExpectationRAM"), 
+setMethod("qualifyNames", signature("MxExpectationRAM"),
 	function(.Object, modelname, namespace) {
+    .Object <- callNextMethod()
 		.Object@name <- imxIdentifier(modelname, .Object@name)
-		.Object@A <- imxConvertIdentifier(.Object@A, modelname, namespace, TRUE)
-		.Object@S <- imxConvertIdentifier(.Object@S, modelname, namespace, TRUE)
-		.Object@F <- imxConvertIdentifier(.Object@F, modelname, namespace, TRUE)
-		.Object@M <- imxConvertIdentifier(.Object@M, modelname, namespace, TRUE)
-		.Object@data <- imxConvertIdentifier(.Object@data, modelname, namespace)
-		.Object@thresholds <- sapply(.Object@thresholds, 
-					     imxConvertIdentifier, modelname, namespace)
-		.Object@between <- imxConvertIdentifier(.Object@between, modelname, namespace)
-    callNextMethod(.Object, modelname, namespace)
+    for (sl in c('A', 'S', 'F', 'M', 'thresholds', 'between',
+                 'expectedCovariance', 'expectedMean',
+                 'expectedFullCovariance', 'expectedFullMean')) {
+      slot(.Object, sl) <- imxConvertIdentifier(slot(.Object, sl), modelname, namespace, TRUE)
+    }
+    .Object
 })
 
 setMethod("genericExpRename", signature("MxExpectationRAM"),
 	function(.Object, oldname, newname) {
-		.Object@A <- renameReference(.Object@A, oldname, newname)
-		.Object@S <- renameReference(.Object@S, oldname, newname)
-		.Object@F <- renameReference(.Object@F, oldname, newname)
-		.Object@M <- renameReference(.Object@M, oldname, newname)
-		.Object@data <- renameReference(.Object@data, oldname, newname)
-		.Object@thresholds <- sapply(.Object@thresholds, renameReference, oldname, newname)		
-    callNextMethod(.Object, oldname, newname)
+    .Object <- callNextMethod()
+    for (sl in c('A', 'S', 'F', 'M', 'thresholds', 'between',
+                 'expectedCovariance', 'expectedMean',
+                 'expectedFullCovariance', 'expectedFullMean')) {
+      slot(.Object, sl) <- renameReference(slot(.Object, sl), oldname, newname)
+    }
+    .Object
 })
 
-setMethod("genericExpFunConvert", signature("MxExpectationRAM"), 
+setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 	function(.Object, flatModel, model, labelsData, dependencies) {
-		modelname <- imxReverseIdentifier(model, .Object@name)[[1]]	
+		modelname <- imxReverseIdentifier(model, .Object@name)[[1]]
 		name <- .Object@name
 		aMatrix <- .Object@A
 		sMatrix <- .Object@S
@@ -124,7 +129,7 @@ setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 		}
 		mxDataObject <- flatModel@datasets[[.Object@data]]
 		if (.hasSlot(.Object, "between") && length(.Object@between)) {
-			.Object@between <- sapply(.Object@between, function(bName) {
+			sapply(.Object@between, function(bName) {
 				zMat <- flatModel[[ bName ]]
 				if (is.null(zMat)) {
 					msg <- paste("Level transition matrix", omxQuotes(bName),
@@ -132,36 +137,35 @@ setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 					stop(msg, call. = FALSE)
 				}
 				expName <- paste0(zMat@joinModel, imxSeparatorChar, 'expectation')
-				upperA <- flatModel[[ flatModel@expectations[[ expName ]]$A ]]
-				lowerA <- flatModel[[ aMatrix ]]
+				upperF <- flatModel[[ flatModel@expectations[[ expName ]]$F ]]
+				lowerF <- flatModel[[ fMatrix ]]
 
-				if (length(rownames(zMat)) != length(colnames(lowerA))) {
+				if (length(rownames(zMat)) != length(colnames(lowerF))) {
 					msg <- paste("Join mapping matrix", zMat@name,
-						     "must have", length(colnames(lowerA)), "rows:",
-						     omxQuotes(colnames(lowerA)))
+						     "must have", length(colnames(lowerF)), "rows:",
+						     omxQuotes(colnames(lowerF)))
 					stop(msg, call. = FALSE)
 				}
-				lowerMatch <- rownames(zMat) == colnames(lowerA)
+				lowerMatch <- rownames(zMat) == colnames(lowerF)
 				if (any(!lowerMatch)) {
 					msg <- paste("Join mapping matrix", zMat@name,
 						     "needs mapping rows for",
-						     omxQuotes(colnames(lowerA)[!lowerMatch]))
+						     omxQuotes(colnames(lowerF)[!lowerMatch]))
 					stop(msg, call. = FALSE)
 				}
-				if (length(colnames(zMat)) != length(colnames(upperA))) {
+				if (length(colnames(zMat)) != length(colnames(upperF))) {
 					msg <- paste("Join mapping matrix", zMat@name,
-						     "must have", length(colnames(upperA)), "columns:",
-						     omxQuotes(colnames(upperA)))
+						     "must have", length(colnames(upperF)), "columns:",
+						     omxQuotes(colnames(upperF)))
 					stop(msg, call. = FALSE)
 				}
-				upperMatch <- colnames(zMat) == colnames(upperA)
+				upperMatch <- colnames(zMat) == colnames(upperF)
 				if (any(!upperMatch)) {
 					msg <- paste("Join mapping matrix", zMat@name,
 						     "needs mapping columns for",
-						     omxQuotes(colnames(upperA)[!upperMatch]))
+						     omxQuotes(colnames(upperF)[!upperMatch]))
 					stop(msg, call. = FALSE)
 				}
-				imxLocateIndex(flatModel, bName, name)
 			})
 		}
 		checkNumericData(mxDataObject)
@@ -178,27 +182,27 @@ setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 			stop(msg, call. = FALSE)
 		}
 		hasMeanModel <- !is.na(mMatrix)
-		mMatrix <- flatModel[[mMatrix]]		
+		mMatrix <- flatModel[[mMatrix]]
 		if (hasMeanModel && !is.null(mMatrix)) {
 			means <- dimnames(mMatrix)
 			if (is.null(means)) {
 				msg <- paste("The M matrix associated",
-				"with the RAM expectation function in model", 
+				"with the RAM expectation function in model",
 				omxQuotes(modelname), "does not contain dimnames.")
-				stop(msg, call. = FALSE)	
+				stop(msg, call. = FALSE)
 			}
 			meanRows <- means[[1]]
 			meanCols <- means[[2]]
 			if (!is.null(meanRows) && length(meanRows) > 1) {
 				msg <- paste("The M matrix associated",
-				"with the RAM expectation function in model", 
+				"with the RAM expectation function in model",
 				omxQuotes(modelname), "is not a 1 x N matrix.")
 				stop(msg, call. = FALSE)
 			}
 			if (!identical(dimnames(fMatrix)[[2]], meanCols)) {
 				msg <- paste("The column names of the F matrix",
 					"and the column names of the M matrix",
-					"in model", 
+					"in model",
 					omxQuotes(modelname), "do not contain identical",
 					"names.")
 				stop(msg, call. = FALSE)
@@ -211,7 +215,6 @@ setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 			if (mxDataObject@type == 'raw') {
 				threshName <- .Object@thresholds
 				verifyThresholds(flatModel, model, labelsData, data, translatedNames, threshName)
-				.Object@thresholds <- imxLocateIndex(flatModel, threshName, name)
 				if (length(mxDataObject@observed) == 0) {
 					.Object@data <- as.integer(NA)
 				}
@@ -230,7 +233,7 @@ setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 					if (length(varsNotInData) > 0) {
 						msg <- paste(msg,
 							     "To get you started, the following variables are used but",
-							     "are not in the observed data:", 
+							     "are not in the observed data:",
 							     omxQuotes(varsNotInData))
 					}
 					stop(msg, call. = FALSE)
@@ -239,6 +242,7 @@ setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 		} else {
 			.Object@thresholds <- as.integer(NA)
 		}
+    .Object@selectionPlan <- prepSelectionPlan(.Object@selectionPlan, translatedNames)
 		if(length(.Object@dims) > nrow(fMatrix) && length(translatedNames) == nrow(fMatrix)){
 			.Object@dims <- translatedNames
 		}
@@ -247,19 +251,18 @@ setMethod("genericExpFunConvert", signature("MxExpectationRAM"),
 
 setMethod("genericNameToNumber", signature("MxExpectationRAM"),
 	  function(.Object, flatModel, model) {
+      .Object <- callNextMethod()
 		  name <- .Object@name
-		  data <- .Object@data
-		  .Object@data <- imxLocateIndex(flatModel, data, name)
-		  .Object@A <- imxLocateIndex(flatModel, .Object@A, name)
-		  .Object@S <- imxLocateIndex(flatModel, .Object@S, name)
-		  .Object@F <- imxLocateIndex(flatModel, .Object@F, name)
-		  .Object@M <- imxLocateIndex(flatModel, .Object@M, name)
-      callNextMethod(.Object, flatModel, model)
+    for (sl in c('A', 'S', 'F', 'M', 'between',
+                 'expectedFullCovariance', 'expectedFullMean')) {
+		  slot(.Object,sl) <- imxLocateIndex(flatModel, slot(.Object,sl), name)
+    }
+      .Object
 	  })
 
 setMethod("genericGetExpected", signature("MxExpectationRAM"),
 	  function(.Object, model, what, defvar.row=1, subname=model@name) {
-		  ret <- list()
+		  ret <- callNextMethod()
 		  Aname <- .modifyDottedName(subname, .Object@A, sep=".")
 		  Sname <- .modifyDottedName(subname, .Object@S, sep=".")
 		  Fname <- .modifyDottedName(subname, .Object@F, sep=".")
@@ -268,10 +271,37 @@ setMethod("genericGetExpected", signature("MxExpectationRAM"),
 		  S <- mxEvalByName(Sname, model, compute=TRUE, defvar.row=defvar.row)
 		  F <- mxEvalByName(Fname, model, compute=TRUE, defvar.row=defvar.row)
 		  I <- diag(1, nrow=nrow(A))
+      # need to compute covariance when there is Pearson selection
+      ImA <- solve(I-A)
+      origCov <- list()
+      origCov[[1]] <- ImA %*% S %*% t(ImA)
+      if (single.na(.Object@selectionVector)) {
+        cov <- origCov[[1]]
+      } else {
+        selPlan <- .Object@selectionPlan
+        selVec <- mxEvalByName(.Object@selectionVector, model, compute=TRUE, defvar.row=defvar.row)
+        sx <- 1L
+        rx <- 1L
+        curStep <- selPlan[sx,'step']
+        newCov <- list()
+        while (rx <= nrow(selPlan)) {
+          nc <- origCov[[sx]]
+          nc[selPlan[rx,'from'],selPlan[rx,'to']] <- selVec[rx,1]
+          nc[selPlan[rx,'to'],selPlan[rx,'from']] <- selVec[rx,1]
+          if (rx == nrow(selPlan) || (rx < nrow(selPlan) && curStep != selPlan[rx+1,'step'])) {
+            newCov[[sx]] <- nc
+            cov <- mxPearsonSelCov(origCov[[sx]], newCov[[sx]])
+            if (rx < nrow(selPlan)) {
+              sx <- sx + 1
+              origCov[[sx]] <- cov
+              curStep <- selPlan[sx,'step']
+            }
+          }
+          rx <- rx + 1
+        }
+      }
 		  if (any(c('covariance','covariances') %in% what)) {
-			  ImA <- solve(I-A)
-			  cov <- F %*% ImA %*% S %*% t(ImA) %*% t(F)
-			  ret[['covariance']] <- cov
+			  ret[['covariance']] <- F %*% cov %*% t(F)
 		  }
 		  if (any(c('mean','means') %in% what)) {
 				if(single.na(Mname)){
@@ -279,18 +309,16 @@ setMethod("genericGetExpected", signature("MxExpectationRAM"),
 				} else {
 					Mname <- .modifyDottedName(subname, Mname, sep=".")
 					M <- mxEvalByName(Mname, model, compute=TRUE, defvar.row=defvar.row)
-					mean <- M %*% t(solve(I-A)) %*% t(F)
+					fullMean <- M %*% t(solve(I-A))
+          if (!single.na(.Object@selectionVector)) {
+            for (sx in 1:length(origCov)) {
+              fullMean <- t(mxPearsonSelMean(origCov[[sx]], newCov[[sx]], t(fullMean)))
+            }
+          }
+          mean <- fullMean %*% t(F)
 			  }
 				ret[['means']] <- mean
 			}
-			if ('thresholds' %in% what) {
-				thrname <- .Object@thresholds
-				if(!single.na(thrname)){
-					thrname <- .modifyDottedName(subname, thrname, sep=".")
-					thr <- mxEvalByName(thrname, model, compute=TRUE, defvar.row=defvar.row)
-				} else {thr <- matrix( , 0, 0)}
-				ret[['thresholds']] <- thr
-			}
 			ret
 })
 
@@ -298,10 +326,10 @@ setMethod("genericGetExpected", signature("MxExpectationRAM"),
 ##'
 ##' Get the potency of a matrix for inversion speed-up
 ##'
-##' @param A MxMatrix object 
+##' @param A MxMatrix object
 ##' @param maxdepth Numeric. maximum depth to check
 ##' @details This function is used internally by the \link{mxExpectationRAM} function
-##' to determine how far to expand \eqn{(I-A)^{-1} = I + A + A^2 + A^3 + ...}.  It is 
+##' to determine how far to expand \eqn{(I-A)^{-1} = I + A + A^2 + A^3 + ...}.  It is
 ##' similarly used by \link{mxExpectationLISREL} in expanding \eqn{(I-B)^{-1} = I + B + B^2 + B^3 + ...}.
 ##' In many situations \eqn{A^2} is a zero matrix (nilpotent of order 2).  So when \eqn{A} has large
 ##' dimension it is much faster to compute \eqn{I+A} than \eqn{(I-A)^{-1}}.
@@ -324,7 +352,7 @@ generateDepthHelper <- function(aValues, currentProduct, depth, maxdepth) {
 	if (depth > maxdepth) {
 		return(as.integer(NA))
 	}
-	if (all(currentProduct == 0)) { 
+	if (all(currentProduct == 0)) {
 		return(as.integer(depth))
 	} else {
 		return(generateDepthHelper(aValues, currentProduct %*% aValues, depth + 1, maxdepth))
@@ -360,20 +388,20 @@ updateRAMdimnames <- function(flatExpectation, flatJob) {
 	fMatrix <- flatJob[[fMatrixName]]
 	if (is.null(fMatrix)) {
 		modelname <- getModelName(flatExpectation)
-		stop(paste("Unknown F matrix name", 
+		stop(paste("Unknown F matrix name",
 			omxQuotes(simplifyName(fMatrixName, modelname)),
 			"detected in the RAM expectation function",
 			"of model", omxQuotes(modelname)), call. = FALSE)
 	}
 	dims <- flatExpectation@dims
-	if (!is.null(dimnames(fMatrix)) && !single.na(dims) && 
+	if (!is.null(dimnames(fMatrix)) && !single.na(dims) &&
 		!identical(dimnames(fMatrix)[[2]], dims)) {
 		modelname <- getModelName(flatExpectation)
 		msg <- paste("The F matrix associated",
-			"with the RAM expectation function in model", 
+			"with the RAM expectation function in model",
 			omxQuotes(modelname), "contains dimnames and",
 			"the expectation function has specified dimnames")
-		stop(msg, call.=FALSE)		
+		stop(msg, call.=FALSE)
 	}
 	if (is.null(dimnames(fMatrix)) && !single.na(dims)) {
 		dimnames(flatJob[[fMatrixName]]) <- list(c(), dims)
@@ -387,10 +415,10 @@ updateRAMdimnames <- function(flatExpectation, flatJob) {
 		!identical(dimnames(mMatrix), list(NULL, dims))) {
 		modelname <- getModelName(flatExpectation)
 		msg <- paste("The M matrix associated",
-			"with the RAM expectation function in model", 
+			"with the RAM expectation function in model",
 			omxQuotes(modelname), "contains dimnames and",
 			"the expectation function has specified dimnames")
-		stop(msg, call.=FALSE)	
+		stop(msg, call.=FALSE)
 	}
 
 	if (is.null(dimnames(mMatrix)) && !single.na(dims)) {
@@ -434,7 +462,7 @@ setMethod("genericExpConvertEntities", "MxExpectationRAM",
 				omxQuotes(modelname))
 			stop(msg, call.=FALSE)
 		}
-		
+
 		flatModel <- updateRAMdimnames(.Object, flatModel)
 
 		if (flatModel@datasets[[.Object@data]]@type != 'raw') {
@@ -469,7 +497,9 @@ imxSimpleRAMPredicate <- function(model) {
 
 mxExpectationRAM <- function(A="A", S="S", F="F", M = NA, dimnames = NA, thresholds = NA,
                              threshnames = dimnames, ..., between=NULL, verbose=0L, .useSparse=NA,
-                             expectedCovariance=NULL, expectedMean=NULL) {
+                             expectedCovariance=NULL, expectedMean=NULL,
+                             discrete = as.character(NA), selectionVector = as.character(NA),
+                             expectedFullCovariance=NULL, expectedFullMean=NULL) {
 
 	prohibitDotdotdot(list(...))
 
@@ -477,7 +507,7 @@ mxExpectationRAM <- function(A="A", S="S", F="F", M = NA, dimnames = NA, thresho
 		msg <- paste("argument 'A' is not a string",
 			"(the name of the 'A' matrix)")
 		stop(msg)
-	}	
+	}
 	if (typeof(S) != "character") {
 		msg <- paste("argument 'S' is not a string",
 			"(the name of the 'S' matrix)")
@@ -511,7 +541,8 @@ mxExpectationRAM <- function(A="A", S="S", F="F", M = NA, dimnames = NA, thresho
 	threshnames <- checkThreshnames(threshnames)
 	return(new("MxExpectationRAM", A, S, F, M, dimnames, thresholds, threshnames,
              between, as.integer(verbose), as.logical(.useSparse),
-             expectedCovariance, expectedMean))
+             expectedCovariance, expectedMean, discrete, selectionVector,
+             expectedFullCovariance, expectedFullMean))
 }
 
 displayMxExpectationRAM <- function(expectation) {
@@ -528,36 +559,41 @@ displayMxExpectationRAM <- function(expectation) {
 		cat("$dims : NA \n")
 	} else {
 		cat("$dims :", omxQuotes(expectation@dims), '\n')
-	}		
+	}
 	if (single.na(expectation@thresholds)) {
 		cat("$thresholds : NA \n")
 	} else {
 		cat("$thresholds :", omxQuotes(expectation@thresholds), '\n')
 	}
+	if (single.na(expectation@discrete)) {
+		cat("$discrete : NA \n")
+	} else {
+		cat("$discrete :", omxQuotes(expectation@discrete), '\n')
+	}
 	if (length(expectation@between)) {
 		cat("$between :", omxQuotes(expectation@between), fill=TRUE)
 	}
 	invisible(expectation)
 }
 
-setMethod("print", "MxExpectationRAM", function(x,...) { 
-	displayMxExpectationRAM(x) 
+setMethod("print", "MxExpectationRAM", function(x,...) {
+	displayMxExpectationRAM(x)
 })
 
-setMethod("show", "MxExpectationRAM", function(object) { 
-	displayMxExpectationRAM(object) 
+setMethod("show", "MxExpectationRAM", function(object) {
+	displayMxExpectationRAM(object)
 })
 
 
 #------------------------------------------------------------------------------
 setMethod("genericGenerateData", signature("MxExpectationRAM"),
 	function(.Object, model, nrows, subname, empirical, returnModel, use.miss,
-		 .backend, nrowsProportion)
+		 .backend, nrowsProportion, silent)
 	{
 	  fellner <- length(model$expectation$between)
 	  if (!fellner) {
 	    return(generateNormalData(model, nrows, subname, empirical, returnModel, use.miss,
-				      .backend, nrowsProportion))
+				      .backend, nrowsProportion, silent))
 	  } else {
 	    if (!use.miss) {
 	      stop("use.miss=FALSE is not implemented for relational models")
