@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 The OpenMx Project
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -21,13 +21,15 @@ setClass(Class = "MxFitFunctionGREML",
            numObs = "integer",
            aug = "MxCharOrNumber",
            augGrad = "MxCharOrNumber",
-           augHess = "MxCharOrNumber"),
+           augHess = "MxCharOrNumber",
+           autoDerivType = "character",
+           infoMatType = "character"),
          contains = "MxBaseFitFunction")
 
 
 setMethod("initialize", "MxFitFunctionGREML",
           function(.Object, name = 'fitfunction', dV=character(0), MLfit=0, vector=FALSE, numObs=0L, aug=character(0),
-          				 augGrad=character(0), augHess=character(0)) {
+          				 augGrad=character(0), augHess=character(0), autoDerivType=character(0), infoMatType=character(0)) {
             .Object@name <- name
             .Object@dV <- dV
             .Object@dVnames <- as.character(names(dV))
@@ -37,6 +39,8 @@ setMethod("initialize", "MxFitFunctionGREML",
             .Object@aug <- aug
             .Object@augGrad <- augGrad
             .Object@augHess <- augHess
+            .Object@autoDerivType <- autoDerivType
+            .Object@infoMatType <- infoMatType
             return(.Object)
           }
 )
@@ -122,6 +126,10 @@ setMethod("generateReferenceModels", "MxFitFunctionGREML",
 					})
 
 
-mxFitFunctionGREML <- function(dV=character(0), aug=character(0), augGrad=character(0), augHess=character(0)){
-  return(new("MxFitFunctionGREML",dV=dV,aug=aug,augGrad=augGrad,augHess=augHess))
+mxFitFunctionGREML <- function(
+	dV=character(0), aug=character(0), augGrad=character(0), augHess=character(0), autoDerivType=c("semiAnalyt","numeric"),
+	infoMatType=c("average","expected")){
+	autoDerivType = as.character(match.barg(autoDerivType,c("semiAnalyt","numeric")))
+	infoMatType = as.character(match.barg(infoMatType,c("average","expected")))
+  return(new("MxFitFunctionGREML",dV=dV,aug=aug,augGrad=augGrad,augHess=augHess,autoDerivType=autoDerivType,infoMatType=infoMatType))
 }
