@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -203,7 +203,7 @@ generateIntervalListHelper <- function(interval, flatModel, modelname,
 		entityName <- components[[1]]
 		rows <- eval(parse(text = components[[2]]), envir = globalenv())
 		cols <- eval(parse(text = components[[3]]), envir = globalenv())
-		entityValue <- eval(substitute(mxEval(x, flatModel, compute=TRUE),
+		entityValue <- eval(substitute(mxEval(x, flatModel, compute=TRUE, .extraBack=7L),
 			list(x = as.symbol(entityName))))
 		if (is.null(rows)) {
 			rows <- 1:nrow(entityValue)
@@ -348,6 +348,7 @@ omxParallelCI <- function(model, run = TRUE, verbose=0L, independentSubmodels=TR
 	if(missing(model) || !is(model, "MxModel")) {
 		stop("first argument must be a MxModel object")
 	}
+	assertModelRunAndFresh(model)
 	if(length(model@output) == 0) {
 		stop("'model' argument to omxParallelCI must be a fitted model")
 	}
