@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -120,6 +120,7 @@ setGeneric("imxInitModel", function(model) {
 ##' @param name name
 ##' @param manifestVars manifestVars
 ##' @param latentVars latentVars
+##' @param productVars productVars
 ##' @param submodels submodels
 ##' @param remove remove
 ##' @param independent independent
@@ -128,7 +129,7 @@ setGeneric("imxInitModel", function(model) {
 ##' imxModelBuilder,MxModel-method
 ##' imxModelBuilder,MxRAMModel-method
 setGeneric("imxModelBuilder", function(model, lst, name, 
-	manifestVars, latentVars, submodels, remove, independent) {
+	manifestVars, latentVars, productVars, submodels, remove, independent) {
 	return(standardGeneric("imxModelBuilder")) } )
 
 ##' imxTypeName
@@ -301,7 +302,7 @@ mxModel <- function(model = NA, ..., manifestVars = NA, latentVars = NA,
 	submodels <- lst[filter]
 	lst <- lst[!filter]
 	model <- imxModelBuilder(model, lst, name, manifestVars,
-		latentVars, submodels, remove, independent)
+		latentVars, productVars, submodels, remove, independent)
 	return(model)
 }
 
@@ -316,7 +317,7 @@ productArgument <- function(x){
 		productVars <- x[[which(filter0)]]
 		x <- x[!filter0]
 	} else {
-		productVars <- NA
+		productVars <- character()
 	}
 	return(list(x, productVars=productVars))
 }
@@ -368,13 +369,14 @@ typeArgument <- function(model, type) {
 ##' @param name name
 ##' @param manifestVars manifestVars
 ##' @param latentVars latentVars
+##' @param productVars productVars
 ##' @param submodels submodels
 ##' @param remove remove
 ##' @param independent independent
 imxGenericModelBuilder <- function(model, lst, name, 
-	manifestVars, latentVars, submodels, remove, independent) {
+	manifestVars, latentVars, productVars, submodels, remove, independent) {
 	model <- nameArgument(model, name)
-	model <- variablesArgument(model, manifestVars, latentVars, submodels, remove)
+	model <- variablesArgument(model, manifestVars, latentVars, productVars, submodels, remove)
 	model <- listArgument(model, lst, remove)
 	model <- independentArgument(model, independent)
 	return(model)
@@ -430,13 +432,16 @@ varsToCharacter2 <- function(vars, vartype) {
 	}
 }
 
-variablesArgument <- function(model, manifestVars, latentVars, submodels, remove) {
+variablesArgument <- function(model, manifestVars, latentVars, productVars, submodels, remove) {
 	if (single.na(manifestVars)) {
 		manifestVars <- character()
 	}
 	if (single.na(latentVars)) {
 		latentVars <- character()
 	}
+	if(length(productVars) > 0 && !single.na(productVars)) {
+		stop("Whoopsie! Product nodes/variables are not currently supported for generic MxModel objects.")
+	}
 	if (remove == TRUE) {
 		model <- modelRemoveVariables(model, latentVars, manifestVars)
 		if (length(submodels)) for(i in 1:length(submodels)) {
