@@ -1,12 +1,12 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
 #   You may obtain a copy of the License at
-# 
+#
 #        http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 #   Unless required by applicable law or agreed to in writing, software
 #   distributed under the License is distributed on an "AS IS" BASIS,
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -35,11 +35,11 @@ imxVariableTypes <- c("exogenous", "endogenous")
 
 # Define generic functions
 
-setMethod("imxTypeName", "MxRAMModel", 
+setMethod("imxTypeName", "MxRAMModel",
 	function(model) { "RAM" }
 )
 
-setMethod("imxInitModel", "MxRAMModel", 
+setMethod("imxInitModel", "MxRAMModel",
 	function(model) {
 		if (any(!is.na(match(model@name, c('A','S','F','M'))))) {
 			stop(paste("The name", omxQuotes(model@name), "is not valid",
@@ -63,15 +63,15 @@ setMethod("imxInitModel", "MxRAMModel",
 	}
 )
 
-setMethod("imxModelBuilder", "MxRAMModel", 
-	function(model, lst, name, 
-		 manifestVars, latentVars, submodels, remove, independent) {
+setMethod("imxModelBuilder", "MxRAMModel",
+	function(model, lst, name,
+		 manifestVars, latentVars, productVars, submodels, remove, independent) {
 		model <- nameArgument(model, name)
-		model <- variablesArgumentRAM(model, manifestVars, latentVars, submodels, remove)
+		model <- variablesArgumentRAM(model, manifestVars, latentVars, productVars, submodels, remove)
 		model <- listArgumentRAM(model, lst, remove)
 		notPathOrData <- getNotPathsOrData(lst)
-		callNextMethod(model, notPathOrData, NA, character(), 
-			character(), list(), remove, independent)
+		callNextMethod(model, notPathOrData, NA, character(),
+			character(), character(), list(), remove, independent)
 	}
 )
 
@@ -85,28 +85,6 @@ setMethod("imxVerifyModel", "MxRAMModel",
 				       " mxPath(from = 'x1', to = 'y1')")
 			  stop(msg, call. = FALSE)
 		  }
-		  expectation <- model$expectation
-		  if (!is.null(expectation) && is(expectation, "MxExpectationRAM")) {
-			  if (!is.null(model@data)) {
-				  threshNames <- intersect(getDataThresholdNames(model@data), model@manifestVars)
-				  # Only pay attention to (1) manifest variables (2) that need thresholds.
-				  # This saves the case where an mxFactor is used as a definition variable.
-				  if(!is.null(model[[expectation@thresholds]])) {
-					  missingThresholds <- setdiff(threshNames, colnames(model[[expectation@thresholds]]))
-				  } else {
-					  missingThresholds <- threshNames
-				  }
-				  if(length(missingThresholds)) {
-					  msg <- paste("The RAM model", omxQuotes(model@name),
-						       "contains data that requires thresholds for columns",
-						       omxQuotes(missingThresholds), "but has not specified any",
-							   "thresholds for those columns.",
-							   "You can specify thresholds for your model like this:",
-							   "mxThreshold(vars='x1', nThresh=1, values=0)")
-					  stop(msg, call. = FALSE)
-				  }
-			  }
-		  }
 		  if (length(model@submodels) > 0) {
 			  return(all(sapply(model@submodels, imxVerifyModel)))
 		  }
@@ -116,7 +94,7 @@ setMethod("imxVerifyModel", "MxRAMModel",
 
 # Helper functions used by the generic functions
 
-variablesArgumentRAM <- function(model, manifestVars, latentVars, submodels, remove) {
+variablesArgumentRAM <- function(model, manifestVars, latentVars, productVars, submodels, remove) {
 	manifestVars <- unlist(manifestVars)
 	latentVars <- unlist(latentVars)
 	if (single.na(manifestVars)) {
@@ -125,6 +103,10 @@ variablesArgumentRAM <- function(model, manifestVars, latentVars, submodels, rem
 	if (single.na(latentVars)) {
 		latentVars <- character()
 	}
+	if (single.na(productVars)) {
+		productVars <- character()
+	}
+	latentVars <- c(latentVars, productVars)
 	if (remove == TRUE) {
 		if (length(latentVars) + length(manifestVars) > 0) {
 			model <- removeVariablesRAM(model, latentVars, manifestVars)
@@ -138,6 +120,7 @@ variablesArgumentRAM <- function(model, manifestVars, latentVars, submodels, rem
 			manifestVars <- varsToCharacter(manifestVars, "manifest")
 			checkVariables(model, latentVars, manifestVars)
 			model <- addVariablesRAM(model, latentVars, manifestVars)
+			model[['expectation']]$isProductNode <- colnames(model$A) %in% productVars
 		}
 		if (length(submodels)) for(i in 1:length(submodels)) {
 			model <- addSingleNamedEntity(model, submodels[[i]])
@@ -246,6 +229,11 @@ addEntriesRAM <- function(model, entries) {
 	if(length(thresholds) > 0) {
 		model <- insertAllThresholdsRAM(model, thresholds)
 	}
+	filter <- sapply(entries, is, "DiscreteBase")
+	discrete <- entries[filter]
+	if(length(discrete)) {
+		model <- insertDiscreteRAM(model, discrete)
+	}
 	filter <- sapply(entries, is, "MxData")
 	data <- entries[filter]
 	if (length(data) > 0) {
@@ -293,18 +281,18 @@ getNotPathsOrData <- function(lst) {
 	pathfilter <- sapply(lst, is, "MxPath")
 	datafilter <- sapply(lst, is, "MxData")
 	thresholdfilter <- sapply(lst, is, "MxThreshold")
-	retval <- lst[!(pathfilter | datafilter | thresholdfilter)]
+	discretefilter <- sapply(lst, is, "DiscreteBase")
+	retval <- lst[!(pathfilter | datafilter | thresholdfilter | discretefilter)]
 	return(retval)
 }
 
-expectationIsMissingThresholds <- function(model) {
+expectationIsMissing <- function(model, what) {
 	expectation <- model@expectation
 	return(!is.null(expectation) &&
-	       (is(expectation, "MxExpectationRAM") || is(expectation, "MxExpectationLISREL")) &&
-		is.na(expectation@thresholds))
+           is(expectation, "BaseExpectationNormal") &&
+           is.na(slot(expectation, what)))
 }
 
-
 expectationIsMissingMeans <- function(model) {
 	expectation <- model@expectation
 	return(!is.null(expectation) &&
@@ -312,17 +300,74 @@ expectationIsMissingMeans <- function(model) {
 		is.na(expectation@M))
 }
 
+insertDiscreteRAM <- function(model, discrete) {
+  DiscSpec <- model@expectation@discreteSpec
+	Disc <- model[[model@expectation@discrete]]
+	if (is.null(Disc)) {
+		Disc <- mxMatrix("Full", 0, 0, name="Discrete", condenseSlots=FALSE)
+		if(expectationIsMissing(model, 'discrete')) {
+			model@expectation@discrete <- "Discrete"
+		} else {
+			Disc <- model[[model@expectation@discrete]]
+		}
+	}
+
+	legalVars <- model@manifestVars
+	if (is.list(legalVars)) {
+	  # for LISREL
+	  legalVars <- legalVars$endogenous
+	}
+
+	allVars <- unique(sapply(discrete, slot, "variable"))
+	varExist <- allVars %in% legalVars
+	if(!all(varExist)) {
+		missingVars <- allVars[!varExist]
+		stop(paste("You need to add", omxQuotes(missingVars),
+               "to the manifestVars before you",
+               "use them as discrete indicators"), call. = FALSE)
+	}
+
+	if(!is.list(discrete)) discrete <- list(discrete)
+  todoSpec <- lapply(discrete, function(x) getSpec(x))
+  todo <- lapply(discrete, function(x) as(x, 'MxMatrix'))
+  newRows <- max(sapply(todo, nrow))
+
+	newVars <- union(colnames(Disc), allVars)
+	if(length(newVars) > ncol(Disc)) {
+		newDisc <- mxMatrix("Full", newRows, length(newVars),
+                        dimnames=list(NULL, newVars), name=Disc@name, condenseSlots=FALSE)
+    newDisc$values <- NA
+		if (ncol(Disc)) newDisc[1:nrow(Disc), 1:ncol(Disc)] <- Disc
+		Disc <- newDisc
+    newSpec <- matrix(NA, 2, length(newVars))
+    colnames(newSpec) <- newVars
+    newSpec[,colnames(DiscSpec)] <- DiscSpec
+    DiscSpec <- newSpec
+	}
+
+  for (c1 in 1:length(allVars)) {
+    n1 <- allVars[c1]
+    DiscSpec[,n1] <- todoSpec[[c1]]
+    mat <- todo[[c1]]
+    Disc[1:nrow(mat),n1] <- mat
+  }
+
+  model@expectation@discreteSpec <- DiscSpec
+	model[[Disc@name]] <- Disc
+  model
+}
+
 insertAllThresholdsRAM <- function(model, thresholds) {
 	Thresh <- model[[model@expectation@thresholds]]
-	if (is.null(Thresh)) { 
+	if (is.null(Thresh)) {
 		Thresh <- mxMatrix("Full", 0, 0, name="Thresholds", condenseSlots=FALSE)
-		if(expectationIsMissingThresholds(model)) {
+		if(expectationIsMissing(model, 'thresholds')) {
 			model@expectation@thresholds <- "Thresholds"
 		} else {
 			Thresh <- model[[model@expectation@thresholds]]
 		}
 	}
-	
+
 	legalVars <- model@manifestVars
 	if (is.list(legalVars)) {
 	  # for LISREL
@@ -334,26 +379,27 @@ insertAllThresholdsRAM <- function(model, thresholds) {
 	varExist <- allVars %in% legalVars
 	if(!all(varExist)) {
 		missingVars <- allVars[!varExist]
-		stop(paste("You need to add", 
-		omxQuotes(missingVars), 
+		stop(paste("You need to add",
+		omxQuotes(missingVars),
 			"to the manifestVars before you",
 			"can assign them thresholds."), call. = FALSE)
 	}
 
 	maxNThresh <- max(sapply(thresholds, getElement, "nThresh"))
-	
+
 	newVars <- union(colnames(Thresh), allVars)
 	if(length(newVars) > ncol(Thresh)) {  # Rebuild Threshold matrix if needed
 		oldCols <- ncol(Thresh)
 		oldRows <- nrow(Thresh)
 		newCols <- length(newVars)
 		newRows <- max(nrow(Thresh), maxNThresh)
-		newThresh <- mxMatrix("Full", newRows, newCols, dimnames=list(NULL,
-			newVars), name=Thresh@name, condenseSlots=FALSE)  # Maintains the old ordering
+		newThresh <- mxMatrix("Full", newRows, newCols, dimnames=list(NULL, newVars),
+                          name=Thresh@name, condenseSlots=FALSE)  # Maintains the old ordering
+    newThresh@values[,] <- NA
 		if(oldRows > 0 && oldCols > 0) {
 			newThresh[1:oldRows, 1:oldCols] <- Thresh
 		}
-		Thresh <- newThresh 
+		Thresh <- newThresh
 	}
 	if(!is.list(thresholds)) { thresholds <- list(thresholds)}
 	for(i in 1:length(thresholds)) {
@@ -364,7 +410,7 @@ insertAllThresholdsRAM <- function(model, thresholds) {
 			msg = paste("The thresholds you are attempting to specify",
 				"does not have any starting values, but type='RAM' models require them.")
 		}
-		if(!identical(values, sort(values, na.last=NA))) { 
+		if(!identical(values, sort(values, na.last=NA))) {
 			msg = paste("The thresholds you are attempting to specify",
 				"has starting values that are not strictly increasing,",
 				"but type='RAM' models require them to be.")
@@ -383,19 +429,19 @@ insertAllThresholdsRAM <- function(model, thresholds) {
 	}
 
 	model[['Thresholds']] <- Thresh
-	
+
 	return(model)
 }
 
 insertAllPathsRAM <- function(model, paths) {
 	if (is.null(model[['A']])) { model[['A']] <- createMatrixA(model) }
 	if (is.null(model[['S']])) { model[['S']] <- createMatrixS(model) }
-	
+
 	for(i in 1:length(paths)) {
 		path <- paths[[i]]
-	
+
 		if ("one" %in% path@from && is.null(model[['M']])) {
-			model[['M']] <- createMatrixM(model) 
+			model[['M']] <- createMatrixM(model)
 			if(expectationIsMissingMeans(model)) {
 				model@expectation@M <- "M"
 			}
@@ -420,8 +466,8 @@ removeAllPathsRAM <- function(model, paths) {
 			path@to <- path@from
 			paths[[i]] <- path
 		}
-		
-		if (length(path@from) == 1 && (path@from == "one")) {		
+
+		if (length(path@from) == 1 && (path@from == "one")) {
 			M <- removeMeansPathRAM(path, M)
 		} else {
 			expanded <- expandPathConnect(path@from, path@to, path@connect)
@@ -465,9 +511,11 @@ insertPathRAM <- function(path, model) {
 	S_labels <- S@labels
 	S_lbound <- S@lbound
 	S_ubound <- S@ubound
+  selVec <- model[['selectionVector']]
+  selPlan <- model$expectation$selectionPlan
 
 	legalVars <- c(colnames(A), "one")
-		
+
 	for(i in 0:(maxlength - 1)) {
 		from <- allfrom[[i %% length(allfrom) + 1]]
 		to <- allto[[i %% length(allto) + 1]]
@@ -478,6 +526,7 @@ insertPathRAM <- function(path, model) {
 		nextubound <- allubound[[i %% length(allubound) + 1]]
 		nextlbound <- alllbound[[i %% length(alllbound) + 1]]
 		nextjoinKey <- alljoinKey[[i %% length(alljoinKey) + 1]]
+    nextstep <- path@step[[i %% length(path@step) + 1]]
 
 		if (!is.na(nextjoinKey)) {
 			upperFrom <- strsplit(from, imxSeparatorChar, fixed = TRUE)
@@ -568,7 +617,7 @@ insertPathRAM <- function(path, model) {
 		}
 
 		allFromTo <- unique(c(from, to))
-		varExist <- allFromTo %in% legalVars 
+		varExist <- allFromTo %in% legalVars
 		if(!all(varExist)) {
 			fromComponents <- length(strsplit(from, imxSeparatorChar, fixed = TRUE)[[1]])
 			if (fromComponents == 2) {
@@ -578,8 +627,8 @@ insertPathRAM <- function(path, model) {
 			}
 
 			missingVars <- allFromTo[!varExist]
-			stop(paste("You need to add", 
-				   omxQuotes(missingVars), 
+			stop(paste("You need to add",
+				   omxQuotes(missingVars),
 				   "to either manifestVars or latentVars before you",
 				   "can use them in a path."), call. = FALSE)
 		}
@@ -597,7 +646,25 @@ insertPathRAM <- function(path, model) {
 			next
 		}
 
-		if (arrows == 1) {
+    if (arrows == 0) {
+      # directly modify the unfiltered covariance matrix
+      # expectation: data.frame w/ step, rowname, colname ; name of mxMatrix
+      # mxMatrix: vector of parameters to plop in
+      pair <- c(from,to)[order(match(c(from,to), legalVars))]  # canonical order
+      r1 <- data.frame(step=nextstep, from=pair[1], to=pair[2])
+      selPlan <- rbind(selPlan, r1)
+      oldSelVec <- selVec
+      selVec <- mxMatrix('Full', nrow(selPlan), 1)
+      if (!is.null(oldSelVec)) selVec[1:nrow(oldSelVec),1] <- oldSelVec
+      selVec[nrow(selVec),1]$free <- nextfree
+      selVec[nrow(selVec),1]$values <- nextvalue
+      selVec[nrow(selVec),1]$labels <- nextlabel
+      selVec[nrow(selVec),1]$ubound <- nextubound
+      selVec[nrow(selVec),1]$lbound <- nextlbound
+      p1 <- order(selPlan$step, selPlan$from, selPlan$to)
+      selPlan <- selPlan[p1,]
+      selVec <- selVec[p1,]
+    } else if (arrows == 1) {
 			A_free[to, from] <- nextfree
 			A_values[to, from] <- nextvalue
 			A_labels[to, from] <- nextlabel
@@ -608,7 +675,7 @@ insertPathRAM <- function(path, model) {
 			S_free[to, from] <- FALSE
 			S_values[from, to] <- 0
 			S_labels[from, to] <- as.character(NA)
-			S_free[from, to] <- FALSE			
+			S_free[from, to] <- FALSE
 		} else if (arrows == 2) {
 			S_free[to, from] <- nextfree
 			S_values[to, from] <- nextvalue
@@ -627,25 +694,30 @@ insertPathRAM <- function(path, model) {
 			A_labels[to, from] <- as.character(NA)
 			A_free[to, from] <- FALSE
 		} else {
-			stop(paste("Unknown arrow type", arrows, 
-				   "with source", omxQuotes(from), 
+			stop(paste("Unknown arrow type", arrows,
+				   "with source", omxQuotes(from),
 				   "and sink", omxQuotes(to)),
 			     call. = FALSE)
 		}
 	}
 	A@free <- A_free
-	A@values <-	A_values 
-	A@labels <-	A_labels 
-	A@lbound <-	A_lbound 
-	A@ubound <-	A_ubound 
-	S@free <- S_free   
-	S@values <-	S_values 
-	S@labels <-	S_labels 
-	S@lbound <- S_lbound 
-	S@ubound <- S_ubound 
+	A@values <-	A_values
+	A@labels <-	A_labels
+	A@lbound <-	A_lbound
+	A@ubound <-	A_ubound
+	S@free <- S_free
+	S@values <-	S_values
+	S@labels <-	S_labels
+	S@lbound <- S_lbound
+	S@ubound <- S_ubound
 	model[['A']] <- A
 	model[['S']] <- S
 	if (!is.null(M)) model[['M']] <- M
+  if (!is.null(selVec)) {
+    model$expectation$selectionVector <- 'selectionVector'
+    model[['selectionVector']] <- selVec
+  }
+  if (!is.null(selPlan)) model$expectation$selectionPlan <- selPlan
 	model
 }
 
@@ -681,27 +753,27 @@ removePathRAM <- function(path, A, S) {
 		if (arrows == 1) {
 			A_values[to, from] <- 0
 			A_labels[to, from] <- as.character(NA)
-			A_free[to, from] <- FALSE		
+			A_free[to, from] <- FALSE
 		} else if (arrows == 2) {
 			S_values[to, from] <- 0
 			S_labels[to, from] <- as.character(NA)
-			S_free[to, from] <- FALSE		
+			S_free[to, from] <- FALSE
 			S_values[from, to] <- 0
 			S_labels[from, to] <- as.character(NA)
-			S_free[from, to] <- FALSE					
+			S_free[from, to] <- FALSE
 		} else {
-			stop(paste("Unknown arrow type", arrows, 
-					"with source", omxQuotes(from), 
+			stop(paste("Unknown arrow type", arrows,
+					"with source", omxQuotes(from),
 					"and sink", omxQuotes(to)),
 					call. = FALSE)
 		}
 	}
 	A@free <- A_free
-	A@values <-	A_values 
-	A@labels <-	A_labels 
-	S@free <- S_free   
-	S@values <-	S_values 
-	S@labels <-	S_labels 	
+	A@values <-	A_values
+	A@labels <-	A_labels
+	S@free <- S_free
+	S@values <-	S_values
+	S@labels <-	S_labels
 	return(list(A, S))
 }
 
@@ -777,16 +849,16 @@ createMatrixF <- function(model) {
 addVariablesAS <- function(oldmatrix, model, newLatent, newManifest) {
 	newLatent <- length(newLatent)
 	newManifest <- length(newManifest)
-	oldmatrix@values <- addVariablesMatrix(oldmatrix@values, 0, 
+	oldmatrix@values <- addVariablesMatrix(oldmatrix@values, 0,
 		model, newLatent, newManifest)
-	oldmatrix@free <- addVariablesMatrix(oldmatrix@free, FALSE, 
+	oldmatrix@free <- addVariablesMatrix(oldmatrix@free, FALSE,
 		model, newLatent, newManifest)
-	oldmatrix@labels <- addVariablesMatrix(oldmatrix@labels, as.character(NA), 
+	oldmatrix@labels <- addVariablesMatrix(oldmatrix@labels, as.character(NA),
 		model, newLatent, newManifest)
-	oldmatrix@lbound <- addVariablesMatrix(oldmatrix@lbound, as.numeric(NA), 
+	oldmatrix@lbound <- addVariablesMatrix(oldmatrix@lbound, as.numeric(NA),
+		model, newLatent, newManifest)
+	oldmatrix@ubound <- addVariablesMatrix(oldmatrix@ubound, as.numeric(NA),
 		model, newLatent, newManifest)
-	oldmatrix@ubound <- addVariablesMatrix(oldmatrix@ubound, as.numeric(NA), 
-		model, newLatent, newManifest)		
 	variables <- c(model@manifestVars, model@latentVars)
 	dimnames(oldmatrix) <- list(variables, variables)
 	return(oldmatrix)
@@ -796,10 +868,10 @@ addVariablesM <- function(oldmatrix, model, newLatent, newManifest) {
 	oldmatrix@values <- addVariablesMatrixM(oldmatrix@values, 0, 0, model, newLatent, newManifest)
 	oldmatrix@free   <- addVariablesMatrixM(oldmatrix@free, FALSE, TRUE, model, newLatent, newManifest)
 	oldmatrix@labels <- addVariablesMatrixM(oldmatrix@labels, as.character(NA), as.character(NA),
-		model, newLatent, newManifest) 
-	oldmatrix@lbound <- addVariablesMatrixM(oldmatrix@lbound, as.numeric(NA), as.numeric(NA), 
 		model, newLatent, newManifest)
-	oldmatrix@ubound <- addVariablesMatrixM(oldmatrix@ubound, as.numeric(NA), as.numeric(NA), 
+	oldmatrix@lbound <- addVariablesMatrixM(oldmatrix@lbound, as.numeric(NA), as.numeric(NA),
+		model, newLatent, newManifest)
+	oldmatrix@ubound <- addVariablesMatrixM(oldmatrix@ubound, as.numeric(NA), as.numeric(NA),
 		model, newLatent, newManifest)
 	dimnames(oldmatrix) <- list(NULL, c(model@manifestVars, model@latentVars))
 	return(oldmatrix)
@@ -841,10 +913,10 @@ addVariablesMatrix <- function(oldmatrix, value, model, newLatent, newManifest)
 		latentXlatent <- matrix(value, 0, 0)
 	}
 	newtop <- cbind(manifestXmanifest, matrix(value, currentManifest, newManifest),
-					manifestXlatent, matrix(value, currentManifest, newLatent)) 
+					manifestXlatent, matrix(value, currentManifest, newLatent))
 	newtop <- rbind(newtop, matrix(value, newManifest, newSize))
 	newbottom <- cbind(latentXmanifest, matrix(value, currentLatent, newManifest),
-					latentXlatent, matrix(value, currentLatent, newLatent)) 
+					latentXlatent, matrix(value, currentLatent, newLatent))
 	newbottom <- rbind(newbottom, matrix(value, newLatent, newSize))
 	newmatrix <- rbind(newtop, newbottom)
 	return(newmatrix)
@@ -855,7 +927,7 @@ addVariablesMatrixM <- function(oldmatrix, newLatentValue, newManifestValue, mod
 	newLatent <- length(newLatent)
 	currentManifest <- length(model@manifestVars) - newManifest
 	currentLatent <- length(model@latentVars) - newLatent
-	values <- c(oldmatrix[1, 1:currentManifest], 
+	values <- c(oldmatrix[1, 1:currentManifest],
 		rep.int(newManifestValue, newManifest),
 		oldmatrix[1, (currentManifest + 1) : (currentLatent + currentManifest)],
 		rep.int(newLatentValue, newLatent))
