@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -20,9 +20,7 @@ getAllModelNames <- function(model) {
 }
 
 mxRename <- function(model, newname, oldname = NA) {
-	if( !is(model, "MxModel")) {
-		stop("'model' argument is not a MxModel object")
-	}
+  warnModelCreatedByOldVersion(model)
 	if( !is.character(newname)) {
 		stop("'newname' argument is not a character string")
 	}
