@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 by the individuals mentioned in the source code history
+#   Copyright 2007-2019 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -165,17 +165,13 @@ Jor82Ex1 <- mxModel(
 #Jor82Ex1 <- mxOption(Jor82Ex1, "Standard Errors", "No")
 #Jor82Ex1 <- mxOption(Jor82Ex1, "Major iterations", 0)
 
-ex1Run <- omxCheckWarning(mxRun(Jor82Ex1),
-                          paste("OpenMx does not yet correctly handle mxData(type='cor')",
-                                'standard errors and fit statistics.',
-                                'See Steiger (1980), "Tests for comparing elements of a correlation matrix".'))
-
+ex1Run <- mxRun(Jor82Ex1)
 summary(ex1Run)
 
 
 #--------------------------------------------------------------------
 # Check for errors/warnings
-omxCheckWarning(omxCheckError(mxGetExpected(ex1Run, 'means'), "Requested component(s) 'means' had dimension or length 0.\nRequested component not in model. All I have is yours, but I ain't got no 'means'"), "Means requested, but model has no means.\nAdd appropriate TX, TY, KA, and/or AL matrices to get real means.")
+omxCheckWarning(mxGetExpected(ex1Run, 'means'), "Means requested, but model has no means.\nAdd appropriate TX, TY, KA, and/or AL matrices to get real means.")
 
 
 #------------------------------------------------------------------------------
