@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2019 by the individuals mentioned in the source code history
+#   Copyright 2007-2020 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -32,7 +32,8 @@
 # Load package define a useful function
 
 
-require(OpenMx)
+library(OpenMx)
+library(testthat)
 
 if (mxOption(NULL, "Default optimizer") == 'NPSOL') stop('SKIP')
 
@@ -166,6 +167,13 @@ summary(ramResult1)
 omxCheckCloseEnough(coef(jointWlsResults) - coef(ramWlsResults),
                     rep(0,15), 1e-3)
 
+expect_equal(jointDlsResults$output$fit, 5.061, .1)
+expect_equal(summary(jointDlsResults)$Chi, 10.126, .1)
+expect_equal(summary(jointDlsResults)$ChiDoF, 5)
+
+noOptResult <- mxRun(jointDlsResults, useOptimizer = FALSE)
+expect_true(is.null(summary(noOptResult)$Chi))
+
 #------------------------------------------------------------------------------
 # Compare ML and WLS estimates
 
@@ -230,6 +238,9 @@ omxCheckWithinPercentError(as.numeric(shan$Chi), swls$Chi, 28)
 
 obsStats <- jointWlsResults$data$observedStats
 
+expect_error(mxGetExpected(jointResults2, 'covariance', subname="xyz"),
+             "not found")
+
 mxGetExpected(jointResults2, 'covariance')
 obsStats$cov
 
