@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2020 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -261,7 +261,7 @@ static void ensureElemConform(const char *opName, FitContext *fc, omxMatrix **ma
 			EigenMatrixAdaptor m(mat1);
 			detail += mxStringifyMatrix(mat1->name(), m, empty);
 		}
-		stop("Element-wise '%s' not conformable: '%s' is %dx%d and '%s' is %dx%d\n%s",
+		mxThrow("Element-wise '%s' not conformable: '%s' is %dx%d and '%s' is %dx%d\n%s",
 			 opName, mat0->name(), mat0->rows, mat0->cols,
 			 mat1->name(), mat1->rows, mat1->cols, detail.c_str());
 	}
@@ -277,7 +277,7 @@ static void omxBroadcast(FitContext *fc, omxMatrix** matList, int numArgs, omxMa
 	}
 
 	if (src->rows != 1 || src->cols != 1) {
-		stop("Don't know how to broadcast from %dx%d source "
+		mxThrow("Don't know how to broadcast from %dx%d source "
 			 "matrix '%s' to %dx%d result matrix '%s'",
 			 src->rows, src->cols, src->name(),
 			 result->rows, result->cols, result->name());
@@ -356,7 +356,7 @@ static void omxMatrixElementMult(FitContext *fc, omxMatrix** matList, int numArg
 	int rows = first->rows;
 	int cols = first->cols;
 	int size = rows * cols;
-	
+
 	if (first->colMajor == second->colMajor) {
 		for(int i = 0; i < size; i++) {
 			omxSetVectorElement(result, i,
@@ -666,7 +666,7 @@ static void omxBinaryApproxEquals(FitContext *fc, omxMatrix** matList, int numAr
 {
         omxMatrix* first  = matList[0];
 	omxMatrix* second = matList[1];
-	omxMatrix* epsilon = matList[2]; 
+	omxMatrix* epsilon = matList[2];
 	int rows = first->rows;
 	int cols = first->cols;
 	int size = rows * cols;
@@ -679,7 +679,7 @@ static void omxBinaryApproxEquals(FitContext *fc, omxMatrix** matList, int numAr
                 double ith_value = omxVectorElement(first, i) -
                         		   omxVectorElement(second, i);
 				double epsilon_value = omxVectorElement(epsilon, i);
-				
+
 				if (ith_value < 0.0){
 					ith_value = ith_value * negativeOne;
 				}
@@ -743,7 +743,7 @@ static void omxMatrixAdd(FitContext *fc, omxMatrix** matList, int numArgs, omxMa
 	}
 }
 
-template <typename T> 
+template <typename T>
 static void matrixExtractIndices(omxMatrix *source, int dimLength, Eigen::ArrayBase<T> &out)
 {
 	/* Case 1: the source vector contains no elements */
@@ -924,7 +924,7 @@ static void omxMatrixDeterminant(FitContext *fc, omxMatrix** matList, int numArg
 	omxMatrix* inMat = matList[0];
 	int rows = inMat->rows;
 	int cols = inMat->cols;
-	if(rows != cols) stop("Determinant of non-square matrix '%s' cannot be found", inMat->name());
+	if(rows != cols) mxThrow("Determinant of non-square matrix '%s' cannot be found", inMat->name());
 
 	EigenMatrixAdaptor Ein(inMat);
 	Eigen::PartialPivLU<Eigen::MatrixXd> lu(Ein);
@@ -982,7 +982,7 @@ static void omxMatrixArithmeticMean(FitContext *fc, omxMatrix** matList, int num
 	double mean = omxVectorElement(input, 0);
 	for(int i = 1; i < matLength; i++) {
 		double val = omxVectorElement(input, i);
-		mean += (val - mean) / (i + 1);	
+		mean += (val - mean) / (i + 1);
 	}
 
 	omxSetMatrixElement(result, 0, 0, mean);
@@ -1181,91 +1181,91 @@ static void omxElementTanh(FitContext *fc, omxMatrix** matList, int numArgs, omx
 static void omxElementArcCosine(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix* inMat = matList[0];
-	
+
 	int max = inMat->cols * inMat->rows;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < max; j++) {
 		data[j] = acos(data[j]);
 	}
-	
+
 }
 
 static void omxElementArcSine(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix* inMat = matList[0];
-	
+
 	int max = inMat->cols * inMat->rows;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < max; j++) {
 		data[j] = asin(data[j]);
 	}
-	
+
 }
 
 static void omxElementArcTangent(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix* inMat = matList[0];
-	
+
 	int max = inMat->cols * inMat->rows;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < max; j++) {
 		data[j] = atan(data[j]);
 	}
-	
+
 }
 
 static void omxElementAsinh(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix* inMat = matList[0];
-	
+
 	int max = inMat->cols * inMat->rows;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < max; j++) {
 		data[j] = asinh(data[j]);
 	}
-	
+
 }
 
 static void omxElementAcosh(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix* inMat = matList[0];
-	
+
 	int max = inMat->cols * inMat->rows;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < max; j++) {
 		data[j] = acosh(data[j]);
 	}
-	
+
 }
 
 static void omxElementAtanh(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix* inMat = matList[0];
-	
+
 	int max = inMat->cols * inMat->rows;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < max; j++) {
 		data[j] = atanh(data[j]);
 	}
-	
+
 }
 
 static void omxElementExponent(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
@@ -1347,11 +1347,11 @@ static void omxElementPtoZ(FitContext *fc, omxMatrix** matList, int numArgs, omx
 static void omxElementLogPtoZ(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix *inMat = matList[0];
-	
+
 	int max = inMat->cols * inMat->rows;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < max; j++) {
 		data[j] = Rf_qnorm5(data[j], 0, 1, 1, 1);
@@ -1375,11 +1375,11 @@ static void omxElementLgamma(FitContext *fc, omxMatrix** matList, int numArgs, o
 static void omxElementLgamma1p(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix *inMat = matList[0];
-	
+
 	int max = inMat->cols * inMat->rows;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < max; j++) {
 		data[j] = Rf_lgamma1p(data[j]);
@@ -1393,21 +1393,21 @@ static void omxElementDbeta(FitContext *fc, omxMatrix** matList, int numArgs, om
 	omxMatrix *b = matList[2];
 	omxMatrix *ncp = matList[3];
 	omxMatrix *give_log = matList[4];
-	
+
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(a);
 	omxEnsureColumnMajor(b);
 	omxEnsureColumnMajor(ncp);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int aDataSize = a->rows * a->cols;
 	int bDataSize = b->rows * b->cols;
 	int ncpDataSize = ncp->rows * ncp->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		if( Rf_sign(ncp->data[j%ncpDataSize]) == -1 ){
@@ -1427,22 +1427,22 @@ static void omxElementPbeta(FitContext *fc, omxMatrix** matList, int numArgs, om
 	omxMatrix *ncp = matList[3];
 	omxMatrix *lower_tail = matList[4];
 	omxMatrix *give_log = matList[5];
-	
+
 	int lower_tail_arg = (int)(lower_tail->data[0] != 0);
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(a);
 	omxEnsureColumnMajor(b);
 	omxEnsureColumnMajor(ncp);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int aDataSize = a->rows * a->cols;
 	int bDataSize = b->rows * b->cols;
 	int ncpDataSize = ncp->rows * ncp->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		if( Rf_sign(ncp->data[j%ncpDataSize]) == -1 ){
@@ -1459,17 +1459,17 @@ static void omxElementDpois(FitContext *fc, omxMatrix** matList, int numArgs, om
 	omxMatrix *inMat = matList[0];
 	omxMatrix *lambda = matList[1];
 	omxMatrix *give_log = matList[2];
-	
+
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(lambda);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int lambdaDataSize = lambda->rows * lambda->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		data[j] = Rf_dpois(data[j],lambda->data[j%lambdaDataSize],give_log_arg);
@@ -1482,18 +1482,18 @@ static void omxElementPpois(FitContext *fc, omxMatrix** matList, int numArgs, om
 	omxMatrix *lambda = matList[1];
 	omxMatrix *lower_tail = matList[2];
 	omxMatrix *give_log = matList[3];
-	
+
 	int lower_tail_arg = (int)(lower_tail->data[0] != 0);
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(lambda);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int lambdaDataSize = lambda->rows * lambda->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		data[j] = Rf_ppois(data[j],lambda->data[j%lambdaDataSize],lower_tail_arg,give_log_arg);
@@ -1507,25 +1507,25 @@ static void omxElementDnbinom(FitContext *fc, omxMatrix** matList, int numArgs,
 	omxMatrix *prob = matList[2];
 	omxMatrix *mu = matList[3];
 	omxMatrix *give_log = matList[4];
-	
+
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(size);
 	omxEnsureColumnMajor(prob);
 	omxEnsureColumnMajor(mu);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int sizeDataSize = size->rows * size->cols;
 	int probDataSize = prob->rows * prob->cols;
 	int muDataSize = mu->rows * mu->cols;
 	int isSizeNeg=0, isProbNeg=0, isMuNeg=0, jumpVal=0;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	double sizecurr=0, probcurr=0, mucurr=0;
-	
+
 	for(int j = 0; j < inMatDataSize; j++) {
 		sizecurr = size->data[j%sizeDataSize];
 		probcurr = prob->data[j%probDataSize];
@@ -1559,26 +1559,26 @@ static void omxElementPnbinom(FitContext *fc, omxMatrix** matList, int numArgs,
 	omxMatrix *mu = matList[3];
 	omxMatrix *lower_tail = matList[4];
 	omxMatrix *give_log = matList[5];
-	
+
 	int lower_tail_arg = (int)(lower_tail->data[0] != 0);
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(size);
 	omxEnsureColumnMajor(prob);
 	omxEnsureColumnMajor(mu);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int sizeDataSize = size->rows * size->cols;
 	int probDataSize = prob->rows * prob->cols;
 	int muDataSize = mu->rows * mu->cols;
 	int isSizeNeg=0, isProbNeg=0, isMuNeg=0, jumpVal=0;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	double sizecurr=0, probcurr=0, mucurr=0;
-	
+
 	for(int j = 0; j < inMatDataSize; j++) {
 		sizecurr = size->data[j%sizeDataSize];
 		probcurr = prob->data[j%probDataSize];
@@ -1610,19 +1610,19 @@ static void omxElementDchisq(FitContext *fc, omxMatrix** matList, int numArgs, o
 	omxMatrix *df = matList[1];
 	omxMatrix *ncp = matList[2];
 	omxMatrix *give_log = matList[3];
-	
+
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(df);
 	omxEnsureColumnMajor(ncp);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int dfDataSize = df->rows * df->cols;
 	int ncpDataSize = ncp->rows * ncp->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		if( Rf_sign(ncp->data[j%ncpDataSize]) == -1 ){
@@ -1641,20 +1641,20 @@ static void omxElementPchisq(FitContext *fc, omxMatrix** matList, int numArgs, o
 	omxMatrix *ncp = matList[2];
 	omxMatrix *lower_tail = matList[3];
 	omxMatrix *give_log = matList[4];
-	
+
 	int lower_tail_arg = (int)(lower_tail->data[0] != 0);
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(df);
 	omxEnsureColumnMajor(ncp);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int dfDataSize = df->rows * df->cols;
 	int ncpDataSize = ncp->rows * ncp->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		if( Rf_sign(ncp->data[j%ncpDataSize]) == -1 ){
@@ -1672,19 +1672,19 @@ static void omxElementDbinom(FitContext *fc, omxMatrix** matList, int numArgs, o
 	omxMatrix *n = matList[1];
 	omxMatrix *p = matList[2];
 	omxMatrix *give_log = matList[3];
-	
+
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(n);
 	omxEnsureColumnMajor(p);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int nDataSize = n->rows * n->cols;
 	int pDataSize = p->rows * p->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		data[j] = Rf_dbinom(data[j],n->data[j%nDataSize],p->data[j%pDataSize],give_log_arg);
@@ -1698,20 +1698,20 @@ static void omxElementPbinom(FitContext *fc, omxMatrix** matList, int numArgs, o
 	omxMatrix *p = matList[2];
 	omxMatrix *lower_tail = matList[3];
 	omxMatrix *give_log = matList[4];
-	
+
 	int lower_tail_arg = (int)(lower_tail->data[0] != 0);
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(n);
 	omxEnsureColumnMajor(p);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int nDataSize = n->rows * n->cols;
 	int pDataSize = p->rows * p->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		data[j] = Rf_pbinom(data[j],n->data[j%nDataSize],p->data[j%pDataSize],lower_tail_arg,give_log_arg);
@@ -1724,19 +1724,19 @@ static void omxElementDcauchy(FitContext *fc, omxMatrix** matList, int numArgs,
 	omxMatrix *location = matList[1];
 	omxMatrix *scale = matList[2];
 	omxMatrix *give_log = matList[3];
-	
+
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(location);
 	omxEnsureColumnMajor(scale);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int locationDataSize = location->rows * location->cols;
 	int scaleDataSize = scale->rows * scale->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		data[j] = Rf_dcauchy(data[j],location->data[j%locationDataSize],scale->data[j%scaleDataSize],give_log_arg);
@@ -1750,20 +1750,20 @@ static void omxElementPcauchy(FitContext *fc, omxMatrix** matList, int numArgs,
 	omxMatrix *scale = matList[2];
 	omxMatrix *lower_tail = matList[3];
 	omxMatrix *give_log = matList[4];
-	
+
 	int lower_tail_arg = (int)(lower_tail->data[0] != 0);
 	int give_log_arg = (int)(give_log->data[0] != 0);
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(location);
 	omxEnsureColumnMajor(scale);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int locationDataSize = location->rows * location->cols;
 	int scaleDataSize = scale->rows * scale->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		data[j] = Rf_pcauchy(data[j],location->data[j%locationDataSize],scale->data[j%scaleDataSize],lower_tail_arg,give_log_arg);
@@ -1775,18 +1775,18 @@ static void omxElementBesselI(FitContext *fc, omxMatrix** matList, int numArgs,
 	omxMatrix *inMat = matList[0];
 	omxMatrix *nu = matList[1];
 	omxMatrix *expo = matList[2];
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(nu);
 	omxEnsureColumnMajor(expo);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int nuDataSize = nu->rows * nu->cols;
 	int expoDataSize = expo->rows * expo->cols;
 	double expocurr;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		expocurr = (expo->data[j%expoDataSize] != 0) ? 2 : 1;
@@ -1798,15 +1798,15 @@ static void omxElementBesselJ(FitContext *fc, omxMatrix** matList, int numArgs,
 {
 	omxMatrix *inMat = matList[0];
 	omxMatrix *nu = matList[1];
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(nu);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int nuDataSize = nu->rows * nu->cols;
 
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		data[j] = Rf_bessel_j(data[j],nu->data[j%nuDataSize]);
@@ -1818,18 +1818,18 @@ static void omxElementBesselK(FitContext *fc, omxMatrix** matList, int numArgs,
 	omxMatrix *inMat = matList[0];
 	omxMatrix *nu = matList[1];
 	omxMatrix *expo = matList[2];
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(nu);
 	omxEnsureColumnMajor(expo);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int nuDataSize = nu->rows * nu->cols;
 	int expoDataSize = expo->rows * expo->cols;
 	double expocurr;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		expocurr = (expo->data[j%expoDataSize] != 0) ? 2 : 1;
@@ -1841,15 +1841,15 @@ static void omxElementBesselY(FitContext *fc, omxMatrix** matList, int numArgs,
 {
 	omxMatrix *inMat = matList[0];
 	omxMatrix *nu = matList[1];
-	
+
 	omxEnsureColumnMajor(inMat);
 	omxEnsureColumnMajor(nu);
-	
+
 	int inMatDataSize = inMat->rows * inMat->cols;
 	int nuDataSize = nu->rows * nu->cols;
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
 	double* data = result->data;
 	for(int j = 0; j < inMatDataSize; j++) {
 		data[j] = Rf_bessel_y(data[j],nu->data[j%nuDataSize]);
@@ -1907,13 +1907,7 @@ static void omxMatrixVechs(FitContext *fc, omxMatrix** matList, int numArgs, omx
 		}
 	}
 
-	if(counter != size) {
-		char *errstr = (char*) calloc(250, sizeof(char));
-		sprintf(errstr, "Internal stop in vechs().\n");
-		omxRaiseError(errstr);
-		free(errstr);
-	}
-
+	if(counter != size) mxThrow("Internal error in vechs()");
 }
 
 static void omxRowVectorize(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
@@ -2208,7 +2202,7 @@ static void omxRealEigenvalues(FitContext *fc, omxMatrix** matList, int numArgs,
 {
 	omxMatrix* B = matList[0];
 
-	if(B->cols != B->rows) stop("Non-square matrix '%s' in eigenvalue decomposition", B->name());
+	if(B->cols != B->rows) mxThrow("Non-square matrix '%s' in eigenvalue decomposition", B->name());
 
 	if(result->rows != B->rows || result->cols != 1)
 		omxResizeMatrix(result, B->rows, 1);
@@ -2226,7 +2220,7 @@ static void omxRealEigenvectors(FitContext *fc, omxMatrix** matList, int numArgs
 {
 	omxMatrix* B = matList[0];
 
-	if(B->cols != B->rows) stop("Non-square matrix '%s' in eigenvalue decomposition", B->name());
+	if(B->cols != B->rows) mxThrow("Non-square matrix '%s' in eigenvalue decomposition", B->name());
 
 	omxResizeMatrix(result, B->rows, B->cols);
 
@@ -2243,7 +2237,7 @@ static void omxImaginaryEigenvalues(FitContext *fc, omxMatrix** matList, int num
 {
 	omxMatrix* B = matList[0];
 
-	if(B->cols != B->rows) stop("Non-square matrix '%s' in eigenvalue decomposition", B->name());
+	if(B->cols != B->rows) mxThrow("Non-square matrix '%s' in eigenvalue decomposition", B->name());
 
 	if(result->rows != B->rows || result->cols != 1)
 		omxResizeMatrix(result, B->rows, 1);
@@ -2261,7 +2255,7 @@ static void omxImaginaryEigenvectors(FitContext *fc, omxMatrix** matList, int nu
 {
 	omxMatrix* B = matList[0];
 
-	if(B->cols != B->rows) stop("Non-square matrix '%s' in eigenvalue decomposition", B->name());
+	if(B->cols != B->rows) mxThrow("Non-square matrix '%s' in eigenvalue decomposition", B->name());
 
 	omxResizeMatrix(result, B->rows, B->cols);
 
@@ -2282,7 +2276,7 @@ static void omxSelectRows(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 	int rows = inMat->rows;
     int selectLength = selector->rows * selector->cols;
     Eigen::VectorXi toRemove(rows);
-    
+
     if((selector->cols != 1) && selector->rows !=1) {
 		char *errstr = (char*) calloc(250, sizeof(char));
 		sprintf(errstr, "Selector must have a single row or a single column.\n");
@@ -2298,9 +2292,9 @@ static void omxSelectRows(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 		free(errstr);
 		return;
 	}
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
     for(int index = 0; index < selectLength; index++) {
         if(omxVectorElement(selector, index) == 0) {
             toRemove[index] = 1;
@@ -2308,7 +2302,7 @@ static void omxSelectRows(FitContext *fc, omxMatrix** matList, int numArgs, omxM
             toRemove[index] = 0;
         }
     }
-    
+
     std::vector<int> zeros(inMat->cols);
     omxRemoveRowsAndColumns(result, toRemove.data(), zeros.data());
 
@@ -2327,7 +2321,7 @@ static void omxSelectCols(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 		char *errstr = (char*) calloc(250, sizeof(char));
 		sprintf(errstr, "Selector must have a single row or a single column.\n");
         omxRaiseError(errstr);
-		free(errstr);        
+		free(errstr);
 		return;
     }
 
@@ -2338,9 +2332,9 @@ static void omxSelectCols(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 		free(errstr);
 		return;
 	}
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
     for(int index = 0; index < selectLength; index++) {
         if(omxVectorElement(selector, index) == 0) {
             toRemove[index] = 1;
@@ -2348,7 +2342,7 @@ static void omxSelectCols(FitContext *fc, omxMatrix** matList, int numArgs, omxM
             toRemove[index] = 0;
         }
     }
-    
+
     std::vector<int> zeros(inMat->rows);
     omxRemoveRowsAndColumns(result, zeros.data(), toRemove.data());
 }
@@ -2367,7 +2361,7 @@ static void omxSelectRowsAndCols(FitContext *fc, omxMatrix** matList, int numArg
 		char *errstr = (char*) calloc(250, sizeof(char));
 		sprintf(errstr, "Selector must have a single row or a single column.\n");
         omxRaiseError(errstr);
-		free(errstr);        
+		free(errstr);
 		return;
     }
 
@@ -2386,9 +2380,9 @@ static void omxSelectRowsAndCols(FitContext *fc, omxMatrix** matList, int numArg
 		free(errstr);
 		return;
 	}
-	
+
 	omxCopyMatrix(result, inMat);
-	
+
     for(int index = 0; index < selectLength; index++) {
         if(omxVectorElement(selector, index) == 0) {
             toRemove[index] = 1;
@@ -2396,7 +2390,7 @@ static void omxSelectRowsAndCols(FitContext *fc, omxMatrix** matList, int numArg
             toRemove[index] = 0;
         }
     }
-    
+
     omxRemoveRowsAndColumns(result, toRemove.data(), toRemove.data());
 }
 
@@ -2429,14 +2423,14 @@ static void omxCovToCor(FitContext *fc, omxMatrix** matList, int numArgs, omxMat
     if (inMat->colMajor) {
         for(int col = 0; col < rows; col++) {
             for(int row = 0; row < rows; row++) {
-                result->data[col * rows + row] = inMat->data[col * rows + row] * 
+                result->data[col * rows + row] = inMat->data[col * rows + row] *
                     intermediate->data[row] * intermediate->data[col];
             }
         }
     } else {
         for(int col = 0; col < rows; col++) {
             for(int row = 0; row < rows; row++) {
-                result->data[col * rows + row] = inMat->data[row * rows + col] * 
+                result->data[col * rows + row] = inMat->data[row * rows + col] *
                     intermediate->data[row] * intermediate->data[col];
             }
         }
@@ -2453,14 +2447,14 @@ static void omxCholesky(FitContext *fc, omxMatrix** matList, int numArgs, omxMat
 {
 	omxMatrix* inMat = matList[0];
 	if(inMat->rows != inMat->cols) {
-		stop("Cholesky decomposition of non-square matrix '%s' is not defined", inMat->name());
+		mxThrow("Cholesky decomposition of non-square matrix '%s' is not defined", inMat->name());
 	}
 	omxCopyMatrix(result, inMat);
 	EigenMatrixAdaptor Ei(result);
 
 	Eigen::LLT< Eigen::Ref<Eigen::MatrixXd>, Eigen::Upper > sc(Ei);
 	if (sc.info() != Eigen::Success) {
-		stop("Cholesky factor of '%s' failed", inMat->name());
+		mxThrow("Cholesky factor of '%s' failed", inMat->name());
 	}
 	Ei.triangularView<Eigen::StrictlyLower>().setZero();
 }
@@ -2468,7 +2462,7 @@ static void omxCholesky(FitContext *fc, omxMatrix** matList, int numArgs, omxMat
 static void omxVechToMatrix(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix *inMat = matList[0];
-	
+
 	int dim = (inMat->cols > inMat->rows) ? inMat->cols : inMat->rows;
 
 	int size = sqrt(2.0 * dim + 0.25) - 0.5;
@@ -2506,9 +2500,9 @@ static void omxVechToMatrix(FitContext *fc, omxMatrix** matList, int numArgs, om
 static void omxVechsToMatrix(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix *inMat = matList[0];
-	
+
 	int dim = (inMat->cols > inMat->rows) ? inMat->cols : inMat->rows;
-	
+
 	int size = sqrt(2.0 * dim + 0.25) + 0.5; //note the plus 0.5
 
 	int counter = 0;
@@ -2552,7 +2546,7 @@ static void omxExponential(FitContext *fc, omxMatrix** matList, int numArgs, omx
 	}
 
 	omxMatrix* inMat = matList[0];
-	if (inMat->rows != inMat->cols) stop("omxExponential requires a symmetric matrix");
+	if (inMat->rows != inMat->cols) mxThrow("omxExponential requires a symmetric matrix");
 	omxEnsureColumnMajor(inMat);
 	omxResizeMatrix(result, inMat->rows, inMat->cols);
 	result->colMajor = true;
@@ -2563,7 +2557,7 @@ static void omxExponential(FitContext *fc, omxMatrix** matList, int numArgs, omx
 static void mxMatrixLog(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	omxMatrix* inMat = matList[0];
-	if (inMat->rows != inMat->cols) stop("logm requires a symmetric matrix");
+	if (inMat->rows != inMat->cols) mxThrow("logm requires a symmetric matrix");
 	omxEnsureColumnMajor(inMat);
 	omxResizeMatrix(result, inMat->rows, inMat->cols);
 	result->colMajor = true;
@@ -2613,7 +2607,8 @@ static void evaluateOnGrid(FitContext *fc, omxMatrix** matList, int numArgs, omx
 			return;
 		}
 		abscissaParamMap[rx] = got;
-		fc->profiledOut[got] = true;
+		fc->profiledOutZ[got] = true;
+    fc->calcNumFree();
 	}
 
 	omxState *os = result->currentState;
@@ -2641,91 +2636,42 @@ static void evaluateOnGrid(FitContext *fc, omxMatrix** matList, int numArgs, omx
 	}
 }
 
-template <typename T1, typename T2, typename T3, typename T4, typename T5>
-void partitionCovariance(const Eigen::MatrixBase<T1> &gcov,
-		     T2 filterTest,
-		     Eigen::MatrixBase<T3> &v11,
-		     Eigen::MatrixBase<T4> &v12,
-		     Eigen::MatrixBase<T5> &v22)
-{
-	for (int gcx=0, c1=0,c2=0,c3=0; gcx < gcov.cols(); gcx++) {
-		for (int grx=0, r1=0,r2=0,r3=0; grx < gcov.rows(); grx++) {
-			if (filterTest(grx)) {
-				if (filterTest(gcx)) {
-					v11(r1++, c1) = gcov(grx, gcx);
-				} else {
-					v12(r2++, c2) = gcov(grx, gcx);
-				}
-			} else {
-				if (!filterTest(gcx)) {
-					v22(r3++, c3) = gcov(grx, gcx);
-				}
-			}
-		}
-		if (filterTest(gcx)) {
-			c1 += 1;
-		} else {
-			c2 += 1;
-			c3 += 1;
-		}
-	}
-}
-
-template <typename T1, typename T2, typename T3, typename T4, typename T5>
-void partitionCovarianceSet(Eigen::MatrixBase<T1> &gcov,
-			    T2 filterTest,
-			    const Eigen::MatrixBase<T3> &v11,
-			    const Eigen::MatrixBase<T4> &v12,
-			    const Eigen::MatrixBase<T5> &v22)
-{
-	for (int gcx=0, c1=0,c2=0,c3=0,c4=0; gcx < gcov.cols(); gcx++) {
-		for (int grx=0, r1=0,r2=0,r3=0,r4=0; grx < gcov.rows(); grx++) {
-			if (filterTest(grx)) {
-				if (filterTest(gcx)) {
-					gcov(grx, gcx) = v11(r1++, c1);
-				} else {
-					gcov(grx, gcx) = v12(r2++, c2);
-				}
-			} else {
-				if (!filterTest(gcx)) {
-					gcov(grx, gcx) = v22(r3++, c3);
-				} else {
-					gcov(grx, gcx) = v12(c4, r4++);
-				}
-			}
-		}
-		if (filterTest(gcx)) {
-			c1 += 1;
-			c4 += 1;
-		} else {
-			c2 += 1;
-			c3 += 1;
-		}
-	}
-}
-
-template <typename T> 
+template <typename T>
 void buildFilterVec(omxMatrix *origCov, omxMatrix *newCov, std::vector<T> &filter)
 {
-	if (origCov->rows != origCov->cols) stop("'%s' must be square", origCov->name());
-	if (origCov->rows != int(origCov->rownames.size())) stop("'%s' must have dimnames", origCov->name());
-	if (newCov->rows != newCov->cols) stop("'%s' must be square", newCov->name());
-	if (newCov->rows != int(newCov->rownames.size())) stop("'%s' must have dimnames", newCov->name());
-	for (int r1=0; r1 < int(newCov->rownames.size()); ++r1) {
-		bool found = false;
-		for (int r2=0; r2 < int(origCov->rownames.size()); ++r2) {
-			if (strEQ(newCov->rownames[r1], origCov->rownames[r2])) {
-				if (filter[r2]) {
-					omxRaiseErrorf("Cannot filter row '%s' in '%s' more than once",
-						       newCov->rownames[r1], origCov->name());
+	if (origCov->rows != origCov->cols) mxThrow("'%s' must be square", origCov->name());
+	if (newCov->rows != newCov->cols) mxThrow("'%s' must be square", newCov->name());
+
+	if (newCov->sameSize(origCov)) {
+		EigenMatrixAdaptor EorigCov(origCov);
+		EigenMatrixAdaptor EnewCov(newCov);
+		Eigen::Array<bool, Eigen::Dynamic, Eigen::Dynamic> NotMatch =
+			EorigCov.array() != EnewCov.array();
+		Eigen::Array<bool, Eigen::Dynamic, 1> mask =
+			NotMatch.rowwise().maxCoeff() || NotMatch.colwise().maxCoeff().transpose();
+		std::copy(&mask.coeffRef(0), &mask.coeffRef(mask.size()), filter.begin());
+	} else {
+		if (origCov->rows != int(origCov->rownames.size()))
+			mxThrow("'%s' must have dimnames", origCov->name());
+		if (newCov->rows != int(newCov->rownames.size()))
+			mxThrow("'%s' must have dimnames", newCov->name());
+
+		for (int r1=0; r1 < int(newCov->rownames.size()); ++r1) {
+			bool found = false;
+			for (int r2=0; r2 < int(origCov->rownames.size()); ++r2) {
+				if (strEQ(newCov->rownames[r1], origCov->rownames[r2])) {
+					if (filter[r2]) {
+						omxRaiseErrorf("Cannot filter row '%s' in '%s' more than once",
+													 newCov->rownames[r1], origCov->name());
+					}
+					filter[r2] = true;
+					found = true;
+					break;
 				}
-				filter[r2] = true;
-				found = true;
-				break;
 			}
+			if (!found) omxRaiseErrorf("Cannot find row '%s' in '%s'",
+																 newCov->rownames[r1], origCov->name());
 		}
-		if (!found) omxRaiseErrorf("Cannot find row '%s' in '%s'",
-				     newCov->rownames[r1], origCov->name());
 	}
 }
 
@@ -2734,19 +2680,31 @@ static void pearsonSelCov(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 	omxMatrix *origCov = matList[0];
 	omxMatrix *newCov = matList[1];
 	EigenMatrixAdaptor EorigCov(origCov);
-	EigenMatrixAdaptor EnewCov(newCov);
 
 	omxResizeMatrix(result, origCov->rows, origCov->cols);
 	EigenMatrixAdaptor Eresult(result);
-	
+
 	// cache this in the result matrix somehow? TODO
 	std::vector<bool> filter(origCov->rows, false);
 	buildFilterVec(origCov, newCov, filter);
 	if (isErrorRaised()) return;
 
-	Eigen::MatrixXd v11(EnewCov.rows(), EnewCov.cols());
-	Eigen::MatrixXd v12(EnewCov.rows(), EorigCov.cols() - EnewCov.cols());
-	Eigen::MatrixXd v22(EorigCov.rows() - EnewCov.rows(), EorigCov.cols() - EnewCov.cols());
+	EigenMatrixAdaptor EnewCov(newCov);
+	Eigen::MatrixXd nc;
+	if (newCov->sameSize(origCov)) {
+		subsetCovariance(EnewCov, [&filter](int x)->bool{ return filter[x]; },
+										 std::accumulate(filter.begin(), filter.end(), 0), nc);
+	} else {
+		nc = EnewCov;
+	}
+	if (nc.rows() == 0) {
+		Eresult.derived() = EorigCov;
+		return;
+	}
+
+	Eigen::MatrixXd v11(nc.rows(), nc.cols());
+	Eigen::MatrixXd v12(nc.rows(), EorigCov.cols() - nc.cols());
+	Eigen::MatrixXd v22(EorigCov.rows() - nc.rows(), EorigCov.cols() - nc.cols());
 	partitionCovariance(EorigCov, [&](int xx){ return filter[xx]; },
 			    v11, v12, v22);
 
@@ -2756,11 +2714,11 @@ static void pearsonSelCov(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 		return;
 	}
 	iv11 = iv11.selfadjointView<Eigen::Lower>();
-	Eigen::MatrixXd n12 = EnewCov * iv11 * v12;
-	Eigen::MatrixXd n22 = v22 - v12.transpose() * (iv11 - iv11 * EnewCov * iv11) * v12;
-	
+	Eigen::MatrixXd n12 = nc * iv11 * v12;
+	Eigen::MatrixXd n22 = v22 - v12.transpose() * (iv11 - iv11 * nc * iv11) * v12;
+
 	partitionCovarianceSet(Eresult, [&](int xx){ return filter[xx]; },
-			       EnewCov, n12, n22);
+			       nc, n12, n22);
 }
 
 static void pearsonSelMean(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
@@ -2768,13 +2726,12 @@ static void pearsonSelMean(FitContext *fc, omxMatrix** matList, int numArgs, omx
 	omxMatrix *origCov = matList[0];
 	omxMatrix *newCov = matList[1];
 	omxMatrix *origMean = matList[2];
-	if (origMean->cols > 1) stop("'%s' must be a column vector", origMean->name());
+	if (origMean->cols > 1) mxThrow("'%s' must be a column vector", origMean->name());
 	if (origMean->rows != origCov->rows) {
-		stop("'%s' of dimension %d must have same dimension as '%s' (%d)",
+		mxThrow("'%s' of dimension %d must have same dimension as '%s' (%d)",
 			 origMean->name(), origMean->rows, origCov->name(), origCov->rows);
 	}
 	EigenMatrixAdaptor EorigCov(origCov);
-	EigenMatrixAdaptor EnewCov(newCov);
 	EigenVectorAdaptor EorigMean(origMean);
 
 	omxResizeMatrix(result, EorigMean.size(), 1);
@@ -2785,9 +2742,22 @@ static void pearsonSelMean(FitContext *fc, omxMatrix** matList, int numArgs, omx
 	buildFilterVec(origCov, newCov, filter);
 	if (isErrorRaised()) return;
 
-	Eigen::MatrixXd v11(EnewCov.rows(), EnewCov.cols());
-	Eigen::MatrixXd v12(EnewCov.rows(), EorigCov.cols() - EnewCov.cols());
-	Eigen::MatrixXd v22(EorigCov.rows() - EnewCov.rows(), EorigCov.cols() - EnewCov.cols());
+	EigenMatrixAdaptor EnewCov(newCov);
+	Eigen::MatrixXd nc;
+	if (newCov->sameSize(origCov)) {
+		subsetCovariance(EnewCov, [&filter](int x)->bool{ return filter[x]; },
+										 std::accumulate(filter.begin(), filter.end(), 0), nc);
+	} else {
+		nc = EnewCov;
+	}
+	if (nc.rows() == 0) {
+		Eresult.derived() = EorigMean;
+		return;
+	}
+
+	Eigen::MatrixXd v11(nc.rows(), nc.cols());
+	Eigen::MatrixXd v12(nc.rows(), EorigCov.cols() - nc.cols());
+	Eigen::MatrixXd v22(EorigCov.rows() - nc.rows(), EorigCov.cols() - nc.cols());
 	partitionCovariance(EorigCov, [&](int xx){ return filter[xx]; },
 			    v11, v12, v22);
 
@@ -2797,10 +2767,10 @@ static void pearsonSelMean(FitContext *fc, omxMatrix** matList, int numArgs, omx
 		return;
 	}
 	iv11 = iv11.selfadjointView<Eigen::Lower>();
-	
+
 	Eigen::VectorXd obsMean;
 	subsetVector(EorigMean, [&](int xx){ return filter[xx]; },
-		     EnewCov.rows(), obsMean);
+		     nc.rows(), obsMean);
 
 	Eigen::VectorXd adj = v12.transpose() * iv11 * obsMean;
 	for (int v1=0, a1=0; v1 < EorigMean.rows(); ++v1) {
