@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2020 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -51,96 +51,96 @@ omxComputeNM::omxComputeNM()
 
 void omxComputeNM::initFromFrontend(omxState *globalState, SEXP rObj){
 	super::initFromFrontend(globalState, rObj);
-	
+
 	//TODO: use const defined in Rmath.h:
 	const double myPI	=	3.141592653589793238462643383280;
-	
+
 	SEXP slotValue;
 	fitMatrix = omxNewMatrixFromSlot(rObj, globalState, "fitfunction");
 	omxCompleteFitFunction(fitMatrix);
-	
+
 	ScopedProtect p1(slotValue, R_do_slot(rObj, Rf_install("verbose")));
 	verbose = Rf_asInteger(slotValue);
 	if(OMX_DEBUG){
 		mxLog("omxComputeNM member 'verbose' is %d", verbose);
 	}
-	
+
 	ScopedProtect p2(slotValue, R_do_slot(rObj, Rf_install("nudgeZeroStarts")));
 	nudge = Rf_asLogical(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'nudgeZeroStarts' is %d", nudge);
 	}
-	
+
 	ScopedProtect p3(slotValue, R_do_slot(rObj, Rf_install("defaultMaxIter")));
 	defaultMaxIter = Rf_asLogical(slotValue);
-	
+
 	ScopedProtect p4(slotValue, R_do_slot(rObj, Rf_install("maxIter")));
 	if(defaultMaxIter){maxIter = Global->majorIterations * 10;}
 	else{maxIter = Rf_asInteger(slotValue);}
 	if(verbose){
 		mxLog("omxComputeNM member 'maxIter' is %d", maxIter);
 	}
-	
+
 	ScopedProtect p5(slotValue, R_do_slot(rObj, Rf_install("alpha")));
 	alpha = Rf_asReal(slotValue);
-	if(alpha<=0){stop("reflection coefficient 'alpha' must be positive");}
+	if(alpha<=0){mxThrow("reflection coefficient 'alpha' must be positive");}
 	if(verbose){
 		mxLog("omxComputeNM member 'alpha' is %f", alpha);
 	}
-	
+
 	ScopedProtect p6(slotValue, R_do_slot(rObj, Rf_install("betao")));
 	betao = Rf_asReal(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'betao' is %f", betao);
 	}
-	
+
 	ScopedProtect p7(slotValue, R_do_slot(rObj, Rf_install("betai")));
 	betai = Rf_asReal(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'betai' is %f", betai);
 	}
 	if(betao<=0 || betao>=1 || betai<=0 || betai>=1){
-		stop("contraction coefficients 'betao' and 'betai' must both be within unit interval (0,1)");
+		mxThrow("contraction coefficients 'betao' and 'betai' must both be within unit interval (0,1)");
 	}
-	
+
 	ScopedProtect p8(slotValue, R_do_slot(rObj, Rf_install("gamma")));
 	gamma = Rf_asReal(slotValue);
 	if(gamma>0 && gamma<=alpha){
-		stop("if positive, expansion coefficient 'gamma' must be greater than reflection coefficient 'alpha'");
+		mxThrow("if positive, expansion coefficient 'gamma' must be greater than reflection coefficient 'alpha'");
 	}
 	if(verbose){
 		mxLog("omxComputeNM member 'gamma' is %f", gamma);
 	}
-	
+
 	ScopedProtect p9(slotValue, R_do_slot(rObj, Rf_install("sigma")));
 	sigma = Rf_asReal(slotValue);
-	if(sigma>=1){stop("shrink coefficient 'sigma' must be less than 1.0");}
+	if(sigma>=1){mxThrow("shrink coefficient 'sigma' must be less than 1.0");}
 	if(verbose){
 		mxLog("omxComputeNM member 'sigma' is %f", sigma);
 	}
-	
+
 	ScopedProtect p10(slotValue, R_do_slot(rObj, Rf_install("bignum")));
 	bignum = Rf_asReal(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'bignum' is %f", bignum);
 	}
-	
+
 	ScopedProtect p11(slotValue, R_do_slot(rObj, Rf_install("iniSimplexType")));
 	if(strEQ(CHAR(Rf_asChar(slotValue)),"regular")){iniSimplexType = 1;}
 	else if(strEQ(CHAR(Rf_asChar(slotValue)),"right")){iniSimplexType = 2;}
 	else if(strEQ(CHAR(Rf_asChar(slotValue)),"smartRight")){iniSimplexType = 3;}
 	else if(strEQ(CHAR(Rf_asChar(slotValue)),"random")){iniSimplexType = 4;}
-	else{stop("unrecognized character string provided for Nelder-Mead 'iniSimplexType'");}
+	else{mxThrow("unrecognized character string provided for Nelder-Mead 'iniSimplexType'");}
 	if(verbose){
 		mxLog("omxComputeNM member 'iniSimplexType' is %d", iniSimplexType);
 	}
-	
+
 	ScopedProtect p12(slotValue, R_do_slot(rObj, Rf_install("iniSimplexEdge")));
 	iniSimplexEdge = Rf_asReal(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'iniSimplexEdge' is %f", iniSimplexEdge);
 	}
-	
+
 	ScopedProtect p13(slotValue, R_do_slot(rObj, Rf_install("iniSimplexMat")));
 	if (Rf_length(slotValue)) {
 		SEXP matrixDims;
@@ -150,7 +150,7 @@ void omxComputeNM::initFromFrontend(omxState *globalState, SEXP rObj){
 		int cols = dimList[1];
 		iniSimplexMat = Eigen::Map< Eigen::MatrixXd >(REAL(slotValue), rows, cols);
 	}
-	
+
 	ScopedProtect p26(slotValue, R_do_slot(rObj, Rf_install(".iniSimplexColnames")));
 	int cnameslen = Rf_length(slotValue);
 	if(cnameslen){
@@ -164,82 +164,82 @@ void omxComputeNM::initFromFrontend(omxState *globalState, SEXP rObj){
 			}
 		}
 	}
-	
+
 	ScopedProtect p14(slotValue, R_do_slot(rObj, Rf_install("greedyMinimize")));
 	greedyMinimize = Rf_asLogical(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'greedyMinimize' is %d", greedyMinimize);
 	}
-	
+
 	ScopedProtect p15(slotValue, R_do_slot(rObj, Rf_install("altContraction")));
 	altContraction = Rf_asLogical(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'altContraction' is %d", altContraction);
 	}
-	
+
 	ScopedProtect p16(slotValue, R_do_slot(rObj, Rf_install("degenLimit")));
 	degenLimit = Rf_asReal(slotValue);
-	if(degenLimit<0 || degenLimit>myPI){stop("'degenLimit' must ge within interval [0,pi]");}
+	if(degenLimit<0 || degenLimit>myPI){mxThrow("'degenLimit' must ge within interval [0,pi]");}
 	if(verbose){
 		mxLog("omxComputeNM member 'degenLimit' is %f", degenLimit);
 	}
-	
+
 	ScopedProtect p17(slotValue, R_do_slot(rObj, Rf_install("stagnCtrl")));
-	if(Rf_length(slotValue)!=2){stop("'stagnCtrl' must be an integer vector of length 2");}
+	if(Rf_length(slotValue)!=2){mxThrow("'stagnCtrl' must be an integer vector of length 2");}
 	stagnCtrl[0] = INTEGER(slotValue)[0];
 	stagnCtrl[1] = INTEGER(slotValue)[1];
 	if(verbose){
 		mxPrintMat("omxComputeNM member 'stagnCtrl':", stagnCtrl);
 	}
-	
+
 	ScopedProtect p18(slotValue, R_do_slot(rObj, Rf_install("validationRestart")));
 	validationRestart = Rf_asLogical(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'validationRestart' is %d", validationRestart);
 	}
-	
+
 	ScopedProtect p19(slotValue, R_do_slot(rObj, Rf_install("xTolProx")));
 	xTolProx = Rf_asReal(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'xTolProx' is %f", xTolProx);
 	}
-	
+
 	ScopedProtect p20(slotValue, R_do_slot(rObj, Rf_install("fTolProx")));
 	fTolProx = Rf_asReal(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'fTolProx' is %f", fTolProx);
 	}
-	
+
 	//Prevent user blunder w/r/t convergence criteria:
 	if(xTolProx<=0 && fTolProx<=0){
 		fTolProx = 1e-14;
 		Rf_warning("both 'xTolProx' and 'fTolProx' are non-positive; 'fTolProx' will be assigned a value of 1e-14");
 	}
-	
+
 	ScopedProtect p30(slotValue, R_do_slot(rObj, Rf_install("doPseudoHessian")));
 	doPseudoHessian = Rf_asLogical(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'doPseudoHessian' is %d", doPseudoHessian);
 	}
-	
+
 	ScopedProtect p24(slotValue, R_do_slot(rObj, Rf_install("ineqConstraintMthd")));
 	if(strEQ(CHAR(Rf_asChar(slotValue)),"soft")){ineqConstraintMthd = 0;}
 	else if(strEQ(CHAR(Rf_asChar(slotValue)),"eqMthd")){ineqConstraintMthd = 1;}
-	else{stop("unrecognized character string provided for Nelder-Mead 'ineqConstraintMthd'");}
+	else{mxThrow("unrecognized character string provided for Nelder-Mead 'ineqConstraintMthd'");}
 	if(verbose){
 		mxLog("omxComputeNM member 'ineqConstraintMthd' is %d", ineqConstraintMthd);
 	}
-	
+
 	ScopedProtect p25(slotValue, R_do_slot(rObj, Rf_install("eqConstraintMthd")));
 	if(strEQ(CHAR(Rf_asChar(slotValue)),"soft")){eqConstraintMthd = 1;}
 	else if(strEQ(CHAR(Rf_asChar(slotValue)),"backtrack")){eqConstraintMthd = 2;}
 	else if(strEQ(CHAR(Rf_asChar(slotValue)),"GDsearch")){eqConstraintMthd = 3;}
 	else if(strEQ(CHAR(Rf_asChar(slotValue)),"l1p")){eqConstraintMthd = 4;}
-	else{stop("unrecognized character string provided for Nelder-Mead 'eqConstraintMthd'");}
+	else{mxThrow("unrecognized character string provided for Nelder-Mead 'eqConstraintMthd'");}
 	if(verbose){
 		mxLog("omxComputeNM member 'eqConstraintMthd' is %d", eqConstraintMthd);
 	}
-	
+
 	ScopedProtect p28(slotValue, R_do_slot(rObj, Rf_install("backtrackCtrl1")));
 	backtrackCtrl1 = Rf_asReal(slotValue);
 	if(verbose){
@@ -250,29 +250,28 @@ void omxComputeNM::initFromFrontend(omxState *globalState, SEXP rObj){
 	if(verbose){
 		mxLog("omxComputeNM member 'backtrackCtrl2' is %d", backtrackCtrl2);
 	}
-	
+
 	ScopedProtect p31(slotValue, R_do_slot(rObj, Rf_install("centerIniSimplex")));
 	centerIniSimplex = Rf_asLogical(slotValue);
 	if(verbose){
 		mxLog("omxComputeNM member 'centerIniSimplex' is %d", centerIniSimplex);
 	}
-	
+
 	feasTol = Global->feasibilityTolerance;
 }
 
 
-void omxComputeNM::computeImpl(FitContext *fc){
-	
+void omxComputeNM::computeImpl(FitContext *fc)
+{
 	omxAlgebraPreeval(fitMatrix, fc);
-	if (isErrorRaised()) return;
-	
+
 	fc->ensureParamWithinBox(nudge);
-	fc->createChildren(fitMatrix);
-	
+	fc->createChildren(fitMatrix, true);
+
 	NelderMeadOptimizerContext nmoc(fc, this);
 
 	if (nmoc.numFree <= 0) { complainNoFreeParam(); return; }
-	
+
 	nmoc.verbose = verbose;
 	nmoc.maxIter = maxIter;
 	nmoc.iniSimplexType = iniSimplexType;
@@ -284,7 +283,7 @@ void omxComputeNM::computeImpl(FitContext *fc){
 	nmoc.ineqConstraintMthd = ineqConstraintMthd;
 	nmoc.eqConstraintMthd = eqConstraintMthd;
 	nmoc.countConstraintsAndSetupBounds();
-	if(nmoc.eqConstraintMthd==4 && (nmoc.numEqC || (nmoc.ineqConstraintMthd && nmoc.numIneqC))){
+	if(nmoc.eqConstraintMthd==4 && (nmoc.EqC.getCount() || (nmoc.ineqConstraintMthd && nmoc.IneqC.getCount()))){
 		if(verbose){mxLog("starting l1-penalty algorithm");}
 		fc->iterations = 0; //<--Not sure about this
 		nmoc.maxIter = maxIter/10;
@@ -305,7 +304,7 @@ void omxComputeNM::computeImpl(FitContext *fc){
 					nmoc.iniSimplexEdge = iniSimplexEdge;
 				}
 				else{ //It's making progress w/r/t the constraints, so re-initialize the simplex with a small edge:
-					nmoc.iniSimplexEdge = 
+					nmoc.iniSimplexEdge =
 						sqrt((nmoc.vertices[nmoc.n] - nmoc.vertices[0]).dot(nmoc.vertices[nmoc.n] - nmoc.vertices[0]));
 					//It's a good idea to reduce the penalty coefficient if the algorithm is making progress.
 					//That helps prevent it from stopping at a non-optimal point:
@@ -327,13 +326,13 @@ void omxComputeNM::computeImpl(FitContext *fc){
 		nmoc.invokeNelderMead();
 		fc->iterations = nmoc.itersElapsed;
 	}
-	
+
 	if(validationRestart && nmoc.statuscode==0){
 		NelderMeadOptimizerContext nmoc2(fc, this);
 		nmoc2.verbose = verbose;
 		nmoc2.maxIter = 2 * nmoc.n;
 		nmoc2.iniSimplexType = 1;
-		nmoc2.iniSimplexEdge = 
+		nmoc2.iniSimplexEdge =
 			sqrt((nmoc.vertices[nmoc.n] - nmoc.vertices[0]).dot(nmoc.vertices[nmoc.n] - nmoc.vertices[0]));
 		nmoc2.centerIniSimplex = true;
 		nmoc2.fit2beat = nmoc.bestfit;
@@ -348,7 +347,7 @@ void omxComputeNM::computeImpl(FitContext *fc){
 		if(nmoc2.statuscode==10){
 			fc->resetIterationError();
 		}
-		
+
 		if(nmoc2.bestfit < nmoc.bestfit && (nmoc2.statuscode==0 || nmoc2.statuscode==4)){
 			nmoc.bestfit = nmoc2.bestfit;
 			nmoc.est = nmoc2.est;
@@ -366,20 +365,20 @@ void omxComputeNM::computeImpl(FitContext *fc){
 				nmoc.statuscode = 4;
 			}
 		}
-		
+
 		//Not sure about this:
 		fc->iterations += nmoc2.itersElapsed;
 	}
-	
-	if(doPseudoHessian && (nmoc.statuscode==0 || nmoc.statuscode==4) && !nmoc.vertexInfeas.sum() && !nmoc.numEqC && !nmoc.addPenalty){
+
+	if(doPseudoHessian && (nmoc.statuscode==0 || nmoc.statuscode==4) && !nmoc.vertexInfeas.sum() && !nmoc.EqC.getCount() && !nmoc.addPenalty){
 		nmoc.calculatePseudoHessian();
 	}
-	
+
 	if(nmoc.estInfeas && nmoc.statuscode!=10){nmoc.statuscode = 3;}
-	
+
 	switch(nmoc.statuscode){
 	case -1:
-		stop("unknown Nelder-Mead optimizer error");
+		mxThrow("unknown Nelder-Mead optimizer error");
 		break;
 	case 0:
 		fc->setInform(INFORM_CONVERGED_OPTIMUM);
@@ -394,7 +393,7 @@ void omxComputeNM::computeImpl(FitContext *fc){
 		fc->setInform(INFORM_STARTING_VALUES_INFEASIBLE);
 		break;
 	}
-	
+
 	size_t i=0;
 	Eigen::VectorXd xdiffs(nmoc.n);
 	Eigen::MatrixXd fdiffs(nmoc.n,1);
@@ -410,7 +409,7 @@ void omxComputeNM::computeImpl(FitContext *fc){
 	}
 	fproxOut = fdiffs.array().maxCoeff();
 	for(i=0; i < size_t(nmoc.n); i++){
-		if(!nmoc.numEqC){
+		if(!nmoc.EqC.getCount()){
 			Q.col(i) = verticesOut.row(i+1) - verticesOut.row(0);
 			xdiffs[i] = (Q.col(i)).array().abs().maxCoeff();
 		}
@@ -419,7 +418,7 @@ void omxComputeNM::computeImpl(FitContext *fc){
 		}
 	}
 	xproxOut = xdiffs.array().maxCoeff();
-	if(!nmoc.vertexInfeas.sum() && !nmoc.numEqC && !nmoc.addPenalty){
+	if(!nmoc.vertexInfeas.sum() && !nmoc.EqC.getCount() && !nmoc.addPenalty){
 		Eigen::FullPivLU< Eigen::MatrixXd > luq(Q);
 		if(luq.isInvertible()){
 			Eigen::MatrixXd Qinv(nmoc.n, nmoc.n);
@@ -429,22 +428,22 @@ void omxComputeNM::computeImpl(FitContext *fc){
 			if(verbose){mxPrintMat("simplex gradient: ",simplexGradient);}
 		}
 	}
-	
+
 	nmoc.finalize();
-	
+
 	fc->wanted |= FF_COMPUTE_BESTFIT;
-	
+
 	return;
 }
 
 
 void omxComputeNM::reportResults(FitContext *fc, MxRList *slots, MxRList *out){
 	omxPopulateFitFunction(fitMatrix, out);
-	
+
 	MxRList output;
-	SEXP pn, cn, cr, cc, cv, vrt, fv, vinf, fpm, xpm, phess, sg, bf;
+	SEXP pn, cv, vrt, fv, vinf, fpm, xpm, phess, sg, bf;
 	size_t i=0;
-	
+
 	if( fc->varGroup->vars.size() ){
 		Rf_protect(pn = Rf_allocVector( STRSXP, fc->varGroup->vars.size() ));
 		for(i=0; i < fc->varGroup->vars.size(); i++){
@@ -452,19 +451,7 @@ void omxComputeNM::reportResults(FitContext *fc, MxRList *slots, MxRList *out){
 		}
 		output.add("paramNames", pn);
 	}
-	if( fc->state->conListX.size() ){
-		Rf_protect(cn = Rf_allocVector( STRSXP, fc->state->conListX.size() ));
-		Rf_protect(cr = Rf_allocVector( INTSXP, fc->state->conListX.size() ));
-		Rf_protect(cc = Rf_allocVector( INTSXP, fc->state->conListX.size() ));
-		for(i=0; i < fc->state->conListX.size(); i++){
-			SET_STRING_ELT( cn, i, Rf_mkChar(fc->state->conListX[i]->name) );
-			INTEGER(cr)[i] = fc->state->conListX[i]->nrows;
-			INTEGER(cc)[i] = fc->state->conListX[i]->ncols;
-		}
-		output.add("constraintNames", cn);
-		output.add("constraintRows", cr);
-		output.add("constraintCols", cc);
-	}
+  fc->state->reportConstraints(output);
 	if( fc->constraintFunVals.size() ){
 		Rf_protect(cv = Rf_allocVector( REALSXP, fc->constraintFunVals.size() ));
 		memcpy( REAL(cv), fc->constraintFunVals.data(), sizeof(double) * fc->constraintFunVals.size() );
@@ -483,48 +470,50 @@ void omxComputeNM::reportResults(FitContext *fc, MxRList *slots, MxRList *out){
 	if( vertexInfeasOut.size() ){
 		Rf_protect(vinf = Rf_allocVector( INTSXP, vertexInfeasOut.size() ));
 		memcpy( INTEGER(vinf), vertexInfeasOut.data(), sizeof(int) * vertexInfeasOut.size() );
-		output.add("finalVertexInfeas", vinf); 
+		output.add("finalVertexInfeas", vinf);
 	}
 	if( pseudohess.rows() && pseudohess.cols() ){
 		Rf_protect(phess = Rf_allocMatrix( REALSXP, pseudohess.rows(), pseudohess.cols() ));
 		memcpy( REAL(phess), pseudohess.data(), sizeof(double) * pseudohess.rows() * pseudohess.cols() );
-		output.add("pseudoHessian", phess); 
+		output.add("pseudoHessian", phess);
 	}
 	if( simplexGradient.rows() && simplexGradient.cols() ){
 		Rf_protect(sg = Rf_allocVector( REALSXP, simplexGradient.rows() ));
 		memcpy( REAL(sg), simplexGradient.data(), sizeof(double) * simplexGradient.rows() );
-		output.add("simplexGradient", sg); 
+		output.add("simplexGradient", sg);
 	}
-	
+
 	Rf_protect(fpm = Rf_allocVector(REALSXP, 1));
 	//it would also work to do 'REAL(fpm)[0] = fproxOut;':
 	memcpy( REAL(fpm), &fproxOut, sizeof(double) );
 	output.add("rangeProximityMeasure", fpm);
-	
+
 	Rf_protect(xpm = Rf_allocVector(REALSXP, 1));
 	memcpy( REAL(xpm), &xproxOut, sizeof(double) );
 	output.add("domainProximityMeasure", xpm);
-	
+
 	Rf_protect(bf = Rf_allocVector(REALSXP, 1));
 	memcpy( REAL(bf), &bestfitOut, sizeof(double) );
 	output.add("penalizedFit", bf);
-	
+
 	slots->add("output", output.asR());
-	return;
 }
 
 //-------------------------------------------------------
 
 NelderMeadOptimizerContext::NelderMeadOptimizerContext(FitContext* _fc, omxComputeNM* _nmo)
-	: fc(_fc), NMobj(_nmo), numFree(_fc->calcNumFree()),
-	  subsidiarygoc(GradientOptimizerContext(_fc, 0L, GradientAlgorithm_Forward, 1L, 1e-5, _nmo))
+	: fc(_fc), NMobj(_nmo), numFree(_fc->getNumFree()),
+    IneqC(_fc, "ineq",
+          [](const omxConstraint &con){ return con.opCode != omxConstraint::EQUALITY; }),
+    EqC(_fc, "eq",
+        [](const omxConstraint &con){ return con.opCode == omxConstraint::EQUALITY; }),
+	  subsidiarygoc(_fc, 0L, _nmo)
 {
 	est.resize(numFree);
 	copyParamsFromFitContext(est.data());
 	statuscode = -1;
 	addPenalty = false;
 	rho = 1;
-	checkRedundantEqualities = true;
 }
 
 void NelderMeadOptimizerContext::copyBounds()
@@ -537,9 +526,9 @@ void NelderMeadOptimizerContext::countConstraintsAndSetupBounds()
 	solLB.resize(numFree);
 	solUB.resize(numFree);
 	copyBounds();
-	
-	omxState *globalState = fc->state;
-	globalState->countNonlinearConstraints(numEqC, numIneqC, false);
+
+	int numEqC = EqC.getCount();
+  int numIneqC = IneqC.getCount();
 	if(verbose){
 		mxLog("counted %d equality constraints",numEqC);
 		mxLog("counted %d inequality constraints",numIneqC);
@@ -551,39 +540,13 @@ void NelderMeadOptimizerContext::countConstraintsAndSetupBounds()
 	if(!numIneqC){ineqConstraintMthd = 0;}
 	equality.resize(numEqC);
 	inequality.resize(numIneqC);
-	
-	fc->equality.resize(numEqC);
-	
-	//Check for redundant equality constraints, and warn if found:
-	if(numEqC > 1 && checkRedundantEqualities){
-		NldrMd_equality_functional eqf(this, fc);
-		Eigen::MatrixXd ej(numEqC, numFree);
-		ej.setConstant(NA_REAL);
-		eqf(est, equality);
-		fd_jacobian<true>(
-			GradientAlgorithm_Central, 4, 1.0e-7,
-			eqf, equality, est, ej);
-		Eigen::MatrixXd ejt = ej.transpose();
-		//mxPrintMat("ej: ",ej);
-		Eigen::FullPivHouseholderQR<Eigen::MatrixXd> qrj;
-		qrj.compute(ejt);
-		if(qrj.rank() < numEqC){
-			Rf_warning(
-				"counted %d equality constraints, but equality-constraint Jacobian is apparently rank %d at the start values; " 
-				"Nelder-Mead will not work correctly unless equality constraints are linearly independent "
-				"(this warning may be spurious if there are non-smooth equality constraints)", numEqC, qrj.rank()
-			);
-			checkRedundantEqualities = false;
-		}
-	}
-	
+
 	if(numEqC + numIneqC || eqConstraintMthd==3){
 		subsidiarygoc.setEngineName("SLSQP");
 		subsidiarygoc.ControlTolerance = 2 * Global->optimalityTolerance;
-		subsidiarygoc.useGradient = true;
 		subsidiarygoc.maxMajorIterations = Global->majorIterations;
 		subsidiarygoc.setupSimpleBounds();
-		//stop("so far, so good");
+		//mxThrow("so far, so good");
 	}
 }
 
@@ -618,43 +581,21 @@ bool NelderMeadOptimizerContext::checkBounds(Eigen::VectorXd &x){
 
 void NelderMeadOptimizerContext::evalIneqC()
 {
-	if(!numIneqC){return;}
-	
-	omxState *st = fc->state;
-	int ineqType = omxConstraint::LESS_THAN;
-	
-	int cur=0, j=0;
-	for (j=0; j < int(st->conListX.size()); j++) {
-		omxConstraint &con = *st->conListX[j];
-		if (con.opCode == omxConstraint::EQUALITY) continue;
-		con.refreshAndGrab(fc, (omxConstraint::Type) ineqType, &inequality(cur));
-		//Nelder-Mead, of course, does not use constraint Jacobians...
-		cur += con.size;
-	}
-	//Nelder-Mead will not care about the function values of inactive inequality constraints:
-	inequality = inequality.array().max(0.0);
-	
+  if (!IneqC.getCount()) return;
+
+  IneqC.eval(fc, inequality.data());
+
 	if (NMobj->verbose >= 3) {
 		mxPrintMat("inequality", inequality);
 	}
-	
 }
 
 void NelderMeadOptimizerContext::evalEqC()
 {
-	if(!numEqC){return;}
-	
-	omxState *st = fc->state;
-	
-	int cur=0, j=0;
-	for(j = 0; j < int(st->conListX.size()); j++) {
-		omxConstraint &con = *st->conListX[j];
-		if (con.opCode != omxConstraint::EQUALITY) continue;
-		con.refreshAndGrab(fc, &equality(cur));
-		//Nelder-Mead, of course, does not use constraint Jacobians...
-		cur += con.size;
-	}
-	
+	if(!EqC.getCount()) return;
+
+  EqC.eval(fc, equality.data());
+
 	if (NMobj->verbose >= 3) {
 		mxPrintMat("equality", equality);
 	}
@@ -691,6 +632,8 @@ void NelderMeadOptimizerContext::checkNewPointInfeas(Eigen::VectorXd &x, Eigen::
 	int i=0;
 	double feasTol = NMobj->feasTol;
 	ifcr.setZero(2);
+	int numEqC = EqC.getCount();
+  int numIneqC = IneqC.getCount();
 	if(!numIneqC && !numEqC){return;}
 	copyParamsFromOptimizer(x,fc);
 	evalIneqC();
@@ -758,7 +701,7 @@ void NelderMeadOptimizerContext::evalFirstPoint(Eigen::VectorXd &x, double &fv,
 				infeas = 1L;
 				return;
 			}
-			//stop("'GDsearch' Not Yet Implemented");
+			//mxThrow("'GDsearch' Not Yet Implemented");
 		case 4:
 			fv = evalFit(x);
 			infeas = 1L;
@@ -837,7 +780,7 @@ void NelderMeadOptimizerContext::evalNewPoint(Eigen::VectorXd &newpt, Eigen::Vec
 				newInfeas = 1L;
 				return;
 			}
-			//stop("'GDsearch' Not Yet Implemented");
+			//mxThrow("'GDsearch' Not Yet Implemented");
 		case 4:
 			fv = evalFit(newpt);
 			newInfeas = 1L;
@@ -890,10 +833,10 @@ void NelderMeadOptimizerContext::initializeSimplex(Eigen::VectorXd startpt, doub
 		Eigen::MatrixXd SiniSupp, iniSimplexMat2;
 		Eigen::VectorXi paramMap(numFree);
 		if(iniSimplexMat.cols() != numFree){
-			stop("'iniSimplexMat' has %d columns, but %d columns expected",iniSimplexMat.cols(), numFree);
+			mxThrow("'iniSimplexMat' has %d columns, but %d columns expected",iniSimplexMat.cols(), numFree);
 		}
 		if( int(NMobj->iniSimplexColnames.size()) != numFree){
-			stop("'iniSimplexMat' has %d column names, but %d column names expected", int(NMobj->iniSimplexColnames.size()), numFree);
+			mxThrow("'iniSimplexMat' has %d column names, but %d column names expected", int(NMobj->iniSimplexColnames.size()), numFree);
 		}
 		if(iniSimplexMat.rows()>n+1){
 			Rf_warning("'iniSimplexMat' has %d rows, but %d rows expected; extraneous rows will be ignored",iniSimplexMat.rows(), n+1);
@@ -901,7 +844,7 @@ void NelderMeadOptimizerContext::initializeSimplex(Eigen::VectorXd startpt, doub
 		}
 		iniSimplexMat2.resize(iniSimplexMat.rows(), numFree);
 		int gx=0;
-		/*If there are no problems, then every time vx gets incremented, it should become equal to the current 
+		/*If there are no problems, then every time vx gets incremented, it should become equal to the current
 		 value of gx*/
 		for (int vx=0; vx < int(fc->varGroup->vars.size()); ++vx) {
 			for (int nx=0; nx < int(NMobj->iniSimplexColnames.size()); ++nx) {
@@ -913,7 +856,7 @@ void NelderMeadOptimizerContext::initializeSimplex(Eigen::VectorXd startpt, doub
 			}
 		}
 		if ( gx != int(NMobj->iniSimplexColnames.size()) ){
-			stop("error in mapping column names of 'iniSimplexMat' to free-parameter labels");
+			mxThrow("error in mapping column names of 'iniSimplexMat' to free-parameter labels");
 		}
 		for(i=0; i < iniSimplexMat.cols(); i++){
 			iniSimplexMat2.col(paramMap[i]) = iniSimplexMat.col(i);
@@ -988,7 +931,7 @@ void NelderMeadOptimizerContext::initializeSimplex(Eigen::VectorXd startpt, doub
 			}
 			break;
 		case 3:
-			//TODO: this could be even smarter if it also figured out different edge lengths 
+			//TODO: this could be even smarter if it also figured out different edge lengths
 			//to account for different scaling of the free parameters:
 			if(n==numFree){
 				vertices[0] = startpt;
@@ -1096,7 +1039,7 @@ void NelderMeadOptimizerContext::fullSort()
 	scfcr.setZero();
 	checkNewPointInfeas(subcentroid, scfcr);
 	badsc = (scfcr.sum()) ? 1 : 0;
-	
+
 	needFullSort = false;
 	if(verbose){
 		mxLog("full sort complete...");
@@ -1154,7 +1097,7 @@ void NelderMeadOptimizerContext::fastSort()
 	scfcr.setZero();
 	checkNewPointInfeas(subcentroid, scfcr);
 	badsc = (scfcr.sum()) ? 1 : 0;
-	
+
 	if(verbose){
 		mxLog("fast sort complete...");
 		printProblemState();
@@ -1341,7 +1284,7 @@ bool NelderMeadOptimizerContext::checkProgress(){
 	if(failedContraction && NMobj->sigma<=0){
 		return(true);
 	}
-	if(NMobj->stagnCtrl[0]>0 && NMobj->stagnCtrl[1]>0 && 
+	if(NMobj->stagnCtrl[0]>0 && NMobj->stagnCtrl[1]>0 &&
     unchangedx0count>=NMobj->stagnCtrl[0] && NMobj->stagnCtrl[1]<restartsUsed){
 		return(true);
 	}
@@ -1365,7 +1308,7 @@ bool NelderMeadOptimizerContext::checkProgress(){
 
 
 void NelderMeadOptimizerContext::invokeNelderMead(){
-	n = numFree - numEqC;
+	n = numFree - EqC.getCount();
 	vertices.resize(n+1);
 	fvals.resize(n+1);
 	vertexInfeas.resize(n+1);
@@ -1383,7 +1326,7 @@ void NelderMeadOptimizerContext::invokeNelderMead(){
 	bool stopflag=false;
 	itersElapsed = 0;
 	restartsUsed = 0;
-	
+
 	//Loop is: sort, check convergence, check progress, transform;
 	do{
 		if(verbose){
@@ -1394,12 +1337,12 @@ void NelderMeadOptimizerContext::invokeNelderMead(){
 			//Order the vertices by fit value:
 			if(needFullSort){fullSort();}
 			else{fastSort();}
-			
+
 			stopflag = checkConvergence();
 			if(stopflag){
 				break;
 			}
-			
+
 			needRestart = checkProgress();
 			if(needRestart){
 				initializeSimplex(vertices[0], sqrt((vertices[0]-vertices[1]).dot(vertices[0]-vertices[1])), true);
@@ -1410,18 +1353,18 @@ void NelderMeadOptimizerContext::invokeNelderMead(){
 				continue;
 			}
 		}
-		
+
 		simplexTransformation();
-		
+
 		eucentroidPrev = eucentroidCurr;
 		itersElapsed++;
 		Global->reportProgress("MxComputeNelderMead", fc);
 	} while (!stopflag);
-	
+
 	est = vertices[0];
 	bestfit = fvals[0];
 	estInfeas = vertexInfeas[0];
-	
+
 	double centFit;
 	int centInfeas;
 	evalNewPoint(subcentroid, vertices[0], centFit, centInfeas, vertexInfeas[0]);
@@ -1429,19 +1372,19 @@ void NelderMeadOptimizerContext::invokeNelderMead(){
 		est = subcentroid;
 		bestfit = centFit;
 		estInfeas = 0;
-		
+
 	}
 	evalNewPoint(eucentroidCurr, vertices[0], centFit, centInfeas, vertexInfeas[0]);
 	if(centFit < bestfit && !centInfeas){
 		est = eucentroidCurr;
 		bestfit = centFit;
 		estInfeas = 0;
-	}	
-	
+	}
+
 	//if(estInfeas){statuscode = 3;}
-	
+
 	if(verbose){mxPrintMat("solution?",est);}
-	
+
 }
 
 
@@ -1461,18 +1404,18 @@ void NelderMeadOptimizerContext::calculatePseudoHessian()
 	Eigen::VectorXi kvec(numpts);
 	Eigen::VectorXd a(n), pmin(n);
 	Eigen::MatrixXd B(n,n), Q(n, n);
-	
+
 	NMobj->pseudohess.setZero(n, n);
 	NMobj->phpts.setZero(numpts, n);
 	NMobj->phFvals.setZero(numpts, 1);
 	NMobj->phInfeas.setZero(numpts);
-	
+
 	for(i=0; i<n; i++){
 		Q.col(i) = vertices[i+1] - vertices[0];
 	}
 	Eigen::FullPivLU< Eigen::MatrixXd > luq(Q);
-	
-	
+
+
 	for(i=0; i<n+1; i++){
 		NMobj->phpts.row(i) = vertices[i];
 		NMobj->phFvals(i,0) = fvals[i];
@@ -1480,7 +1423,7 @@ void NelderMeadOptimizerContext::calculatePseudoHessian()
 		kvec[i] = -1;
 		jvec[i] = -1;
 	}
-	
+
 	i=n+1;
 	for(j=0; j<n; j++){
 		for(k=j+1; k<n+1; k++){
@@ -1510,7 +1453,7 @@ void NelderMeadOptimizerContext::calculatePseudoHessian()
 			i++;
 		}
 	}
-	
+
 	if(canDoAnalyt && luq.isInvertible()){
 		if(verbose){mxLog("analytically calculating pseudoHessian");}
 		a0 = fvals[0];
@@ -1520,7 +1463,7 @@ void NelderMeadOptimizerContext::calculatePseudoHessian()
 		}
 		for(i=n+n+1; i<numpts; i++){
 			if(jvec[i] == kvec[i]){continue;}
-			B(jvec[i]-1,kvec[i]-1) = 2*( NMobj->phFvals(i,0) + a0 - NMobj->phFvals(jvec[i]+(n+1)-1, 0) - 
+			B(jvec[i]-1,kvec[i]-1) = 2*( NMobj->phFvals(i,0) + a0 - NMobj->phFvals(jvec[i]+(n+1)-1, 0) -
 				NMobj->phFvals(kvec[i]+(n+1)-1, 0) );
 			B(kvec[i]-1,jvec[i]-1) = B(jvec[i]-1,kvec[i]-1);
 		}
@@ -1555,7 +1498,7 @@ void NelderMeadOptimizerContext::calculatePseudoHessian()
 			}
 		}
 		polynomb.setZero(numpts,1);
-		
+
 		Eigen::ColPivHouseholderQR< Eigen::MatrixXd > qrx(X);
 		if(qrx.info() != Eigen::Success){
 			NMobj->pseudohess.resize(0,0);
@@ -1566,7 +1509,7 @@ void NelderMeadOptimizerContext::calculatePseudoHessian()
 		}
 		polynomb = qrx.solve(NMobj->phFvals);
 		if(verbose){mxPrintMat("polynomial coefficients:",polynomb);}
-		
+
 		i=n+1;
 		for(j=0; j<n; j++){
 			for(k=j; k<n; k++){
@@ -1606,22 +1549,13 @@ void NelderMeadOptimizerContext::finalize()
 	copyParamsFromOptimizer(est,fc);
 	ComputeFit(engineName, NMobj->fitMatrix, FF_COMPUTE_FIT, fc);
 	/*Doing this here ensures (1) that the fit has just been freshly evaluated at the solution, (2) that this check is done as part of the
-	MxComputeNelderMead step (necessary for bootstrapping), and (3) that Nelder-Mead reports status code 3 for solutions that violate 
+	MxComputeNelderMead step (necessary for bootstrapping), and (3) that Nelder-Mead reports status code 3 for solutions that violate
 	MxConstraints, and status code 10 for	all other kinds of infeasible solutions:*/
 	if(!fc->insideFeasibleSet() && (statuscode==0 || statuscode==4)){fc->setInform(INFORM_STARTING_VALUES_INFEASIBLE);}
-	
-	omxState *st = fc->state;
-	int ineqType = omxConstraint::LESS_THAN;
-	int cur=0, j=0;
-	Eigen::VectorXd cfv(numEqC + numIneqC);
-	
-	for (j=0; j < int(st->conListX.size()); j++) {
-		omxConstraint &con = *st->conListX[j];
-		con.refreshAndGrab(fc, (omxConstraint::Type) ineqType, &cfv(cur));
-		cur += con.size;
-	}
-	
-	fc->constraintFunVals = cfv;
+
+  ConstraintVec cv(fc, "constraint", [](const omxConstraint &con){ return true; });
+  fc->constraintFunVals.resize(cv.getCount());
+  cv.eval(fc, fc->constraintFunVals.data());
 }
 
 
