@@ -1,5 +1,5 @@
  /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2020 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -41,6 +41,7 @@ public:
 			isMixtureInterface(_isMixtureInterface) {};
 	virtual ~MarkovExpectation();
 	virtual void init();
+	virtual void connectToData();
 	virtual void compute(FitContext *fc, const char *what, const char *how);
 	virtual omxMatrix *getComponent(const char*);
 	virtual void populateAttr(SEXP expectation);
@@ -58,8 +59,20 @@ MarkovExpectation::~MarkovExpectation()
 	omxFreeMatrix(scaledTransition);
 }
 
+void MarkovExpectation::connectToData()
+{
+  setConnectedToData(true);
+	auto dc = getDataColumns();
+	for (int cx=0; cx < int(dc.size()); ++cx) {
+		int var = dc[cx];
+		data->assertColumnIsData(var, OMXDATA_REAL);
+	}
+}
+
 void MarkovExpectation::init()
 {
+	loadDataColFromR();
+
 	ProtectedSEXP Rverbose(R_do_slot(rObj, Rf_install("verbose")));
 	verbose = Rf_asInteger(Rverbose);
 
@@ -87,7 +100,7 @@ void MarkovExpectation::init()
 	} else if (strEQ(scaleName, "none")) {
 		scale = SCALE_NONE;
 	} else {
-		stop("%s: unknown scale '%s'", name, scaleName);
+		mxThrow("%s: unknown scale '%s'", name, scaleName);
 	}
 
 	scaledInitial = omxInitMatrix(1, 1, TRUE, currentState);
@@ -99,6 +112,8 @@ void MarkovExpectation::init()
 
 void MarkovExpectation::compute(FitContext *fc, const char *what, const char *how)
 {
+	super::compute(fc, what, how);
+
 	if (fc) {
 		for (auto c1 : components) {
 			c1->compute(fc, what, how);
