@@ -51,7 +51,7 @@ namespace RelationalRAMExpectation {
 		omxExpectation *model;  // read-only
 	public:
 		int row;                 // to load definition variables (never the key)
-		struct independentGroup *ig;
+		class independentGroup *ig;
 		int igIndex;
 		int nextMean;
 
@@ -225,8 +225,8 @@ namespace RelationalRAMExpectation {
 	public:
 		bool isChild() const { return this != parent; }
 		typedef std::vector< std::set<int> > SubgraphType;
-		struct omxExpectation *homeEx;
-		std::set<struct omxExpectation *> allEx;
+		class omxExpectation *homeEx;
+		std::set<class omxExpectation *> allEx;
 		bool hasProductNodes;
 		typedef std::map< std::pair<omxData*,int>, int, RowToLayoutMapCompare> RowToLayoutMapType;
 		RowToLayoutMapType               rowToLayoutMap;
@@ -284,6 +284,11 @@ class omxRAMExpectation : public omxExpectation {
 	std::vector<int> exoDataColumns; // index into omxData
 	Eigen::VectorXd exoPredMean;
 	bool hasProductNodes;
+  bool studiedF;
+  bool openBox;  // can the user access the expectation during optimization?
+	int numExoPred;
+	std::vector<int> exoDataColIndex;
+  void addSlopeMatrix();
 
 	struct MpcIO : PathCalcIO {
 		omxMatrix *M0;
@@ -363,9 +368,11 @@ class omxRAMExpectation : public omxExpectation {
 	void CalculateRAMCovarianceAndMeans(FitContext *fc);
 	void analyzeDefVars(FitContext *fc);
 	void logDefVarsInfluence();
+  bool isOpenBox() const { return openBox; }
 
 	omxMatrix *cov, *means; // observed covariance and means
 	omxMatrixPtr covOwner, meanOwner;
+	omxMatrix *fullCov, *fullMean;
 	omxMatrix *slope;       // exogenous predictor slopes
 	omxMatrix *A, *S, *F, *M;
 
@@ -383,21 +390,32 @@ class omxRAMExpectation : public omxExpectation {
 	int lwork;
 
 	std::vector< omxMatrix* > between;
-	RelationalRAMExpectation::state *rram;
+	RelationalRAMExpectation::state *rram; // should use unique_ptr TODO
 	bool forceSingleGroup;
 
 	void studyF();
 	void studyExoPred();
 
 	virtual void init();
+  virtual void connectToData();
 	virtual void compute(FitContext *fc, const char *what, const char *how);
 	virtual omxMatrix *getComponent(const char*);
 	virtual void populateAttr(SEXP expectation);
-	virtual const std::vector<const char *> &getDataColumnNames() const { return dataColNames; };
+	virtual const std::vector<const char *> &getDataColumnNames() const {
+    if (studiedF) return dataColNames;
+    else return super::getDataColumnNames();
+  };
 	virtual const Eigen::Map<DataColumnIndexVector> getDataColumns() {
-		return Eigen::Map<DataColumnIndexVector>(dataCols.data(), numDataColumns);
+    if (studiedF)
+      return Eigen::Map<DataColumnIndexVector>(dataCols.data(), numDataColumns);
+    else
+      return super::getDataColumns();
 	}
-	virtual std::vector< omxThresholdColumn > &getThresholdInfo() { return thresholds; }
+	virtual std::vector< omxThresholdColumn > &getThresholdInfo()
+  {
+    if (studiedF) return thresholds;
+    else return super::getThresholdInfo();
+  }
 	virtual void invalidateCache();
 	virtual void generateData(FitContext *fc, MxRList &out);
 	virtual void flatten(FitContext *fc);
