@@ -52,13 +52,14 @@ void FitMultigroup::compute(int want, FitContext *fc)
 				if (units == FIT_UNITS_UNINITIALIZED) {
 					units = f1->fitFunction->units;
 				} else if (units != f1->fitFunction->units) {
-					stop("%s: cannot combine units %s and %s (from %s)",
+					mxThrow("%s: cannot combine units %s and %s (from %s)",
 						matrix->name(), fitUnitsToName(units),
 						fitUnitsToName(f1->fitFunction->units), f1->name());
 				}
 			}
 		} else {
 			omxRecompute(f1, fc);
+      if (want & FF_COMPUTE_GRADIENT) invalidateGradient(fc);
 		}
 		if (want & FF_COMPUTE_FIT) {
 			if(f1->rows != 1 || f1->cols != 1) {
@@ -103,7 +104,6 @@ void FitMultigroup::init()
 	if (mg->fits.size()) return; // hack to prevent double initialization, remove TOOD
 
 	oo->units = FIT_UNITS_UNINITIALIZED;
-	oo->gradientAvailable = TRUE;
 	oo->hessianAvailable = TRUE;
 	oo->canDuplicate = true;
 
@@ -120,16 +120,14 @@ void FitMultigroup::init()
 		if (fits[gx] >= 0) {
 			mat = os->algebraList[fits[gx]];
 		} else {
-			stop("Can only add algebra and fitfunction");
+			mxThrow("Can only add algebra and fitfunction");
 		}
-		if (mat == oo->matrix) stop("Cannot add multigroup to itself");
+		if (mat == oo->matrix) mxThrow("Cannot add multigroup to itself");
 		mg->fits.push_back(mat);
 		if (mat->fitFunction) {
 			omxCompleteFitFunction(mat);
-			oo->gradientAvailable = (oo->gradientAvailable && mat->fitFunction->gradientAvailable);
 			oo->hessianAvailable = (oo->hessianAvailable && mat->fitFunction->hessianAvailable);
 		} else {
-			oo->gradientAvailable = FALSE;
 			oo->hessianAvailable = FALSE;
 		}
 	}
@@ -146,8 +144,8 @@ void FitMultigroup::traverse(std::function<void(omxMatrix*)> &fn)
 /* TODO
 void omxMultigroupAdd(omxFitFunction *oo, omxFitFunction *fit)
 {
-	if (oo->initFun != initFitMultigroup) stop("%s is not the multigroup fit", oo->fitType);
-	if (!oo->initialized) stop("Fit not initialized", oo);
+	if (oo->initFun != initFitMultigroup) mxThrow("%s is not the multigroup fit", oo->fitType);
+	if (!oo->initialized) mxThrow("Fit not initialized", oo);
 
 	FitMultigroup *mg = (FitMultigroup*) oo->argStruct;
 	mg->fits.push_back(fit->matrix);
