@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2020 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
  */
 
 /***********************************************************
- * 
+ *
  *  omxData.h
  *
  *  Created: Timothy R. Brick 	Date: 2009-07-15
@@ -31,10 +31,24 @@
 #define _OMXDATA_H_
 
 #include "omxDefines.h"
-#include <R_ext/Rdynload.h> 
+#include <R_ext/Rdynload.h>
 
 class omxData;
-typedef struct omxThresholdColumn omxThresholdColumn;
+struct omxThresholdColumn {
+  // Which column in the matrix/data.frame (DataColumns permutation already applied)
+  // Can use data->columnName(dColumn)
+	int dataColumn;
+	int column;			// Which column in the thresholds matrix
+	int numThresholds;		// And how many thresholds
+	bool isDiscrete;
+
+	// for continuous variables, column=-1, numThresholds=0
+	omxThresholdColumn() :
+		dataColumn(-1), column(-1), numThresholds(0), isDiscrete(false) {};
+
+	void log() { mxLog("dCol=%d discrete=%d col=%d #thr=%d",
+                     dataColumn, isDiscrete, column, numThresholds); }
+};
 
 #include "omxAlgebra.h"
 #include "omxFitFunction.h"
@@ -52,15 +66,10 @@ struct omxDefinitionVar {
 	bool loadData(omxState *state, double val);
 };
 
-struct omxThresholdColumn {
-	int dColumn;			// Which column in the matrix/data.frame
-	int column;			// Which column in the thresholds matrix
-	int numThresholds;		// And how many thresholds
-
-	// for continuous variables, numThresholds=0
-	omxThresholdColumn() : dColumn(-1), column(0), numThresholds(0) {};
-
-	void log() { mxLog("dCol=%d col=%d #thr=%d", dColumn, column, numThresholds); }
+enum OmxDataType {
+									OMXDATA_REAL,
+									OMXDATA_ORDINAL,
+									OMXDATA_COUNT
 };
 
 enum ColumnDataType {
@@ -74,20 +83,66 @@ enum ColumnDataType {
 union dataPtr {
 	double *realData;
 	int *intData;
+	dataPtr() : intData(0) {};
 	dataPtr(double *_p) : realData(_p) {};
 	dataPtr(int *_p) : intData(_p) {};
 	void clear() { realData=0; intData=0; };
 };
 
-struct ColumnData {
+class ColumnData {
+private:
+	dataPtr ptr;
+  bool owner;
+  int minValue; // for count/ordinal only
+  int maxValue; // for count/ordinal only
+public:
 	const char *name;
 	ColumnDataType type;
-	dataPtr ptr;
-	std::vector<std::string> levels;       // factors only
+	std::vector<std::string> levelNames;       // factors only
 
 	const char *typeName();
+  ColumnData(const char *_name) : owner(false), minValue(1), maxValue(NA_INTEGER),
+                                  name(_name), type(COLUMNDATA_INVALID) {}
+  ColumnData(const char *_name, ColumnDataType _type, int *col) :
+    ptr(col), owner(true), minValue(1), maxValue(NA_INTEGER),
+    name(_name), type(_type) {}
+  ~ColumnData() { clear(); }
+  void clear();
+  ColumnData clone() const;
+  void setMinValue(int mv) { minValue = mv; }
+  void verifyMinValue(int nrows);
+  void setZeroMinValue(int rows);
+  int getMinValue() const { return minValue; }
+  int getMaxValue() const { if (maxValue==NA_INTEGER) OOPS; return maxValue; }
+  int getMaxValueUnsafe() const { return maxValue; }
+  void setMaxValue(int val) { maxValue = val; }
+  void setMaxValueFromLevels() { maxValue = minValue + levelNames.size() - 1; }
+  void setMaxValueFromData(int nrows);
+  int getNumThresholds() const { if (maxValue==NA_INTEGER) OOPS; return maxValue - minValue; }
+  int getNumOutcomes() const { return 1 + getNumThresholds(); }
+  int *i() { return ptr.intData; }
+  double *d() { return ptr.realData; }
+  void setOwn(double *_p) { clear(); ptr.realData = _p; owner=true; }
+  void setOwn(int *_p) { clear(); ptr.intData = _p; owner=true; }
+  void setOwn(dataPtr _p) { clear(); ptr = _p; owner=true; }
+  void setBorrow(double *_p) { clear(); ptr.realData = _p; owner=false; }
+  void setBorrow(int *_p) { clear(); ptr.intData = _p; owner=false; }
+  void setBorrow(dataPtr _p) { clear(); ptr = _p; owner=false; }
+  dataPtr steal() { dataPtr ret = ptr; ptr.clear(); return ret; }
 };
 
+inline void ColumnData::clear()
+{
+  if (ptr.intData && owner) {
+    if (type == COLUMNDATA_NUMERIC) {
+      delete [] ptr.realData;
+    } else {
+      delete [] ptr.intData;
+    }
+  }
+  ptr.intData = 0;
+}
+
 typedef Eigen::Matrix<int, Eigen::Dynamic, 1> DataColumnIndexVector;
 
 struct WLSVarData {
@@ -98,11 +153,6 @@ struct WLSVarData {
 	int thrOffset;
 };
 
-struct cstrCmp {
-	bool operator() (const char *s1, const char *s2) const
-	{ return strcmp(s1,s2) < 0; }
-};
-
 typedef std::map< const char *, int, cstrCmp > ColMapType;
 
 class obsSummaryStats {
@@ -165,6 +215,7 @@ class omxData {
 	int freqCol;
 	int *currentFreqColumn;
 	std::unique_ptr< obsSummaryStats > oss;
+  int numEstimatedEntries;
 	bool parallel;
 	bool noExoOptimize;
 	bool modified;
@@ -189,7 +240,7 @@ class omxData {
 	void omxPrintData(const char *header, int maxRows, int *permute);
 	void omxPrintData(const char *header, int maxRows);
 	void omxPrintData(const char *header);
-	void assertColumnIsData(int col);
+	void assertColumnIsData(int col, OmxDataType dt);
 	void setModified() { modified=true; };
 	bool isModified() { return modified; };
 	double getMinVariance() const { return minVariance; };
@@ -208,13 +259,13 @@ class omxData {
 		std::vector<ColumnData> rawCols;
 		std::vector<bool> hasNa;
 		int rows;
-		bool owner;
-		RawData() : rows(0), owner(false) {}
+		RawData() : rows(0) {}
 		void clear();
 		void clearColumn(int col);
 		~RawData();
 		void refreshHasNa();
-		void assertColumnIsData(int col, bool warn);
+		void assertColumnIsData(int col, OmxDataType dt, bool warn);
+    void operator=(const RawData &other);
 	};
 	RawData filtered;
 	RawData unfiltered;
@@ -303,7 +354,6 @@ omxData* omxDataLookupFromState(SEXP dataObject, omxState* state);	// Retrieves
 void omxFreeData(omxData* od);					// Release any held data.
 
 /* Getters 'n Setters */
-int omxDataGetNumFactorLevels(omxData *od, int col);
 double omxDoubleDataElement(omxData *od, int row, int col);
 double *omxDoubleDataColumn(omxData *od, int col);
 int omxIntDataElement(omxData *od, int row, int col);						// Returns one data object as an integer
@@ -313,7 +363,7 @@ bool omxDataElementMissing(omxData *od, int row, int col);
 inline int omxKeyDataElement(omxData *od, int row, int col)
 {
 	ColumnData &cd = od->rawCol(col);
-	return cd.ptr.intData[row];
+	return cd.i()[row];
 }
 
 omxMatrix* omxDataCovariance(omxData *od);
@@ -344,19 +394,19 @@ void omxDataRow(omxData *od, int row, const Eigen::MatrixBase<T> &colList, omxMa
 static OMXINLINE int
 omxIntDataElementUnsafe(omxData *od, int row, int col)
 {
-	return od->rawCol(col).ptr.intData[row];
+	return od->rawCol(col).i()[row];
 }
 
 static OMXINLINE int *omxIntDataColumnUnsafe(omxData *od, int col)
 {
-	return od->rawCol(col).ptr.intData;
+	return od->rawCol(col).i();
 }
 
 double omxDataNumObs(omxData *od);											// Returns number of obs in the dataset
 bool omxDataColumnIsKey(omxData *od, int col);
 const char *omxDataColumnName(omxData *od, int col);
 const char *omxDataType(omxData *od);			      // TODO: convert to ENUM
-	
+
 int omxDataNumNumeric(omxData *od);                   // Number of numeric columns in the data set
 int omxDataNumFactor(omxData *od);                    // Number of factor columns in the data set
 
