@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2020 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -72,7 +72,6 @@ class omxComputeNumericDeriv : public omxCompute {
 	double minimum;
 	Eigen::ArrayXd optima;
 	int numParams;
-	std::vector<int> paramMap;
 	double *gcentral, *gforward, *gbackward;
 	double *hessian;
 	struct hess_struct *hessWorkVector;
@@ -82,7 +81,7 @@ class omxComputeNumericDeriv : public omxCompute {
 	void omxPopulateHessianWork(struct hess_struct *hess_work, FitContext* fc);
 	void omxEstimateHessianOnDiagonal(int i, struct hess_struct* hess_work);
 	void omxEstimateHessianOffDiagonal(int i, int l, struct hess_struct* hess_work);
-	void omxCalcFinalConstraintJacobian(FitContext* fc, int npar);
+	void omxCalcFinalConstraintJacobian(FitContext* fc);
 
 	struct calcHessianEntry {
 		omxComputeNumericDeriv &cnd;
@@ -115,7 +114,7 @@ class omxComputeNumericDeriv : public omxCompute {
 			Global->reportProgress1(cnd.name, detail);
 		}
 	};
-	
+
  public:
         virtual void initFromFrontend(omxState *, SEXP rObj);
         virtual void computeImpl(FitContext *fc);
@@ -142,22 +141,22 @@ void omxComputeNumericDeriv::omxPopulateHessianWork(struct hess_struct *hess_wor
  */
 void omxComputeNumericDeriv::omxEstimateHessianOnDiagonal(int i, struct hess_struct* hess_work)
 {
-	int ix = paramMap[i];
 	static const double v = 2.0; //Note: NumDeriv comments that this could be a parameter, but is hard-coded in the algorithm
 
 	double *Haprox             = hess_work->Haprox;
 	double *Gcentral             = hess_work->Gcentral;
 	double *Gforward             = hess_work->Gforward;
 	double *Gbackward            = hess_work->Gbackward;
-	omxMatrix* fitMatrix = hess_work->fitMatrix; 
-	FitContext* fc = hess_work->fc; 
-	double *freeParams         = fc->est;
+	omxMatrix* fitMatrix = hess_work->fitMatrix;
+	FitContext* fc = hess_work->fc;
+	auto &freeParams         = fc->est;
+	int ix = fc->freeToParamMap[i];
 
 	/* Part the first: Gradient and diagonal */
 	double iOffset = std::max(fabs(stepSize * optima[i]), stepSize);
 	for(int k = 0; k < numIter; k++) {			// Decreasing step size, starting at k == 0
 		freeParams[ix] = optima[i] + iOffset;
-		
+
 		fc->copyParamToModel();
 
 		++hess_work->probeCount;
@@ -205,14 +204,14 @@ void omxComputeNumericDeriv::omxEstimateHessianOnDiagonal(int i, struct hess_str
 
 void omxComputeNumericDeriv::omxEstimateHessianOffDiagonal(int i, int l, struct hess_struct* hess_work)
 {
-	int ix = paramMap[i];
-	int lx = paramMap[l];
     static const double v = 2.0; //Note: NumDeriv comments that this could be a parameter, but is hard-coded in the algorithm
 
 	double *Haprox             = hess_work->Haprox;
-	omxMatrix* fitMatrix = hess_work->fitMatrix; 
-	FitContext* fc = hess_work->fc; 
-	double *freeParams         = fc->est;
+	omxMatrix* fitMatrix = hess_work->fitMatrix;
+	FitContext* fc = hess_work->fc;
+	auto &freeParams         = fc->est;
+	int ix = fc->freeToParamMap[i];
+	int lx = fc->freeToParamMap[l];
 
 	double iOffset = std::max(fabs(stepSize*optima[i]), stepSize);
 	double lOffset = std::max(fabs(stepSize*optima[l]), stepSize);
@@ -281,8 +280,9 @@ void omxComputeNumericDeriv::initFromFrontend(omxState *state, SEXP rObj)
 	SEXP slotValue;
 
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("iterations")));
-	numIter = INTEGER(slotValue)[0];
-	if (numIter < 2) mxThrow("stepSize must be 2 or greater");
+	numIter = Rf_asInteger(slotValue);
+	if (numIter < 2) mxThrow("%s: iterations must be 2 or greater (currently %d)",
+                           name, numIter);
 
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("parallel")));
 	parallel = Rf_asLogical(slotValue);
@@ -333,39 +333,39 @@ void omxComputeNumericDeriv::initFromFrontend(omxState *state, SEXP rObj)
 
 	numParams = 0;
 	totalProbeCount = 0;
-	numParams = 0;
 	recordDetail = true;
 	detail = 0;
 }
 
-void omxComputeNumericDeriv::omxCalcFinalConstraintJacobian(FitContext* fc, int npar){
-	allconstraints_functional acf(*fc, verbose);
-	Eigen::MatrixWrapper< Eigen::ArrayXd > optimaM(optima);
-	Eigen::VectorXd resulttmp(fc->state->numEqC + fc->state->numIneqC);
-	Eigen::MatrixXd jactmp(fc->state->numEqC + fc->state->numIneqC, npar);
-	acf(optimaM, resulttmp, jactmp);
+void omxComputeNumericDeriv::omxCalcFinalConstraintJacobian(FitContext* fc)
+{
+  ConstraintVec cvec(fc, "constraint",
+                     [](const omxConstraint &con){ return true; });
+  if (cvec.getCount() == 0) return;
+
 	/*Gradient algorithm, iterations, and stepsize are hardcoded as they are for two reasons.
-	 * 1.  Differentiating the constraint functions should not take long, expecially compared to 
+	 * 1.  Differentiating the constraint functions should not take long, expecially compared to
 	 * twice-differentiating the fitfunction, so it might as well be done carefully.
-	 * 2.  The default behavior during the ComputeNumericDeriv step uses different values of 
+	 * 2.  The default behavior during the ComputeNumericDeriv step uses different values of
 	 * gradient stepsize and iterations depending on whether or not the MxModel contains thresholds,
 	 * since the numerical accuracy of the -2logL is worse when multivariate-normal integration is involved.
 	 * But, that has no bearing on the constraint functions, so it doesn't really make sense to use
 	 * the stepsize and iterations stored in the omxComputeNumericDeriv object.
 	 */
-	fd_jacobian<true>(
-		GradientAlgorithm_Central, 4, 1.0e-7,
-    acf, resulttmp, optimaM, jactmp);
-	
-	fc->constraintFunVals = resulttmp;
-	fc->constraintJacobian = jactmp;
+  cvec.allocJacTool(fc);
+  cvec.setAlgoOptions(GradientAlgorithm_Central, 4, 1.0e-7);
+
+	fc->constraintFunVals.resize(cvec.getCount());
+	fc->constraintJacobian.resize(cvec.getCount(), fc->getNumFree());
+  cvec.eval(fc, fc->constraintFunVals.data(), fc->constraintJacobian.data());
+
 	/* Subsequent code assumes that fc->est is set to the optimium
 	  point. If there is more than 1 thread then the top-level fc->est
 	  is unchanged, but if there is only 1 thread then the last
 	  parameter in fc->est is offset due to numeric differentiation.
 	  Hence, we set it back to its optimum value: */
-	fc->est[npar-1L] = optima.coeff(npar-1L);
-	return;
+  int lastPar = fc->getNumFree() - 1;
+	fc->est[lastPar] = optima.coeff(lastPar);
 }
 
 void omxComputeNumericDeriv::computeImpl(FitContext *fc)
@@ -378,10 +378,12 @@ void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 		return; //Possible TODO: calculate Hessian anyway?
 	}
 
+	omxAlgebraPreeval(fitMat, fc);
+
 	int newWanted = fc->wanted | FF_COMPUTE_GRADIENT;
 	if (wantHessian) newWanted |= FF_COMPUTE_HESSIAN;
 
-	int nf = fc->calcNumFree();
+	int nf = fc->getNumFree();
 	if (numParams != 0 && numParams != nf) {
 		mxThrow("%s: number of parameters changed from %d to %d",
 			 name, numParams, nf);
@@ -392,26 +394,16 @@ void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 
 	optima.resize(numParams);
 	fc->copyEstToOptimizer(optima);
-	paramMap.resize(numParams);
-	for (int px=0,ex=0; px < numParams; ++ex) {
-		if (fc->profiledOut[ex]) continue;
-		paramMap[px++] = ex;
-	}
 
-	omxAlgebraPreeval(fitMat, fc);
-	fc->createChildren(fitMat); // allow FIML rowwiseParallel even when parallel=false
+  omxCalcFinalConstraintJacobian(fc);
 
-	fc->state->countNonlinearConstraints(fc->state->numEqC, fc->state->numIneqC, false);
-	int c_n = fc->state->numEqC + fc->state->numIneqC;
-	fc->constraintFunVals.resize(c_n);
-	fc->constraintJacobian.resize(c_n, numParams);
-	if(c_n){
-		omxCalcFinalConstraintJacobian(fc, numParams);
-	}
 	// TODO: Allow more than one hessian value for calculation
 
 	int numChildren = 1;
-	if (parallel && !fc->openmpUser && fc->childList.size()) numChildren = fc->childList.size();
+  if (parallel) {
+    fc->createChildren(fitMat, false);
+    numChildren = fc->numOptimizerThreads();
+  }
 
 	if (!fc->haveReferenceFit(fitMat)) return;
 
@@ -501,12 +493,12 @@ void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 
 	Eigen::Map< Eigen::ArrayXi > Gsymmetric(LOGICAL(VECTOR_ELT(detail, 0)), numParams);
 	double gradNorm = 0.0;
-	
+
 	double feasibilityTolerance = Global->feasibilityTolerance;
 	for (int px=0; px < numParams; ++px) {
 		// factor out simliar code in ComputeNR
 		Gsymmetric[px] = true;
-		omxFreeVar &fv = *fc->varGroup->vars[ paramMap[px] ];
+		omxFreeVar &fv = *fc->varGroup->vars[ fc->freeToParamMap[px] ];
 		if ((fabs(optima[px] - fv.lbound) < feasibilityTolerance && Gc[px] > 0) ||
 		    (fabs(optima[px] - fv.ubound) < feasibilityTolerance && Gc[px] < 0)) {
 			Gsymmetric[px] = false;
@@ -519,29 +511,21 @@ void omxComputeNumericDeriv::computeImpl(FitContext *fc)
 			mxLog("%s: param[%d] %d %f", name, px, Gsymmetric[px], relsym);
 		}
 	}
-	
-	fc->haveGrad.assign(fc->numParam, true);
-	fc->gradZ.resize(fc->numParam);
-	fc->gradZ.setZero();
+
+  fc->initGrad();
 	fc->copyGradFromOptimizer(Gc);
-	
-	if(c_n){
-		fc->inequality.resize(fc->state->numIneqC);
-		fc->analyticIneqJacTmp.resize(fc->state->numIneqC, numParams);
-		fc->myineqFun(true, verbose, omxConstraint::LESS_THAN, false);
-	}
 
 	gradNorm = sqrt(gradNorm);
 	double gradThresh = Global->getGradientThreshold(minimum);
-	//The gradient will generally not be near zero at a local minimum if there are equality constraints 
-	//or active inequality constraints:
-	if ( checkGradient && gradNorm > gradThresh && !(fc->state->numEqC || fc->inequality.array().sum()) ) {
+
+	if ( checkGradient && gradNorm > gradThresh && fc->isEffectivelyUnconstrained()) {
 		if (verbose >= 1) {
 			mxLog("Some gradient entries are too large, norm %f", gradNorm);
 		}
 		if (fc->getInform() < INFORM_NOT_AT_OPTIMUM) fc->setInform(INFORM_NOT_AT_OPTIMUM);
 	}
 
+  fc->destroyChildren();
 	fc->setEstFromOptimizer(optima);
 	// auxillary information like per-row likelihoods need a refresh
 	ComputeFit(name, fitMat, FF_COMPUTE_FIT, fc);
@@ -572,4 +556,3 @@ omxCompute *newComputeNumericDeriv()
 {
 	return new omxComputeNumericDeriv;
 }
-
