@@ -1,5 +1,5 @@
  /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2020 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -112,7 +112,7 @@ struct multi_normal_deriv {
 		Eigen::MatrixXd obCov = obCovAdapter;
 		EigenMatrixAdaptor exCovAdapter(omo->expectedCov);
 		Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> exCov = exCovAdapter.cast<T>();
-		
+
 		Eigen::VectorXd obMeans(obCov.rows());
 		Eigen::Matrix<T, Eigen::Dynamic, 1> exMeans;
 		if (omo->observedMeans) {
@@ -154,10 +154,88 @@ void MLFitState::compute(int want, FitContext *fc)
 
 	omxExpectationCompute(fc, expectation, NULL);
 
-	if ((want & FF_COMPUTE_FIT) &&
-	    !(want & (FF_COMPUTE_GRADIENT | FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN | FF_COMPUTE_INFO))) {
+	if (strEQ(expectation->name, "MxExpectationNormal") &&
+		   (want & (FF_COMPUTE_GRADIENT | FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN | FF_COMPUTE_INFO))) {
+		if ((want & FF_COMPUTE_INFO) && fc->infoMethod != INFO_METHOD_HESSIAN) {
+			omxRaiseErrorf("Information matrix approximation method %d is not available",
+				       fc->infoMethod);
+			return;
+		}
+		MLFitState *omo = (MLFitState*) oo;
+		// should forward computation to the expectation TODO
+
+    int numFree = fc->getNumFree();
+		int numSimpleParam = 0;
+		std::vector<bool> fvMask(numFree);
+		for (int fx=0; fx < int(numFree); ++fx) {
+			omxFreeVar *fv = fc->varGroup->vars[fx];
+			bool relevant = fv->getLocation(omo->expectedCov) != NULL;
+			if (omo->expectedMeans) {
+				relevant |= fv->getLocation(omo->expectedMeans) != NULL;
+			}
+			fvMask[fx] = relevant;
+			if (relevant) ++numSimpleParam;
+		}
+    //mxLog("%s: %d/%d analytic derivs", name(), numSimpleParam, numFree);
+
+		if (numSimpleParam == 0) {
+			// if numSimpleParam == 0 then hessian() doesn't compute the fit
+    if (want & FF_COMPUTE_GRADIENT) fc->gradZ.setConstant(NA_REAL);
+			want &= ~(FF_COMPUTE_GRADIENT | FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN | FF_COMPUTE_INFO);
+			if (want) compute(want, fc);
+			return;
+		}
+
+		double init_log_prob = 0.0;
+		Eigen::VectorXd init_grad = Eigen::VectorXd::Zero(numSimpleParam);
+		HessianBlock *hb = new HessianBlock;
+
+		Eigen::VectorXd cont_params = Eigen::VectorXd::Zero(numSimpleParam);
+		int cpx=0;
+		for (int fx=0; fx < int(numFree); ++fx) {
+			if (!fvMask[fx]) continue;
+			hb->vars.push_back(fx);
+			cont_params[cpx++] = fc->est[fx];
+		}
+		hb->mat.resize(numSimpleParam, numSimpleParam);
+		hb->mat.setZero();
+
+		try {
+			multi_normal_deriv model(fc, fvMask, omo);
+			stan::math::hessian(model, cont_params, init_log_prob, init_grad, hb->mat);
+		} catch (const std::exception& e) {
+			init_log_prob = NA_REAL;
+			if (fc) fc->recordIterationError("%s: %s", oo->name(), e.what());
+		} catch (...) {
+			init_log_prob = NA_REAL;
+			if (fc) fc->recordIterationError("%s: unknown error", oo->name());
+		}
+
+		if (want & FF_COMPUTE_FIT) {
+			oo->matrix->data[0] = Scale * init_log_prob;
+		}
+		if (want & FF_COMPUTE_GRADIENT) {
+			int px=0;
+			for (int fx=0; fx < int(numFree); ++fx) {
+				if (!fvMask[fx]) { fc->gradZ[fx] = NA_REAL; continue; }
+				fc->gradZ[fx] += Scale * init_grad[px];
+				++px;
+			}
+		}
+		if (want & (FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN | FF_COMPUTE_INFO)) {
+			const double HScale = (want & FF_COMPUTE_INFO)? -fabs(Scale) : Scale;
+			hb->mat *= HScale;
+			fc->queue(hb);
+		} else {
+			delete hb;
+		}
+	} else if ((want & (FF_COMPUTE_FIT | FF_COMPUTE_GRADIENT)) &&
+	    !(want & (FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN | FF_COMPUTE_INFO))) {
 		// works for any multivariate normal expectation (e.g. vanilla, RAM, LISREL, etc)
 
+    if (want & FF_COMPUTE_GRADIENT) fc->gradZ.setConstant(NA_REAL);
+    if (!(want & FF_COMPUTE_FIT)) return;
+
 		MLFitState *omo = (MLFitState*) oo;
 		EigenMatrixAdaptor obCovAdapter(omo->observedCov);
 		Eigen::MatrixXd obCov = obCovAdapter;
@@ -165,6 +243,10 @@ void MLFitState::compute(int want, FitContext *fc)
 		Eigen::MatrixXd exCov = exCovAdapter;
 		double fit;
 		try {
+      // If we want to drop dependency on stan::math,
+      // it looks like analytic expressions for derivs are available in
+      // https://www.academia.edu/23179079/Finite_Normal_Mixture_Sem_Analysis_by_Fitting_Multiple_Conventional_Sem_Models
+      // Appendix B
 			if (omo->observedMeans) {
 				EigenVectorAdaptor obMeansAdapter(omo->observedMeans);
 				Eigen::VectorXd obMeans = obMeansAdapter;
@@ -186,79 +268,6 @@ void MLFitState::compute(int want, FitContext *fc)
 			if (fc) fc->recordIterationError("%s: unknown error", oo->name());
 		}
 		oo->matrix->data[0] = Scale * fit;
-	} else if (strEQ(expectation->name, "MxExpectationNormal") &&
-		   (want & (FF_COMPUTE_GRADIENT | FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN | FF_COMPUTE_INFO))) {
-		if ((want & FF_COMPUTE_INFO) && fc->infoMethod != INFO_METHOD_HESSIAN) {
-			omxRaiseErrorf("Information matrix approximation method %d is not available",
-				       fc->infoMethod);
-			return;
-		}
-		MLFitState *omo = (MLFitState*) oo;
-		// should forward computation to the expectation TODO
-
-		int num_param = 0;
-		std::vector<bool> fvMask(fc->numParam);
-		for (int fx=0; fx < int(fc->numParam); ++fx) {
-			omxFreeVar *fv = fc->varGroup->vars[fx];
-			bool relevant = fv->getLocation(omo->expectedCov) != NULL;
-			if (omo->expectedMeans) {
-				relevant |= fv->getLocation(omo->expectedMeans) != NULL;
-			}
-			fvMask[fx] = relevant;
-			if (relevant) ++num_param;
-		}
-
-		if (num_param == 0) {
-			// if num_param == 0 then hessian() doesn't compute the fit
-			want &= ~(FF_COMPUTE_GRADIENT | FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN | FF_COMPUTE_INFO);
-			if (want) compute(want, fc);
-			return;
-		}
-
-		double init_log_prob = 0.0;
-		Eigen::VectorXd init_grad = Eigen::VectorXd::Zero(num_param);
-		HessianBlock *hb = new HessianBlock;
-
-		Eigen::VectorXd cont_params = Eigen::VectorXd::Zero(num_param);
-		int cpx=0;
-		for (int fx=0; fx < int(fc->numParam); ++fx) {
-			if (!fvMask[fx]) continue;
-			hb->vars.push_back(fx);
-			cont_params[cpx++] = fc->est[fx];
-		}
-		hb->mat.resize(num_param, num_param);
-		hb->mat.setZero();
-		
-		try {
-			multi_normal_deriv model(fc, fvMask, omo);
-			stan::math::hessian(model, cont_params, init_log_prob, init_grad, hb->mat);
-		} catch (const std::exception& e) {
-			init_log_prob = NA_REAL;
-			if (fc) fc->recordIterationError("%s: %s", oo->name(), e.what());
-		} catch (...) {
-			init_log_prob = NA_REAL;
-			if (fc) fc->recordIterationError("%s: unknown error", oo->name());
-		}
-
-		if (want & FF_COMPUTE_FIT) {
-			oo->matrix->data[0] = Scale * init_log_prob;
-		}
-		if (want & FF_COMPUTE_GRADIENT) {
-			int px=0;
-			for (int fx=0; fx < int(fc->numParam); ++fx) {
-				if (!fvMask[fx]) continue;
-				fc->haveGrad[fx] = true;
-				fc->gradZ[fx] += Scale * init_grad[px];
-				++px;
-			}
-		}
-		if (want & (FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN | FF_COMPUTE_INFO)) {
-			const double HScale = (want & FF_COMPUTE_INFO)? -fabs(Scale) : Scale;
-			hb->mat *= HScale;
-			fc->queue(hb);
-		} else {
-			delete hb;
-		}
 	} else {
 		mxThrow("Not implemented");
 	}
@@ -286,12 +295,12 @@ void MLFitState::populateAttr(SEXP algebra) {
 				REAL(expMeanExt)[col * expMeanInt->rows + row] =
 					omxMatrixElement(expMeanInt, row, col);
 	} else {
-		Rf_protect(expMeanExt = Rf_allocMatrix(REALSXP, 0, 0));		
-	}   
+		Rf_protect(expMeanExt = Rf_allocMatrix(REALSXP, 0, 0));
+	}
 
 	Rf_setAttrib(algebra, Rf_install("expCov"), expCovExt);
 	Rf_setAttrib(algebra, Rf_install("expMean"), expMeanExt);
-	
+
 	double saturated_out;
 	double independence_out;
 	calcExtraLikelihoods(oo, &saturated_out, &independence_out);
@@ -314,11 +323,11 @@ omxFitFunction *MLFitState::initMorph()
 	if (strcmp(expectation->name, "MxExpectationBA81")==0) {
 		return omxChangeFitType(oo, "imxFitFunctionBA81");
 	}
-	
+
 	if (strEQ(expectation->name, "MxExpectationGREML")) {
 		return omxChangeFitType(oo, "imxFitFunciontGRMFIML");
 	}
-	
+
 	if (strEQ(expectation->name, "MxExpectationStateSpace")) {
 		return omxChangeFitType(oo, "imxFitFunciontStateSpace");
 	}
@@ -371,7 +380,7 @@ omxFitFunction *MLFitState::initMorph()
 		} else {
 			to = "imxFitFunctionFIML";
 		}
-		if(OMX_DEBUG) { mxLog("Raw Data: Converting from %s to %s", oo->fitType, to); }
+		if(OMX_DEBUG) { mxLog("%s: Converting from %s to %s", name(), oo->fitType, to); }
 		return omxChangeFitType(oo, to);
 	}
 
@@ -424,7 +433,7 @@ void MLFitState::init()
 		return;
 	}
 
-	// Error Checking: Observed/Expected means must agree.  
+	// Error Checking: Observed/Expected means must agree.
 	// ^ is XOR: true when one is false and the other is not.
 	if((newObj->expectedMeans == NULL) ^ (newObj->observedMeans == NULL)) {
 		if(newObj->expectedMeans != NULL) {
@@ -440,7 +449,6 @@ void MLFitState::init()
 	if (strEQ(expectation->name, "MxExpectationNormal") &&
 	    newObj->expectedCov->isSimple() &&
 	    (!newObj->expectedMeans || newObj->expectedMeans->isSimple())) {
-		oo->gradientAvailable = true;
 		oo->hessianAvailable = true;
 	}
 
