@@ -1,5 +1,5 @@
 /*
- * Copyright 2007-2019 by the individuals mentioned in the source code history
+ * Copyright 2007-2020 by the individuals mentioned in the source code history
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -113,10 +113,13 @@ void omxRAMExpectation::compute(FitContext *fc, const char *what, const char *ho
 	}
 
 	oro->CalculateRAMCovarianceAndMeans(fc);
+
+	super::compute(fc, what, how);
 }
 
 void omxRAMExpectation::invalidateCache()
 {
+  super::invalidateCache();
 	if (rram) {
 		delete rram;
 		rram = 0;
@@ -157,7 +160,8 @@ void omxRAMExpectation::populateAttr(SEXP robj)
 	if (oro->rram) {
 		rram->exportInternalState(dbg);
 	} else {
-		oro->CalculateRAMCovarianceAndMeans(0);
+    // This code should be shared with all normal expectations TODO
+    compute(0, 0, 0);
 		EigenMatrixAdaptor Ecov(oro->cov);
 		out.add("covariance", Rcpp::wrap(Ecov));
 		if (oro->means) {
@@ -167,6 +171,7 @@ void omxRAMExpectation::populateAttr(SEXP robj)
 		if (hasProductNodes) {
 			dbg.add("polyRep", Rcpp::wrap(pcalc.getPolyRep()));
 		}
+    populateNormalAttr(robj, out);
 	}
 
 	Rf_setAttrib(robj, Rf_install("output"), out.asR());
@@ -195,16 +200,16 @@ void omxRAMExpectation::CalculateRAMCovarianceAndMeans(FitContext *fc)
 	if (F->rows == 0) return;
 
 	if (slope) omxRecompute(slope, fc);
-	    
+
 	if(cov == NULL && means == NULL) {
 		return; // We're not populating anything, so why bother running the calculation?
 	}
-	
+
 	EigenMatrixAdaptor Ecov(cov);
 	pcalc.cov(fc, Ecov);
 
 	if(OMX_DEBUG_ALGEBRA) {omxPrintMatrix(cov, "....RAM: Model-implied Covariance Matrix:");}
-	
+
 	if (M) {
 		EigenVectorAdaptor Emean(means);
 		pcalc.mean(fc, Emean);
@@ -235,7 +240,7 @@ static void recordNonzeroCoeff(omxMatrix *m, std::vector<coeffLoc> &vec, bool lo
 }
 
 omxRAMExpectation::omxRAMExpectation(omxState *st, int num)
-	: super(st, num), slope(0)
+	: super(st, num), studiedF(false), openBox(false), numExoPred(0), slope(0), rram(0)
 {
 	if (st->isClone()) {
 		omxRAMExpectation *ram = (omxRAMExpectation *)st->getParent(this);
@@ -248,18 +253,20 @@ omxRAMExpectation::omxRAMExpectation(omxState *st, int num)
 	}
 }
 
-void omxRAMExpectation::init() {
-	if(OMX_DEBUG) { mxLog("Initializing RAM expectation."); }
-	
+void omxRAMExpectation::init()
+{
+	loadDataColFromR();
+	loadThresholdFromR();
+
 	int l, k;
 
 	omxRAMExpectation *RAMexp = this;
 	RAMexp->rram = 0;
-	
+
 	auto oo=this;
 
 	oo->canDuplicate = true;
-	
+
 	ProtectedSEXP Rverbose(R_do_slot(rObj, Rf_install("verbose")));
 	RAMexp->verbose = Rf_asInteger(Rverbose) + OMX_DEBUG;
 
@@ -340,7 +347,7 @@ void omxRAMExpectation::init() {
 				mxLog("%s: join col %d against %s using between matrix %s",
 				      oo->name, foreignKey, fex->name, bmat->name());
 			}
-				
+
 			RAMexp->between.push_back(bmat);
 		}
 	}
@@ -357,31 +364,47 @@ void omxRAMExpectation::init() {
 	isProductNode.assign(A->cols, false);
 	if (R_has_slot(rObj, Rf_install("isProductNode"))) {
 		ProtectedSEXP RprodNode(R_do_slot(rObj, Rf_install("isProductNode")));
-		if (Rf_length(RprodNode)) {
-			if (Rf_length(RprodNode) != A->cols) {
-				mxThrow("isProductNode must be same dimension as A matrix");
-			}
-			for (int px = 0; px < A->cols; ++px) {
-				if (INTEGER(RprodNode)[px]) {
-					isProductNode[px] = true;
-					hasProductNodes = true;
-				}
-			}
-		}
+    // PPML can transform a model such that RprodNode length does
+    // not match A->cols
+    if (Rf_length(RprodNode) == A->cols) {
+      for (int px = 0; px < A->cols; ++px) {
+        if (INTEGER(RprodNode)[px]) {
+          isProductNode[px] = true;
+          hasProductNodes = true;
+        }
+      }
+    }
 	}
 
 	if(OMX_DEBUG) { mxLog("Generating internals for computation."); }
 
 	cov = omxNewMatrixFromSlotOrAnon(rObj, currentState, "expectedCovariance", l, l);
 	if (!cov->hasMatrixNumber) covOwner = omxMatrixPtr(cov);
-	else connectMatrixToExpectation(cov, this, "covariance");
+	else {
+    connectMatrixToExpectation(cov, this, "covariance");
+    openBox = true;
+  }
+  fullCov = omxNewMatrixFromSlot(rObj, currentState, "expectedFullCovariance");
+  if (fullCov) {
+    connectMatrixToExpectation(fullCov, this, "fullCov");
+    openBox = true;
+  }
 
 	if (M) {
 		means =	omxNewMatrixFromSlotOrAnon(rObj, currentState, "expectedMean", 1, l);
 		if (!means->hasMatrixNumber) meanOwner = omxMatrixPtr(means);
-		else connectMatrixToExpectation(means, this, "mean");
+		else {
+      connectMatrixToExpectation(means, this, "mean");
+      openBox = true;
+    }
+    fullMean = omxNewMatrixFromSlot(rObj, currentState, "expectedFullMean");
+    if (fullMean) {
+      connectMatrixToExpectation(fullMean, this, "fullMean");
+      openBox = true;
+    }
 	} else {
 		RAMexp->means  = 	NULL;
+    fullMean = 0;
 	}
 
 	RAMexp->studyF();
@@ -407,15 +430,57 @@ void omxRAMExpectation::init() {
 		sio->S0 = S;
 
 		pcalc.attach(k, l, latentFilter, isProductNode, mio, aio, sio);
+
+    omxMatrix *selVec = omxNewMatrixFromSlot(rObj, currentState, "selectionVector");
+    if (selVec) {
+      DataFrame selPlan(rObj.slot("selectionPlan"));
+      pcalc.attachSelection(selVec, selPlan);
+    }
+    pcalc.attachFullMemory(fullMean, fullCov);
 		pcalc.setAlgo(0, hasProductNodes, useSparse);
 
 		currentState->restoreParam(estSave);
 	}
+  if (!currentState->isTopState()) {
+    auto pex = (omxRAMExpectation*) currentState->getParent(this);
+    if (pex->slope) {
+      numExoPred = pex->numExoPred;
+      exoDataColIndex = pex->exoDataColIndex;
+      addSlopeMatrix();
+    }
+  }
 }
 
 void omxRAMExpectation::studyExoPred()
 {
-	if (data->defVars.size() == 0 || !M || !M->isSimple() || !S->isSimple()) return;
+  if (!currentState->isTopState()) return; // done in init(); defVar already modified
+  // We check each condition separately so we can log the outcome, if requested.
+	if (data->defVars.size() == 0) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: no def vars", name);
+    return;
+  }
+  if (verbose >= 1) mxLog("%s::studyExoPred: found %d def vars",
+                          name, int(data->defVars.size()));
+  if (!M) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: no means", name);
+    return;
+  }
+  if (M->isAlgebra()) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: means model is an algebra", name);
+    return;
+  }
+  if (M->populateDependsOnDefinitionVariables()) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: means model depends on def vars", name);
+    return;
+  }
+  if (S->isAlgebra()) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: S (covariance) is an algebra", name);
+    return;
+  }
+  if (S->populateDependsOnDefinitionVariables()) {
+    if (verbose >= 1) mxLog("%s::studyExoPred: S (covariance) depends on def vars", name);
+    return;
+  }
 
 	Eigen::VectorXd estSave;
 	currentState->setFakeParam(estSave);
@@ -425,8 +490,7 @@ void omxRAMExpectation::studyExoPred()
 	EigenMatrixAdaptor eS(S);
 	hasVariance = eS.diagonal().array().abs().matrix();
 
-	int found = 0;
-	std::vector<int> exoDataCol(S->rows, -1);
+  exoDataColIndex.resize(S->rows, -1);
 	int mNum = ~M->matrixNumber;
 	for (int k=0; k < int(data->defVars.size()); ++k) {
 		omxDefinitionVar &dv = data->defVars[k];
@@ -438,11 +502,15 @@ void omxRAMExpectation::studyExoPred()
 				if (latentFilter[cx]) toManifest = true;
 				else latentName = S->colnames[cx];
 			}
-			if (!toManifest && !latentName) continue;
+			if (!toManifest && !latentName) {
+        if (verbose >= 1) mxLog("%s::studyExoPred: def var '%s' has unknown effect",
+                                name, data->columnName(dv.column));
+        continue;
+      }
 			if (latentName) mxThrow("%s: latent exogenous variables are not supported (%s -> %s)", name,
 						 S->colnames[dv.col], latentName);
-			exoDataCol[dv.col] = dv.column;
-			found += 1;
+			exoDataColIndex[dv.col] = dv.column;
+			numExoPred += 1;
 			dv.loadData(currentState, 0.);
 			if (OMX_DEBUG + verbose >= 1) {
 				mxLog("%s: set defvar '%s' for latent '%s' to exogenous mode",
@@ -454,21 +522,26 @@ void omxRAMExpectation::studyExoPred()
 
 	currentState->restoreParam(estSave);
 
-	if (!found) return;
+  addSlopeMatrix();
+}
 
-	slope = omxInitMatrix(F->rows, found, currentState);
+void omxRAMExpectation::addSlopeMatrix()
+{
+	if (!numExoPred) return;
+
+	slope = omxInitMatrix(F->rows, numExoPred, currentState);
 	EigenMatrixAdaptor eSl(slope);
 	eSl.setZero();
 
 	for (int cx=0, ex=0; cx < S->rows; ++cx) {
-		if (exoDataCol[cx] == -1) continue;
-		auto &rc = data->rawCol( exoDataCol[cx] );
+		if (exoDataColIndex[cx] == -1) continue;
+		auto &rc = data->rawCol( exoDataColIndex[cx] );
 		if (rc.type != COLUMNDATA_NUMERIC) {
 			omxRaiseErrorf("%s: exogenous predictor '%s' must be type numeric (not '%s')",
 				       name, rc.name, rc.typeName());
 			continue;
 		}
-		exoDataColumns.push_back(exoDataCol[cx]);
+		exoDataColumns.push_back(exoDataColIndex[cx]);
 		for (int rx=0, dx=0; rx < S->rows; ++rx) {
 			if (!latentFilter[rx]) continue;
 			slope->addPopulate(A, rx, cx, dx, ex);
@@ -476,11 +549,16 @@ void omxRAMExpectation::studyExoPred()
 		}
 		ex += 1;
 	}
+}
+
+void omxRAMExpectation::connectToData()
+{
+  super::connectToData();
 
 	exoPredMean.resize(exoDataColumns.size());
 	for (int cx=0; cx < int(exoDataColumns.size()); ++cx) {
 		auto &e1 = data->rawCol( exoDataColumns[cx] );
-		Eigen::Map< Eigen::VectorXd > vec(e1.ptr.realData, data->numRawRows());
+		Eigen::Map< Eigen::VectorXd > vec(e1.d(), data->numRawRows());
 		exoPredMean[cx] = vec.mean();
 	}
 }
@@ -508,12 +586,18 @@ void omxRAMExpectation::studyF()
 			dataCols[dx] = newDest;
 			if (origThresholdInfo.size()) {
 				omxThresholdColumn adj = origThresholdInfo[dest];
-				adj.dColumn = dx;
+				adj.dataColumn = newDest;
 				thresholds.push_back(adj);
 			}
 			dx += 1;
 		}
 	}
+  // ensure we don't try to use the wrong stuff
+  origDataColumnNames.clear();
+  origThresholdInfo.clear();
+  studiedF = true;
+  //mxLog("studyF: dataColumns permuted to (%d):", int(dataColNames.size()));
+  //for (auto &dc : dataColNames) mxLog("%s", dc);
 }
 
 omxMatrix* omxRAMExpectation::getComponent(const char* component)
@@ -533,7 +617,7 @@ omxMatrix* omxRAMExpectation::getComponent(const char* component)
 	} else if(strEQ("pvec", component)) {
 		// Once implemented, change compute function and return pvec
 	}
-	
+
 	return retval;
 }
 
@@ -566,7 +650,7 @@ void omxRAMExpectation::analyzeDefVars(FitContext *fc)
 		dvInfluenceMean.assign(numDefVars, M? true : false);
 		if (verbose >= 1) mxLog("%s: defvar effect on mean unknown", name);
 	}
-		
+
 	hasVariance.resize(S->rows);
 	if (S->isSimple()) {
 		omxRecompute(S, fc);
@@ -884,8 +968,8 @@ namespace RelationalRAMExpectation {
 			rowToLayoutMap[ std::make_pair(data, frow) ] = -1;
 		}
 
-		struct addr a1;
-		struct addrSetup as1;
+		class addr a1;
+		class addrSetup as1;
 		as1.clumped = false;
 		a1.ig = 0;
 		as1.parent1 = NA_INTEGER;
@@ -1227,7 +1311,7 @@ namespace RelationalRAMExpectation {
 						      hasMean, hasVar);
 					}
 				}
-				
+
 				if (hasMean || hasVar) {
 					for (auto it : fromDv) {
 						omxRAMExpectation *dvHome =
@@ -1701,7 +1785,7 @@ namespace RelationalRAMExpectation {
 				std::vector<int> dc(a1.clump);
 				std::sort(dc.begin(), dc.end(), RampartClumpCompare(this));
 				for (size_t cx=0; cx < dc.size(); ++cx) {
-					if (dc[cx] != a1.clump[cx]) mxThrow("oops");
+					if (dc[cx] != a1.clump[cx]) OOPS;
 				}
 			}
 		}
@@ -1870,6 +1954,17 @@ namespace RelationalRAMExpectation {
 		hasProductNodes = false;
 		for (auto *ex : allEx) {
 			omxRAMExpectation *ram1 = (omxRAMExpectation*) ex;
+      if (ram1->isOpenBox()) {
+        if (ex != homeEx) {
+          mxThrow("Expectation '%s' is part of a multilevel model with '%s';"
+                  "introspection of per-level expectations is not possible",
+                  ex->name, homeEx->name);
+        } else {
+          mxThrow("Expectation '%s' is part of a multilevel model;"
+                  "introspection of per-level expectations is not possible",
+                  ex->name);
+        }
+      }
 			hasProductNodes |= ram1->getHasProductNodes();
 			if (!ex->data->hasWeight() && !ex->data->hasFreq()) continue;
 			mxThrow("%s: row frequencies or weights provided in '%s' are not compatible with joins",
@@ -1967,7 +2062,7 @@ namespace RelationalRAMExpectation {
 		for (std::vector<independentGroup*>::iterator it = group.begin() ; it != group.end(); ++it) {
 			delete *it;
 		}
-  
+
 		omxFreeMatrix(smallCol);
 	}
 
