@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2020 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@
 #include "omxState.h"
 #include "Compute.h"
 #include "omxImportFrontendState.h"
+#include "npsolswitch.h"
 #include "EnableWarnings.h"
 
 struct omxGlobal *Global = NULL;
@@ -237,6 +238,11 @@ omxGlobal::omxGlobal()
 	userInterrupted = false;
 	lastIndexDone=0;
 	lastIndexDoneTime=0;
+  NPSOL_HACK = 0;
+
+  gradientAlgo = GradientAlgorithm_Auto;
+  gradientIter = 1;
+  gradientStepSize = NA_REAL;
 
 	RAMInverseOpt = true;
 	RAMMaxDepth = 30;
@@ -256,6 +262,29 @@ omxGlobal::omxGlobal()
 	checkpointColnames.reserve(100);
 }
 
+void omxGlobal::setDefaultGradientAlgo()
+{
+  if (gradientAlgo == GradientAlgorithm_Auto) {
+    if (engine == OptEngine_CSOLNP || engine == OptEngine_SD) {
+      gradientAlgo = GradientAlgorithm_Forward;
+    } else {
+      gradientAlgo = GradientAlgorithm_Central;
+    }
+  }
+
+  if (!std::isfinite(gradientStepSize)) {
+    gradientStepSize = 1e-7;
+    if (engine == OptEngine_NLOPT) {
+      gradientStepSize *= GRADIENT_FUDGE_FACTOR(2.0);
+    }
+  }
+
+  if (OMX_DEBUG) {
+    mxLog("omxGlobal::setDefaultGradientAlgo algo=%d iter=%d stepsize=%g",
+          gradientAlgo, gradientIter, gradientStepSize);
+  }
+}
+
 void omxState::restoreParam(const Eigen::Ref<const Eigen::VectorXd> point)
 {
 	if (!hasFakeParam) mxThrow("Cannot restore; fake parameters not loaded");
@@ -276,6 +305,7 @@ omxMatrix *omxState::getMatrixFromIndex(int matnum) const
 
 omxMatrix *omxState::lookupDuplicate(omxMatrix *element) const
 {
+  if (!element) return 0;
 	if (!element->hasMatrixNumber) mxThrow("lookupDuplicate without matrix number");
 	return getMatrixFromIndex(element->matrixNumber);
 }
@@ -294,7 +324,7 @@ omxExpectation *omxState::lookupDuplicate(omxExpectation *element) const
 
 void omxState::setWantStage(int stage)
 {
-	if (wantStage == stage) mxThrow("omxState::setWantStage(%d) is redundent", stage);
+	if (wantStage == stage) mxThrow("omxState::setWantStage(%d) is redundant", stage);
 	wantStage = stage;
 	if (OMX_DEBUG) mxLog("wantStage set to 0x%x", stage);
 }
@@ -407,7 +437,8 @@ void omxState::loadDefinitionVariables(bool start)
 	}
 }
 
-omxState::omxState(omxState *src) : wantStage(0), parent(src), hasFakeParam(false)
+omxState::omxState(omxState *src, bool isTeam) :
+  wantStage(0), parent(src), workBoss(isTeam? src : 0), hasFakeParam(false)
 {
 	init();
 
@@ -440,8 +471,6 @@ omxState::omxState(omxState *src) : wantStage(0), parent(src), hasFakeParam(fals
 	for (size_t xx=0; xx < src->conListX.size(); ++xx) {
 		conListX.push_back(src->conListX[xx]->duplicate(this));
 	}
-
-	usingAnalyticJacobian = src->usingAnalyticJacobian;
 }
 
 void omxState::initialRecalc(FitContext *fc)
@@ -460,10 +489,6 @@ void omxState::initialRecalc(FitContext *fc)
 		omxFitFunctionCompute(matrix->fitFunction, FF_COMPUTE_INITIAL_FIT, fc);
 	}
 
-	for (size_t xx=0; xx < conListX.size(); ++xx) {
-		conListX[xx]->prep(fc);
-	}
-
 	setWantStage(FF_COMPUTE_FIT);
 }
 
@@ -515,6 +540,28 @@ void omxState::connectToData()
 	}
 }
 
+void omxState::reportConstraints(MxRList &out)
+{
+	if (!conListX.size()) return;
+
+  SEXP cn, cr, cc;
+
+  Rf_protect(cn = Rf_allocVector( STRSXP, conListX.size() ));
+  Rf_protect(cr = Rf_allocVector( INTSXP, conListX.size() ));
+  Rf_protect(cc = Rf_allocVector( INTSXP, conListX.size() ));
+  for(int i=0; i < int(conListX.size()); i++){
+    auto &con = *conListX[i];
+    SET_STRING_ELT( cn, i, Rf_mkChar(con.name) );
+    int rows, cols;
+    con.getDim(&rows, &cols);
+    INTEGER(cr)[i] = rows;
+    INTEGER(cc)[i] = cols;
+  }
+  out.add("constraintNames", cn);
+  out.add("constraintRows", cr);
+  out.add("constraintCols", cc);
+}
+
 omxState::~omxState()
 {
 	if(OMX_DEBUG) { mxLog("Freeing %d Constraints.", (int) conListX.size());}
@@ -870,7 +917,7 @@ void omxRaiseError(const char* msg) { // DEPRECATED
 	omxRaiseErrorf("%s", msg);
 }
 
-void omxGlobal::checkpointMessage(FitContext *fc, double *est, const char *fmt, ...)
+void omxGlobal::checkpointMessage(FitContext *fc, const char *fmt, ...)
 {
 	std::string str;
 	va_list ap;
@@ -881,48 +928,77 @@ void omxGlobal::checkpointMessage(FitContext *fc, double *est, const char *fmt,
 	if (OMX_DEBUG) mxLog("checkpointMessage: %s", str.c_str());
 
 	for(size_t i = 0; i < checkpointList.size(); i++) {
-		checkpointList[i]->message(fc, est, str.c_str());
+		checkpointList[i]->message(fc, str.c_str());
 	}
 }
 
-void omxGlobal::checkpointPostfit(const char *callerName, FitContext *fc, double *est, bool force)
+void omxGlobal::checkpointPostfit(const char *callerName, FitContext *fc, bool force)
 {
 	for(size_t i = 0; i < checkpointList.size(); i++) {
-		checkpointList[i]->postfit(callerName, fc, est, force);
+		checkpointList[i]->postfit(callerName, fc, force);
 	}
 }
 
+void omxConstraint::setInitialSize(int sz)
+{
+  origSize = sz;
+	size = sz;
+  redundant.assign(size, false);
+  seenActive.assign(size, false);
+	if (sz == 0) {
+		Rf_warning("Constraint '%s' evaluated to a 0x0 matrix and "
+               "will have no effect", name);
+	}
+
+  int maxVars = Global->findVarGroup(FREEVARGROUP_ALL)->vars.size();
+  initialJac.resize(origSize, maxVars);
+  initialJac.setConstant(NA_REAL);
+}
+
 void UserConstraint::prep(FitContext *fc)
 {
 	refresh(fc);
-	nrows = pad->rows;
-	ncols = pad->cols;
-	size = nrows * ncols;
-	if (size == 0) {
-		Rf_warning("Constraint '%s' evaluated to a 0x0 matrix and will have no effect", name);
-	}
-	omxAlgebraPreeval(pad, fc);
+  setInitialSize(pad->rows * pad->cols);
+
 	if(jacobian){
 		jacMap.resize(jacobian->cols);
-		std::vector<const char*> *jacColNames = &jacobian->colnames;
-		for (size_t nx=0; nx < jacColNames->size(); ++nx) {
-			int to = fc->varGroup->lookupVar((*jacColNames)[nx]);
+		auto &jacColNames = jacobian->colnames;
+		for (size_t nx=0; nx < jacColNames.size(); ++nx) {
+			int to = fc->varGroup->lookupVar(jacColNames[nx]);
+      if (strict && to < 0) {
+        mxThrow("Constraint '%s' has a Jacobian entry for unrecognized "
+                "parameter '%s'. If this is not an mistake and you "
+                "have merely fixed this parameter then you can "
+                "use the strict=FALSE argument to mxConstraint "
+                "to turn off this precautionary check", name, jacColNames[nx]);
+      }
 			jacMap[nx] = to;
 		}
 	}
 }
 
-UserConstraint::UserConstraint(FitContext *fc, const char *_name, omxMatrix *arg1, omxMatrix *arg2, omxMatrix *jac, int lin) :
-	super(_name)
+void UserConstraint::preeval(FitContext *fc)
+{
+	omxRecompute(pad, fc);
+}
+
+UserConstraint::UserConstraint(FitContext *fc, const char *_name, omxMatrix *arg1,
+                               omxMatrix *arg2, omxMatrix *jac, int _verbose) :
+	super(_name), verbose(_verbose)
 {
 	omxState *state = fc->state;
 	omxMatrix *args[2] = {arg1, arg2};
 	pad = omxNewAlgebraFromOperatorAndArgs(10, args, 2, state); // 10 = binary subtract
 	jacobian = jac;
-	linear = lin;
 }
 
-omxConstraint *UserConstraint::duplicate(omxState *dest)
+void UserConstraint::getDim(int *rowsOut, int *colsOut) const
+{
+  *rowsOut = pad->rows;
+  *colsOut = pad->cols;
+}
+
+omxConstraint *UserConstraint::duplicate(omxState *dest) const
 {
 	omxMatrix *args[2] = {
 		dest->lookupDuplicate(pad->algebra->algArgs[0]),
@@ -931,24 +1007,51 @@ omxConstraint *UserConstraint::duplicate(omxState *dest)
 
 	UserConstraint *uc = new UserConstraint(name);
 	uc->opCode = opCode;
+  uc->redundant = redundant;
+  uc->size = size;
 	uc->pad = omxNewAlgebraFromOperatorAndArgs(10, args, 2, dest); // 10 = binary subtract
 	uc->jacobian = jacobian;
-	uc->linear = linear;
+  uc->jacMap = jacMap;
+  uc->verbose = verbose;
 	return uc;
 }
 
-void UserConstraint::refreshAndGrab(FitContext *fc, Type ineqType, double *out)
+void UserConstraint::refreshAndGrab(FitContext *fc, double *out)
 {
-	fc->incrComputeCount();
 	refresh(fc);
 
-	for(int k = 0; k < size; k++) {
+	for(int k = 0, d = 0; k < int(redundant.size()); k++) {
+    if (redundant[k]) continue;
 		double got = pad->data[k];
-		if (opCode != ineqType) got = -got;
-		out[k] = got;
+    // ineq constraints are always translated to LESS_THAN
+		if (opCode == GREATER_THAN) got = -got;
+		out[d++] = got;
 	}
 }
 
+void UserConstraint::analyticJac(FitContext *fc, MatrixStoreFn out)
+{
+  if (!jacobian) return;
+
+  omxRecompute(jacobian, fc);
+  EigenArrayAdaptor cj(jacobian);
+
+  for (int rx=0, dx=0; rx < int(redundant.size()); ++rx) {
+    if (redundant[rx]) continue;
+
+    for (int c=0; c < jacobian->cols; c++){
+      if (jacMap[c] < 0) continue;
+      out(dx, jacMap[c], cj(rx, c));
+    }
+    dx += 1;
+  }
+}
+
+int UserConstraint::getVerbose() const
+{
+  return verbose;
+}
+
 UserConstraint::~UserConstraint()
 {
 	omxFreeMatrix(pad);
@@ -957,7 +1060,218 @@ UserConstraint::~UserConstraint()
 void UserConstraint::refresh(FitContext *fc)
 {
 	omxRecompute(pad, fc);
-	//omxRecompute(jacobian, fc); //<--Not sure if Jacobian needs to be recomputed every time constraint function does.
+}
+
+void omxConstraint::recalcSize()
+{
+  size = std::count(redundant.begin(), redundant.end(), false);
+}
+
+ConstraintVec::ConstraintVec(FitContext *fc, const char *_name,
+                             ConstraintVec::ClassifyFn _cf) :
+  name(_name), cf(_cf), ineqAlwaysActive(false)
+{
+  verbose = 0;
+  count = 0;
+  anyAnalyticJacCache = false;
+  auto &conList = fc->state->conListX;
+  for (int j=0; j < int(conList.size()); j++) {
+    omxConstraint &con = *conList[j];
+    if (!cf(con)) continue;
+    count += con.size;
+    verbose = std::max(verbose, con.getVerbose());
+    anyAnalyticJacCache |= con.hasAnalyticJac(fc);
+  }
+  verifyJac = verbose >= 3;
+}
+
+void ConstraintVec::allocJacTool(FitContext *fc)
+{
+  if (jacTool) return;
+  jacTool =
+    std::unique_ptr< AutoTune<JacobianGadget> >(new AutoTune<JacobianGadget>(name));
+  jacTool->setWork(std::unique_ptr<JacobianGadget>(new JacobianGadget(fc->getNumFree())));
+  jacTool->setMaxThreads(fc->numOptimizerThreads());
+  if (verbose >= 1) mxLog("%s: allocJacTool count=%d", name, getCount());
+  if (verifyJac) mxLog("%s: constraint Jacobian verification enabled", name);
+}
+
+// This is called once before the compute plan begins execution
+void ConstraintVec::markUselessConstraints(FitContext *fc)
+{
+  if (!count) return;
+
+  Eigen::ArrayXd constr(count);
+  Eigen::ArrayXXd ej(count, fc->getNumFree());
+  ej.setConstant(NA_REAL);
+  eval(fc, constr.data(), ej.data());
+
+  //mxPrintMat("ej", ej);
+  auto *state = fc->state;
+
+	for (int j=0, cur=0, d1=0; j < int(state->conListX.size()); j++) {
+		omxConstraint &con = *state->conListX[j];
+    if (!cf(con)) continue;
+    if (con.opCode != omxConstraint::EQUALITY) OOPS;
+    for (int kk=0; kk < con.size; ++kk) {
+      if ((ej.row(cur+kk) == 0.0).all()) {
+        con.redundant[kk] = true;
+        count -= 1;
+        if (con.getVerbose()) {
+          mxLog("Ignoring constraint '%s[%d]' because it does not depend "
+                "on any free parameters", con.name, 1+kk);
+        }
+      }
+      if (d1 < cur+kk) ej.row(d1) = ej.row(cur+kk);
+      if (!con.redundant[kk]) d1 += 1;
+    }
+    cur += con.size;
+    con.recalcSize();
+  }
+
+  if (count <= 1) return;
+
+  Eigen::MatrixXd tmp = ej.block(0,0,count,ej.cols()).transpose();
+  Eigen::FullPivHouseholderQR<Eigen::MatrixXd> qrOp(tmp);
+  Eigen::ArrayXi perm = qrOp.colsPermutation().indices();
+  //mxPrintMat("perm", perm);
+  Eigen::MatrixXd qr = qrOp.matrixQR();
+  //mxPrintMat("qr", qr);
+  double thr = qrOp.maxPivot() * qrOp.threshold();
+
+	for (int j=0, cur=0; j < int(state->conListX.size()); j++) {
+		omxConstraint &con = *state->conListX[j];
+    if (!cf(con)) continue;
+    if (con.opCode != omxConstraint::EQUALITY) OOPS;
+    for (int kk=0, dx=0; kk < int(con.redundant.size()); ++kk) {
+      if (con.redundant[kk]) continue;
+      int xx = perm[cur+dx];
+      if (abs(qr(xx,xx)) < thr) {
+        con.redundant[kk] = true;
+        if (con.getVerbose()) {
+          mxLog("Ignoring constraint '%s[%d]' because it is redundant",
+                con.name, 1+kk);
+        }
+      }
+    }
+    cur += con.size;
+    con.recalcSize();
+  }
+}
+
+void ConstraintVec::eval(FitContext *fc, double *constrOut, double *jacOut)
+{
+	if (count==0) return;
+
+  auto *state = fc->state;
+  Eigen::Map< Eigen::ArrayXd > constr(constrOut, count);
+  Eigen::Map< Eigen::ArrayXXd > constrJac(jacOut, count, fc->getNumFree());
+  if (!constrOut && jacOut) mxThrow("Can't request jacOut without constrOut");
+  if (!constrOut) OOPS;
+
+	for (int j=0, cur=0; j < int(state->conListX.size()); j++) {
+		omxConstraint &con = *state->conListX[j];
+		if (!cf(con)) continue;
+
+    con.refreshAndGrab(fc, &constr(cur));
+
+    if (jacOut) {
+      auto &vars = fc->varGroup->vars;
+      for (int kk=0, dx=0; kk < int(con.redundant.size()); ++kk) {
+        if (con.redundant[kk]) continue;
+        for (int vx=0, px=0; vx < int(vars.size()); ++vx) {
+          if (fc->profiledOutZ[vx]) continue;
+          constrJac(cur+dx, px++) = con.initialJac(kk, vars[vx]->id);
+        }
+        dx += 1;
+      }
+
+      con.analyticJac(fc, [&constrJac, cur](int r, int c, double val){
+                            constrJac(cur+r,c) = val; });
+    }
+    if (con.opCode != omxConstraint::EQUALITY) {
+      if (ineqAlwaysActive) {
+        // For CSOLNP, inequality constraints are always active.
+        // This is by design, since it's an interior-point algorithm.
+      } else {
+        // NPSOL/SLSQP require inactive inequality constraint to be held constant at zero:
+        for (int cx=cur; cx < cur+con.size; ++cx) {
+          constr[cx] = std::max(0.0, constr[cx]);
+          if(jacOut && constr[cx] == 0) {
+            /*The Jacobians of each inactive constraint are set to zero here;
+              as their elements will be zero rather than NaN,
+              the code in finiteDifferences.h will leave them alone:*/
+            constrJac.row(cx).setZero();
+          }
+        }
+      }
+    }
+		cur += con.size;
+	}
+
+	fc->incrComputeCount();
+
+  if (jacOut) {
+    Eigen::ArrayXXd analyticJac;
+    if (verifyJac) {
+      analyticJac = constrJac;
+      constrJac.setConstant(NA_REAL);
+    }
+
+    allocJacTool(fc);
+    (*jacTool)([&](double *myPars, int thrId, auto &result) {
+                FitContext *fc2 = thrId >= 0? fc->childList[thrId] : fc;
+                Eigen::Map< Eigen::VectorXd > Est(myPars, fc2->getNumFree());
+                // Only 1 parameter is different so we could
+                // update only that parameter instead of all
+                // of them.
+                fc2->setEstFromOptimizer(Est);
+                eval(fc2, result.data(), 0);
+              }, constr, [&fc](){ return fc->getCurrentFree(); }, true, constrJac);
+
+    if (verifyJac) {
+      for (int cx=0; cx < analyticJac.cols(); ++cx) {
+        for (int rx=0; rx < analyticJac.rows(); ++rx) {
+          if (std::isfinite(analyticJac(rx,cx))) continue;
+          analyticJac(rx,cx) = constrJac(rx,cx);
+        }
+      }
+
+      for (int rx=0; rx < constrJac.rows(); ++rx) {
+        Eigen::ArrayXd diff = analyticJac.row(rx) - constrJac.row(rx);
+        if (diff.abs().maxCoeff() < 1e-7) continue;
+        std::string info = string_snprintf("%s[%d,]-true", name, rx);
+        mxPrintMat(info.c_str(), diff);
+      }
+      // restore
+      constrJac = analyticJac;
+    }
+
+    for (int j=0, cur=0; j < int(state->conListX.size()); j++) {
+      omxConstraint &con = *state->conListX[j];
+      if (!cf(con)) continue;
+      auto &vars = fc->varGroup->vars;
+      for (int kk=0, dx=0; kk < int(con.redundant.size()); ++kk) {
+        if (con.redundant[kk]) continue;
+        if (constr[cur+dx] != 0.0 && !con.seenActive[kk]) {
+          for (int vx=0, px=0; vx < int(vars.size()); ++vx) {
+            if (fc->profiledOutZ[vx]) continue;
+            if (constrJac(cur+dx, px) == 0.0) {
+              con.initialJac(kk, vars[vx]->id) = 0;
+              if (con.getVerbose() >= 2) {
+                mxLog("Assuming Jacobian %s[%d,%s] is zero",
+                      con.name, 1+kk, vars[vx]->name);
+              }
+            }
+            px+=1;
+          }
+          con.seenActive[kk] = true;
+        }
+        dx += 1;
+      }
+      cur += con.size;
+    }
+  }
 }
 
 omxCheckpoint::omxCheckpoint() : wroteHeader(false), lastCheckpoint(0), lastIterations(0),
@@ -989,12 +1303,12 @@ void omxCheckpoint::omxWriteCheckpointHeader()
 	wroteHeader = true;
 }
 
-void omxCheckpoint::message(FitContext *fc, double *est, const char *msg)
+void omxCheckpoint::message(FitContext *fc, const char *msg)
 {
-	postfit(msg, fc, est, true);
+	postfit(msg, fc, true);
 }
 
-void omxCheckpoint::postfit(const char *context, FitContext *fc, double *est, bool force)
+void omxCheckpoint::postfit(const char *context, FitContext *fc, bool force)
 {
 	const int timeBufSize = 32;
 	char timeBuf[timeBufSize];
@@ -1022,7 +1336,7 @@ void omxCheckpoint::postfit(const char *context, FitContext *fc, double *est, bo
 		size_t numParam = Global->findVarGroup(FREEVARGROUP_ALL)->vars.size();
 		for (size_t px=0; px < numParam; ++px) {
 			if (lx < vars.size() && vars[lx]->id == (int)px) {
-				fprintf(file, "\t%.10g", est[lx]);
+				fprintf(file, "\t%.10g", fc->est[lx]);
 				++lx;
 			} else {
 				fprintf(file, "\tNA");
@@ -1103,3 +1417,24 @@ double omxFreeVar::getCurValue(omxState *os)
 	EigenMatrixAdaptor Emat(os->matrixList[loc.matrix]);
 	return Emat(loc.row, loc.col);
 }
+
+OptEngine nameToGradOptEngine(const char *engineName)
+{
+  OptEngine engine;
+	if (strEQ(engineName, "CSOLNP")) {
+		engine = OptEngine_CSOLNP;
+	} else if (strEQ(engineName, "SLSQP")) {
+		engine = OptEngine_NLOPT;
+	} else if (strEQ(engineName, "NPSOL")) {
+#if HAS_NPSOL
+		engine = OptEngine_NPSOL;
+#else
+		mxThrow("NPSOL is not available in this build. See ?omxGetNPSOL() to download this optimizer");
+#endif
+	} else if(strEQ(engineName, "SD")){
+		engine = OptEngine_SD;
+	} else {
+		mxThrow("Gradient descent engine '%s' is not recognized", engineName);
+	}
+  return engine;
+}
