@@ -1,5 +1,5 @@
  /*
- *  Copyright 2007-2019 by the individuals mentioned in the source code history
+ *  Copyright 2007-2020 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -28,9 +28,7 @@ struct omxWLSFitFunction : omxFitFunction {
 	omxMatrix* expectedFlattened;
 	omxMatrix* P;
 	omxMatrix* B;
-	int numOrdinal;
-	int vectorSize;
-	
+
 	const char *type;
 	const char *continuousType;
 	bool fullWeight;
@@ -41,9 +39,7 @@ struct omxWLSFitFunction : omxFitFunction {
 	virtual void init();
 	virtual void compute(int ffcompute, FitContext *fc);
 	virtual void populateAttr(SEXP algebra);
-	
-	void flattenDataToVector(omxMatrix* cov, omxMatrix* means, omxMatrix *slope, omxMatrix *thresholdMat,
-				 std::vector< omxThresholdColumn > &thresholds, omxMatrix* vector);
+
 	void prepData();
 	virtual void invalidateCache()
 	{
@@ -52,18 +48,10 @@ struct omxWLSFitFunction : omxFitFunction {
 	}
 };
 
-void omxWLSFitFunction::flattenDataToVector(omxMatrix* cov, omxMatrix* means, omxMatrix *slope,
-					    omxMatrix *thresholdMat,
-			 std::vector< omxThresholdColumn > &thresholds, omxMatrix* vector)
-{
-	EigenVectorAdaptor vec1(vector);
-	normalToStdVector(cov, means, slope, thresholdMat, numOrdinal, thresholds, vec1);
-}
-
 omxWLSFitFunction::~omxWLSFitFunction()
 {
 	if(OMX_DEBUG) {mxLog("Freeing WLS FitFunction.");}
-	
+
 	invalidateCache();
 	omxWLSFitFunction* owo = this;
 	omxFreeMatrix(owo->expectedFlattened);
@@ -71,24 +59,30 @@ omxWLSFitFunction::~omxWLSFitFunction()
 	omxFreeMatrix(owo->P);
 }
 
-
 void omxWLSFitFunction::compute(int want, FitContext *fc)
 {
 	auto *oo = this;
 	auto *owo = this;
 	if (want & FF_COMPUTE_INITIAL_FIT) return;
+
+	/* Recompute and recopy */
+	if(OMX_DEBUG) { mxLog("WLSFitFunction Computing expectation"); }
+	omxExpectationCompute(fc, expectation, NULL);
+
 	if ((want & FF_COMPUTE_PREOPTIMIZE) && !observedFlattened) {
 		prepData();
 		return;
 	}
-	
+
 	if(OMX_DEBUG) { mxLog("Beginning WLS Evaluation.");}
 	// Requires: Data, means, covariances.
-	
+
+  if (want & FF_COMPUTE_GRADIENT) invalidateGradient(fc);
+
 	double sum = 0.0;
-	
+
 	omxMatrix *eCov, *eMeans, *oFlat, *eFlat;
-	
+
 	/* Locals for readability.  Compiler should cut through this. */
 	eCov		= owo->expectedCov;
 	eMeans 		= owo->expectedMeans;
@@ -96,55 +90,61 @@ void omxWLSFitFunction::compute(int want, FitContext *fc)
 	if (!observedFlattened) return;
 	oFlat		= observedFlattened;
 	eFlat		= owo->expectedFlattened;
-	
-	/* Recompute and recopy */
-	if(OMX_DEBUG) { mxLog("WLSFitFunction Computing expectation"); }
-	omxExpectationCompute(fc, expectation, NULL);
-	
-	omxMatrix *expThresholdsMat = expectation->thresholdsMat;
-	
-	flattenDataToVector(eCov, eMeans, expectedSlope, expThresholdsMat, eThresh, eFlat);
-	
+
+	EigenVectorAdaptor EeFlat(eFlat);
+	omxExpectation *ex = expectation;
+	normalToStdVector(eCov, eMeans, expectedSlope,
+										[ex](int r, int c)->double{ return ex->getThreshold(r,c); },
+										eThresh, EeFlat);
+
 	omxCopyMatrix(B, oFlat);
-	
+
 	if(OMX_DEBUG) {omxPrintMatrix(eFlat, "....WLS Expected Vector: "); }
 	omxDAXPY(-1.0, eFlat, B);
 	//if(OMX_DEBUG) {omxPrintMatrix(B, "....WLS Observed - Expected Vector: "); }
-	
+
 	omxMatrix *weights = expectation->data->getSingleObsSummaryStats().acovMat;
 	if(weights != NULL) {
 		//if(OMX_DEBUG_ALGEBRA) {omxPrintMatrix(weights, "....WLS Weight Matrix: "); }
-		omxDGEMV(TRUE, 1.0, weights, B, 0.0, P);
+
+		if (units == FIT_UNITS_SQUARED_RESIDUAL) { // diagonally weighted
+      EigenMatrixAdaptor Ew(weights);
+      EigenVectorAdaptor EB(B);
+      EigenVectorAdaptor EP(P);
+      EP.derived() = Ew.diagonal().array() * EB.array();
+    } else {
+      omxDGEMV(TRUE, 1.0, weights, B, 0.0, P);
+    }
 	} else {
 		// ULS Case: Memcpy faster than dgemv.
 		omxCopyMatrix(P, B);
 		omxTransposeMatrix(P);
 	}
-	
+
 	sum = omxDDOT(P, B);
-	
+
 	oo->matrix->data[0] = sum;
-	
+
 	if(OMX_DEBUG) { mxLog("WLSFitFunction value comes to: %f.", oo->matrix->data[0]); }
-	
+
 }
 
 void omxWLSFitFunction::populateAttr(SEXP algebra)
 {
 	if(OMX_DEBUG) { mxLog("Populating WLS Attributes."); }
-	
+
 	omxWLSFitFunction *argStruct = this;
 	omxMatrix *expCovInt = argStruct->expectedCov;	    		// Expected covariance
 	omxMatrix *expMeanInt = argStruct->expectedMeans;			// Expected means
 	omxMatrix *weightInt = expectation->data->getSingleObsSummaryStats().acovMat;
-	
+
 	SEXP expCovExt, expMeanExt, gradients;
 	Rf_protect(expCovExt = Rf_allocMatrix(REALSXP, expCovInt->rows, expCovInt->cols));
 	for(int row = 0; row < expCovInt->rows; row++)
 		for(int col = 0; col < expCovInt->cols; col++)
 			REAL(expCovExt)[col * expCovInt->rows + row] =
 				omxMatrixElement(expCovInt, row, col);
-	
+
 	if (expMeanInt != NULL) {
 		Rf_protect(expMeanExt = Rf_allocMatrix(REALSXP, expMeanInt->rows, expMeanInt->cols));
 		for(int row = 0; row < expMeanInt->rows; row++)
@@ -152,9 +152,9 @@ void omxWLSFitFunction::populateAttr(SEXP algebra)
 				REAL(expMeanExt)[col * expMeanInt->rows + row] =
 					omxMatrixElement(expMeanInt, row, col);
 	} else {
-		Rf_protect(expMeanExt = Rf_allocMatrix(REALSXP, 0, 0));		
+		Rf_protect(expMeanExt = Rf_allocMatrix(REALSXP, 0, 0));
 	}
-	
+
 	if(OMX_DEBUG_ALGEBRA) {omxPrintMatrix(weightInt, "...WLS Weight Matrix: W"); }
 	SEXP weightExt = NULL;
 	if (weightInt) {
@@ -163,8 +163,8 @@ void omxWLSFitFunction::populateAttr(SEXP algebra)
 			for(int col = 0; col < weightInt->cols; col++)
 				REAL(weightExt)[col * weightInt->rows + row] = weightInt->data[col * weightInt->rows + row];
 	}
-	
-	
+
+
 	if(0) {  /* TODO fix for new internal API
 		int nLocs = Global->numFreeParams;
 		double gradient[Global->numFreeParams];
@@ -173,20 +173,20 @@ void omxWLSFitFunction::populateAttr(SEXP algebra)
 		}
 		//oo->gradientFun(oo, gradient);
 		Rf_protect(gradients = Rf_allocMatrix(REALSXP, 1, nLocs));
-		
+
 		for(int loc = 0; loc < nLocs; loc++)
 			REAL(gradients)[loc] = gradient[loc];
 		 */
 	} else {
 		Rf_protect(gradients = Rf_allocMatrix(REALSXP, 0, 0));
 	}
-	
+
 	if(OMX_DEBUG) { mxLog("Installing populated WLS Attributes."); }
 	Rf_setAttrib(algebra, Rf_install("expCov"), expCovExt);
 	Rf_setAttrib(algebra, Rf_install("expMean"), expMeanExt);
 	if (weightExt) Rf_setAttrib(algebra, Rf_install("weights"), weightExt);
 	Rf_setAttrib(algebra, Rf_install("gradients"), gradients);
-	
+
 	ProtectedSEXP Rsat(Rf_ScalarReal(0));
 	Rf_setAttrib(algebra, Rf_install("SaturatedLikelihood"), Rsat);
 	//Rf_setAttrib(algebra, Rf_install("IndependenceLikelihood"), Rf_ScalarReal(0));
@@ -202,17 +202,13 @@ void omxWLSFitFunction::prepData()
 	auto *oo = this;
 	auto *newObj = this;
 
-	if (vectorSize != expectation->numSummaryStats())
-		mxThrow("%s: vectorSize changed from %d -> %d",
-			name(), vectorSize, expectation->numSummaryStats());
-
 	omxData* dataMat = oo->expectation->data;
 
-	if (!matrix->currentState->isClone()) {
+	if (matrix->currentState->isTopState()) {
 		std::vector<int> exoPred;
 		expectation->getExogenousPredictors(exoPred);
 		// how to prohibit def vars? TODO
-	
+
 		dataMat->prepObsStats(matrix->currentState, expectation->getDataColumnNames(),
 				      exoPred, type, continuousType, fullWeight);
 		if (isErrorRaised()) return;
@@ -231,10 +227,9 @@ void omxWLSFitFunction::prepData()
 	omxMatrix *weights = obsStat.acovMat;
 	std::vector< omxThresholdColumn > &oThresh = obsStat.thresholdCols;
 
-	numOrdinal = oo->expectation->numOrdinal;
 	auto &eThresh = oo->expectation->getThresholdInfo();
 
-	// Error Checking: Observed/Expected means must agree.  
+	// Error Checking: Observed/Expected means must agree.
 	// ^ is XOR: true when one is false and the other is not.
 	if((newObj->expectedMeans == NULL) ^ (means == NULL)) {
 		if(newObj->expectedMeans != NULL) {
@@ -250,7 +245,7 @@ void omxWLSFitFunction::prepData()
 			return;
 		}
 	}
-	
+
 	if((eThresh.size()==0) ^ (obsThresholdsMat==0)) {
 		if (eThresh.size()) {
 			omxRaiseError("Observed thresholds not detected, but an expected thresholds matrix was specified.\n   If you wish to model the thresholds, you must provide observed thresholds.\n ");
@@ -273,13 +268,14 @@ void omxWLSFitFunction::prepData()
 		mxLog("observed thresholds:");
 		for (auto &th : oThresh) { th.log(); }
 	}
-	
+
+	int vectorSize = expectation->numSummaryStats();
+
 	if(weights) {
 		if (weights->rows != weights->cols || weights->cols != vectorSize) {
-			omxRaiseErrorf("Developer Error in WLS-based FitFunction object: WLS-based expectation specified an incorrectly-sized weight matrix (%d != %d).\nIf you are not developing a new expectation type, you should probably post this to the OpenMx forums.", vectorSize, weights->cols);
-			return;
+			mxThrow("Developer Error in WLS-based FitFunction object: WLS-based expectation specified an incorrectly-sized weight matrix (%d != %d).\nIf you are not developing a new expectation type, you should probably post this to the OpenMx forums.", vectorSize, weights->cols);
 		}
-	
+
 		EigenMatrixAdaptor Eweight(weights);
 		Eigen::MatrixXd offDiagW = Eweight.triangularView<Eigen::StrictlyUpper>();
 		double offDiag = offDiagW.array().abs().sum();
@@ -287,34 +283,32 @@ void omxWLSFitFunction::prepData()
 	} else {
 		oo->units = FIT_UNITS_SQUARED_RESIDUAL;
 	}
-	
-	if (obsThresholdsMat && oo->expectation->thresholdsMat) {
-		if (obsThresholdsMat->rows != oo->expectation->thresholdsMat->rows ||
-		    obsThresholdsMat->cols != oo->expectation->thresholdsMat->cols) {
-			mxThrow("Observed %dx%d and expected %dx%d threshold matrices must have the same shape",
-							obsThresholdsMat->rows, obsThresholdsMat->cols,
-							expectation->thresholdsMat->rows,
-							expectation->thresholdsMat->cols);
-		}
-	}
-	
+
 	observedFlattened = omxInitMatrix(vectorSize, 1, TRUE, matrix->currentState);
-	flattenDataToVector(cov, means, obsStat.slopeMat, obsThresholdsMat,
-			    oThresh, observedFlattened);
+	EigenVectorAdaptor oFlat(observedFlattened);
+	if (obsThresholdsMat) {
+		EigenMatrixAdaptor oTh(obsThresholdsMat);
+		normalToStdVector(cov, means, obsStat.slopeMat,
+                      [&oThresh, &oTh](int r,int c)->double{ return oTh(r, oThresh[c].column); },
+                      oThresh, oFlat);
+	} else {
+		normalToStdVector(cov, means, obsStat.slopeMat, [](int r,int c)->double{ return 0; },
+											oThresh, oFlat);
+	}
 	if(OMX_DEBUG) {omxPrintMatrix(observedFlattened, "....WLS Observed Vector: "); }
-	flattenDataToVector(newObj->expectedCov, newObj->expectedMeans,
-			    newObj->expectedSlope, oo->expectation->thresholdsMat,
-				eThresh, newObj->expectedFlattened);
+
+	/* Temporary storage for calculation */
+	expectedFlattened = omxInitMatrix(vectorSize, 1, TRUE, matrix->currentState);
+	P = omxInitMatrix(1, vectorSize, TRUE, matrix->currentState);
+	B = omxInitMatrix(vectorSize, 1, TRUE, matrix->currentState);
 }
 
 void omxWLSFitFunction::init()
 {
 	auto *oo = this;
-	
-	omxState *currentState = oo->matrix->currentState;
-	
+
 	if (!oo->expectation) { mxThrow("%s requires an expectation", name()); }
-	
+
 	if (R_has_slot(rObj, Rf_install("type"))) {
 		ProtectedSEXP RwlsType(R_do_slot(rObj, Rf_install("type")));
 		type = CHAR(STRING_ELT(RwlsType,0));
@@ -336,13 +330,26 @@ void omxWLSFitFunction::init()
 	expectedMeans = omxGetExpectationComponent(oo->expectation, "means");
 	expectedSlope = omxGetExpectationComponent(expectation, "slope");
 
-	/* Error check weight matrix size */
-	vectorSize = expectation->numSummaryStats();
-	if(OMX_DEBUG) { mxLog("Intial WLSFitFunction vectorSize comes to: %d.", vectorSize); }
-	
-	/* Temporary storage for calculation */
+  if (expectedSlope) {
+    expectation->invalidateCache();
+    expectation->connectToData();
+  }
+
+  if (false) {
+    // This is too strict because we might use def vars in a way that
+    // doesn't affect the model expectation (i.e. generate a column of
+    // the interaction between two other columns).
+    omxData *data = expectation->data;
+    if (data->defVars.size()) {
+      mxThrow("%s: There are %d definition variables. "
+              "The WLS fit function cannot handle definition variables except as exogenous predictors.",
+              name(), int(data->defVars.size()));
+    }
+  }
+
 	observedFlattened = 0;
-	expectedFlattened = omxInitMatrix(vectorSize, 1, TRUE, currentState);
-	P = omxInitMatrix(1, vectorSize, TRUE, currentState);
-	B = omxInitMatrix(vectorSize, 1, TRUE, currentState);
+	expectedFlattened = 0;
+	B = 0;
+	P = 0;
+  canDuplicate = true;
 }
