@@ -1,5 +1,5 @@
  /*
- *  Copyright 2016-2019 by the individuals mentioned in the source code history
+ *  Copyright 2016-2020 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ struct ssMLFitState : omxFitFunction {
 	omxMatrix *contRow;
 	omxMatrix *rowLikelihoods;
 	omxMatrix* otherRowwiseValues;
-	
+
 	virtual ~ssMLFitState();
 	virtual void init();
 	virtual void compute(int ffcompute, FitContext *fc);
@@ -37,7 +37,7 @@ struct ssMLFitState : omxFitFunction {
 void ssMLFitState::populateAttr(SEXP algebra)
 {
 	ssMLFitState *argStruct = this;
-	
+
 	if(argStruct->populateRowDiagnostics){
 		SEXP rowLikelihoodsExt;
 		SEXP rowObsExt;
@@ -61,33 +61,35 @@ void ssMLFitState::populateAttr(SEXP algebra)
 void ssMLFitState::compute(int want, FitContext *fc)
 {
 	if (want & (FF_COMPUTE_INITIAL_FIT | FF_COMPUTE_PREOPTIMIZE)) return;
-	
+
+  if (want & FF_COMPUTE_GRADIENT) invalidateGradient(fc);
+
 	auto *oo = this;
 	ssMLFitState *state = this;
 	auto dataColumns	= expectation->getDataColumns();
 	omxData *data = expectation->data;
 	int rowcount = data->nrows();
-	
+
 	Eigen::VectorXi contRemove(cov->cols);
-	
+
 	for (int row=0; row < rowcount; ++row) {
 		mxLogSetCurrentRow(row);
-		
+
 		omxDataRow(data, row, dataColumns, smallRow);
 		if (verbose >= 2) {
 			mxLog("row %d", row);
 			omxPrint(smallRow, "row");
 		}
-		
+
 		omxSetExpectationComponent(expectation, "y", smallRow);
-		
+
 		expectation->loadDefVars(row);
 		if(row == 0){
 			omxSetExpectationComponent(expectation, "Reset", NULL);
 		}
-		
+
 		omxExpectationCompute(fc, expectation, NULL);
-		
+
 		int numCont = 0;
 		for(int j = 0; j < dataColumns.size(); j++) {
 			if (omxDataElementMissing(data, row, dataColumns[j])) {
@@ -103,7 +105,7 @@ void ssMLFitState::compute(int want, FitContext *fc)
 			omxSetMatrixElement(otherRowwiseValues, row, 0, NA_REAL); // set first column to NA
 			continue;
 		}
-		
+
 		omxMatrix *smallMeans = omxGetExpectationComponent(expectation, "means");
 		omxRemoveElements(smallMeans, contRemove.data());
 		omxMatrix *smallCov = omxGetExpectationComponent(expectation, "inverse");
@@ -126,10 +128,10 @@ void ssMLFitState::compute(int want, FitContext *fc)
 			omxSetMatrixElement(oo->matrix, 0, 0, NA_REAL);
 			return;
 		}
-		
+
 		double determinant = *omxGetExpectationComponent(expectation, "determinant")->data;
 		if(OMX_DEBUG_ROWS(row)) { mxLog("0.5*log(det(Cov)) is: %3.3f", determinant);}
-		
+
 		omxCopyMatrix(contRow, smallRow);
 		omxRemoveElements(contRow, contRemove.data()); 	// Reduce the row to just continuous.
 		if (verbose >= 2) {
@@ -137,7 +139,7 @@ void ssMLFitState::compute(int want, FitContext *fc)
 			omxPrint(smallMeans, "smallMeans");
 		}
 		omxDAXPY(-1.0, smallMeans, contRow);
-		
+
 		/* Calculate Row Likelihood */
 		/* Mathematically: (2*pi)^cols * 1/sqrt(determinant(ExpectedCov)) * (dataRow %*% (solve(ExpectedCov)) %*% t(dataRow))^(1/2) */
 		double Q;
@@ -154,12 +156,12 @@ void ssMLFitState::compute(int want, FitContext *fc)
 			mxLog("Q=%f", Q);
 		}
 		omxSetMatrixElement(otherRowwiseValues, row, 0, Q);
-		
+
 		double rowLikelihood = pow(2 * M_PI, -.5 * numCont) * (1.0/exp(determinant)) * exp(-.5 * Q);
-		
+
 		omxSetMatrixElement(rowLikelihoods, row, 0, rowLikelihood);
 	}
-	
+
 	if (!state->returnRowLikelihoods) {
 		double sum = 0.0;
 		// floating-point addition is not associative,
@@ -168,7 +170,7 @@ void ssMLFitState::compute(int want, FitContext *fc)
 			double prob = omxVectorElement(state->rowLikelihoods, i);
 			//mxLog("[%d] %g", i, -2.0 * log(prob));
 			sum += log(prob);
-		}	
+		}
 		if(OMX_DEBUG) {mxLog("%s: total likelihood is %3.3f", oo->name(), sum);}
 		omxSetMatrixElement(oo->matrix, 0, 0, -2.0 * sum);
 	} else {
@@ -192,21 +194,21 @@ void ssMLFitState::init()
 {
 	auto *oo = this;
 	auto *state = this;
-	
+
 	oo->openmpUser = false;
 	oo->canDuplicate = true;
-	
+
 	ProtectedSEXP Rverbose(R_do_slot(rObj, Rf_install("verbose")));
 	verbose = Rf_asInteger(Rverbose);
-	
+
 	state->returnRowLikelihoods = Rf_asInteger(R_do_slot(oo->rObj, Rf_install("vector")));
 	units = returnRowLikelihoods? FIT_UNITS_PROBABILITY : FIT_UNITS_MINUS2LL;
-	
+
 	state->populateRowDiagnostics = Rf_asInteger(R_do_slot(oo->rObj, Rf_install("rowDiagnostics")));
-	
+
 	auto *data = expectation->data;
 	if (data->hasWeight() || data->hasFreq()) {
-		stop("%s: row frequencies or weights provided in '%s' are not supported",
+		mxThrow("%s: row frequencies or weights provided in '%s' are not supported",
 			 expectation->name, data->name);
 	}
 
@@ -215,7 +217,7 @@ void ssMLFitState::init()
 	state->rowLikelihoods = omxInitMatrix(rows, 1, currentState);
 	state->otherRowwiseValues = omxInitMatrix(rows, 2, currentState);
 	state->cov = omxGetExpectationComponent(expectation, "cov");
-	
+
 	int covCols = state->cov->cols;
 	state->smallRow = omxInitMatrix(1, covCols, TRUE, currentState);
 	state->contRow = omxInitMatrix(covCols, 1, TRUE, currentState);
